<?php

namespace Gift4U\Admin;

use Gift4U\Inc\Classes\Email;
use Gift4U\Inc\Classes\PDF;
use Gift4U\Inc\Utils;

defined( 'ABSPATH' ) || exit;

class Admin {
	protected static $instance = null;

	public function __construct() {
		add_action( 'admin_menu', [ $this, 'admin_menu_page' ] );
		add_filter( 'set_screen_option_gift4u_per_page', [ $this, 'set_screen_option' ], 10, 3 );
		add_action( 'add_meta_boxes', [ $this, 'gallery_meta_box' ] );
		add_action( 'save_post_gift4u_gallery', [ $this, 'save_gallery' ] );
		add_action( 'save_post_gift4u_tmpl', [ $this, 'save_card_template' ] );

		/*Product edit page*/
		add_filter( 'product_type_selector', [ $this, 'add_product_type' ] );
		add_filter( 'woocommerce_product_data_tabs', [ $this, 'product_data_tabs' ] );
		add_action( 'woocommerce_product_options_general_product_data', array( $this, 'add_giftcard_product_options' ) );
		add_action( 'woocommerce_process_product_meta_giftcard', array( $this, 'save_product_data' ) );
		add_action( 'wp_ajax_gift4u_search_gallery', array( $this, 'search_gallery' ) );
		add_action( 'wp_ajax_gift4u_search_card_template', array( $this, 'search_card_template' ) );
		add_action( 'woocommerce_admin_process_product_object', array( $this, 'admin_save_process_product_object' ) );

		/*Card template*/
		add_action( 'edit_form_after_title', [ $this, 'card_template_editor' ] );

		/*Orders page*/
		add_action( 'woocommerce_before_order_itemmeta', [ $this, 'order_item_meta' ], 10, 3 );

		/*Admin ajax*/
		add_action( 'wp_ajax_gift4u_ajax_action', [ $this, 'ajax' ] );
	}

	public static function instance() {
		return self::$instance == null ? self::$instance = new self() : self::$instance;
	}

	public function admin_menu_page() {

		$main_page = add_menu_page(
			esc_html__( 'Gift Card', 'gift4u-gift-cards-all-in-one-for-woo' ),
			esc_html__( 'Gift Card', 'gift4u-gift-cards-all-in-one-for-woo' ),
			apply_filters( 'gift4u_menu_page_capability', 'manage_woocommerce' ),
			'gift4u',
			[ $this, 'manage_giftcard' ],
			'dashicons-money',
			9
		);

		$manage_giftcard = add_submenu_page(
			'gift4u',
			esc_html__( 'Gift Card', 'gift4u-gift-cards-all-in-one-for-woo' ),
			esc_html__( 'Gift Card', 'gift4u-gift-cards-all-in-one-for-woo' ),
			apply_filters( 'gift4u_menu_page_capability', 'manage_woocommerce' ),
			'gift4u',
			[ $this, 'manage_giftcard' ],
			0
		);

		add_action( "load-$manage_giftcard", array( $this, 'add_screen_option' ) );

		$settings = add_submenu_page(
			'gift4u',
			esc_html__( 'Settings', 'gift4u-gift-cards-all-in-one-for-woo' ),
			esc_html__( 'Settings', 'gift4u-gift-cards-all-in-one-for-woo' ),
			apply_filters( 'gift4u_menu_page_capability', 'manage_woocommerce' ),
			'gift4u-settings',
			[ $this, 'settings_page' ],
			30
		);

		$setup_wizard = add_submenu_page(
			null,
			esc_html__( 'Setup Wizard', 'gift4u-gift-cards-all-in-one-for-woo' ),
			esc_html__( 'Setup Wizard', 'gift4u-gift-cards-all-in-one-for-woo' ),
			'manage_options',
			'gift4u-setup',
			'__return_false'
		);

		Utils::set_page( 'giftcard', $main_page );
		Utils::set_page( 'manage', $manage_giftcard );
		Utils::set_page( 'settings', $settings );
		Utils::set_page( 'setup_wizard', $setup_wizard );
	}

	public function manage_giftcard() {
		if ( ! empty( $_GET['action'] ) && ( $_GET['action'] == 'edit' || $_GET['action'] == 'add-new' ) ) {
			new Giftcard_Edit();

			return;
		}

		new Giftcard_Table();
	}

	public function settings_page() {
		$settings = Settings::instance();
		$settings->settings_page();
	}

	public function add_screen_option() {
		add_screen_option( 'per_page', array( 'default' => 30, 'option' => 'gift4u_per_page' ) );
	}

	public function set_screen_option( $status, $option, $value ) {
		if ( in_array( $option, [ 'gift4u_per_page' ], true ) ) {
			return $value;
		}

		return $status;
	}

	public function add_product_type( $types ) {
		$types['giftcard'] = __( 'Gift Card product', 'gift4u-gift-cards-all-in-one-for-woo' );

		return $types;
	}

	public function product_data_tabs( $tabs ) {
		$tabs['inventory']['class'][] = 'show_if_giftcard';

		return $tabs;
	}

	public function add_giftcard_product_options() {
		?>
		<div class='options_group show_if_giftcard'>
			<div class="gift4u-tabular-menu">
				<div class="gift4u-tab-item gift4u-active" data-tab="general">
					<?php esc_html_e( 'General', 'gift4u-gift-cards-all-in-one-for-woo' ); ?>
				</div>
				<div class="gift4u-tab-item" data-tab="coupon">
					<?php esc_html_e( 'Coupon', 'gift4u-gift-cards-all-in-one-for-woo' ); ?>
				</div>

			</div>
			<?php
			include_once plugin_dir_path( __FILE__ ) . 'views/html-product-general-options.php';
			include_once plugin_dir_path( __FILE__ ) . 'views/html-product-coupon-options.php';
			?>
		</div>
		<?php
	}

	private function pro_option_field( $label ) {
		?>
		<p class="form-field ">
			<label for="crosssell_ids"><?php echo esc_html( $label ) ?></label>
			<a href="#" target="_blank" class="button"><?php echo esc_html__( 'Unlock this feature', 'gift4u-gift-cards-all-in-one-for-woo' ) ?></a>
		</p>
		<?php
	}

	public function search_gallery() {
		check_ajax_referer( 'gift4u_ajax_nonce', 'security' );

		$result = [];
		if ( empty( $_GET['term'] ) ) {
			wp_send_json( $result );
		}

		$key_word = sanitize_text_field( wp_unslash( $_GET['term'] ) );

		$posts = get_posts( [
			'numberposts' => - 1,
			'post_type'   => 'gift4u_gallery',
			's'           => $key_word
		] );

		if ( ! empty( $posts ) ) {
			foreach ( $posts as $post ) {
				$result[ $post->ID ] = $post->post_title;
			}
		}

		wp_send_json( $result );
	}

	public function search_card_template() {
		check_ajax_referer( 'gift4u_ajax_nonce', 'security' );

		$result = [];
		if ( empty( $_GET['term'] ) ) {
			wp_send_json( $result );
		}

		$key_word = sanitize_text_field( wp_unslash( $_GET['term'] ) );

		$posts = get_posts( [
			'numberposts' => - 1,
			'post_type'   => 'gift4u_tmpl',
			's'           => $key_word
		] );

		if ( ! empty( $posts ) ) {
			foreach ( $posts as $post ) {
				$result[ $post->ID ] = $post->post_title;
			}
		}

		wp_send_json( $result );
	}


	/**
	 * @param $product \WC_Product
	 */
	public function admin_save_process_product_object( $product ) {
		if ( $product->get_type() != 'giftcard' ) {
			return;
		}

		if ( ! empty( $_POST['_gift4u_price_type'] ) && $_POST['_gift4u_price_type'] == 'fixed' ) {
			$regular_price = isset( $_POST['_gift4u_giftcard_price'] ) ? wc_clean( wp_unslash( $_POST['_gift4u_giftcard_price'] ) ) : null;
			$sale_price    = isset( $_POST['_gift4u_fixed_discount'] ) ? wc_clean( wp_unslash( $_POST['_gift4u_fixed_discount'] ) ) : null;
			$product->set_regular_price( $regular_price );
			$product->set_sale_price( $sale_price );
		}
	}

	public function save_product_data( $post_id ) {

		$keys = [
			'_gift4u_price_type',
			'_gift4u_giftcard_price',
			'_gift4u_fixed_discount',
			'_gift4u_galleries',
			'_gift4u_override_methods',
			'_gift4u_card_templates',
			'_gift4u_delivery_date',
			'_gift4u_sender_name',
			'_gift4u_recipient_name',
			'_gift4u_message_field',
			'_gift4u_shipping_form',
			'_gift4u_cp_allow_shipping',
			'_gift4u_cp_email_restrictions',
			'_gift4u_cp_expiry_date',
			'_gift4u_cp_minimum_spend',
			'_gift4u_cp_maximum_spend',
			'_gift4u_cp_individual',
			'_gift4u_cp_exclude_sale_items',
			'_gift4u_cp_products',
			'_gift4u_cp_excl_products',
			'_gift4u_cp_categories',
			'_gift4u_cp_excl_categories',
			'_gift4u_user_input_min_price',
			'_gift4u_user_input_max_price',
			'_gift4u_discount_rules',
			'_gift4u_message_character_limit',
		];

		foreach ( $keys as $key ) {
			if ( isset( $_POST[ $key ] ) ) {
				$value = wc_clean( wp_unslash( $_POST[ $key ] ) );

				if ( $key == '_gift4u_discount_rules' ) {
					$raw_rules = $value;
					if ( ! empty( $raw_rules ) && is_array( $raw_rules ) ) {
						foreach ( $raw_rules as $i => $rule ) {
							if ( empty( $rule['from_price'] ) && empty( $rule['to_price'] ) && empty( $rule['amount'] ) ) {
								unset( $raw_rules[ $i ] );
							}
						}
					}

					$value = array_values( $raw_rules );
				}

				update_post_meta( $post_id, $key, $value );
			} else {
				delete_post_meta( $post_id, $key );
			}
		}

		if ( isset( $_POST['_gift4u_tax_status'] ) ) {
			$value = sanitize_text_field( wp_unslash( $_POST['_gift4u_tax_status'] ) );
			update_post_meta( $post_id, '_tax_status', $value );
		}

		if ( isset( $_POST['_gift4u_tax_class'] ) ) {
			$value = sanitize_text_field( wp_unslash( $_POST['_gift4u_tax_class'] ) );
			update_post_meta( $post_id, '_tax_class', $value );
		}

	}

	public function gallery_meta_box() {
		add_meta_box( 'gift4u_gallery',
			esc_html__( 'Images', 'gift4u-gift-cards-all-in-one-for-woo' ),
			[ $this, 'gallery_meta_box_content' ],
			'gift4u_gallery',
			'normal' );
	}

	public function gallery_meta_box_content() {
		include_once plugin_dir_path( __FILE__ ) . 'views/html-gallery.php';
	}

	public function card_template_editor( $post ) {
		if ( $post->post_type !== 'gift4u_tmpl' ) {
			return;
		}
		include_once plugin_dir_path( __FILE__ ) . 'views/html-card-editor.php';
	}

	public function save_gallery( $post_id ) {
		if ( ! empty( $_POST['gift4u_gallery_images'] ) ) {
			$image_ids = array_map( 'absint', $_POST['gift4u_gallery_images'] );
			$image_ids = array_unique( $image_ids );
			update_post_meta( $post_id, '_gift4u_image_ids', $image_ids );
		} else {
			delete_post_meta( $post_id, '_gift4u_image_ids' );
		}
	}

	public function save_card_template( $post_id ) {

		if ( ! empty( $_POST['gift4u_card_template_data'] ) ) {
			$data = wp_unslash( $_POST['gift4u_card_template_data'] );  // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
			$data = json_decode( $data );
			$data = wc_clean( $data );

			update_post_meta( $post_id, '_gift4u_card_template', $data );
		} else {
			delete_post_meta( $post_id, '_gift4u_card_template' );
		}
	}

	public function ajax() {
		check_ajax_referer( 'gift4u_ajax_nonce', 'nonce' );

		$sub_action = ! empty( $_POST['sub_action'] ) ? 'ajax_' . sanitize_text_field( wp_unslash( $_POST['sub_action'] ) ) : '';

		if ( $sub_action && method_exists( $this, $sub_action ) ) {
			$this->$sub_action();
		}
		wp_die();
	}

	public function ajax_send_email() {
		if ( empty( $_POST['coupon_id'] ) ) {
			wp_send_json_error( esc_html__( 'Coupon is not exist', 'gift4u-gift-cards-all-in-one-for-woo' ) );
		}

		$coupon_id = absint( $_POST['coupon_id'] );
		$gc_coupon = Utils::get_giftcard_coupon( $coupon_id );

		if ( ! $gc_coupon ) {
			wp_send_json_error( esc_html__( 'Coupon is not exist', 'gift4u-gift-cards-all-in-one-for-woo' ) );
		}

		$email_template = new Email( $gc_coupon );
		$result         = $email_template->send_email();

		if ( ! $result ) {
			wp_send_json_error( esc_html__( 'Send failed', 'gift4u-gift-cards-all-in-one-for-woo' ) );
		}

		$delivery_statuses = Utils::delivery_status_list();

		wp_send_json_success( [
			'message'         => esc_html__( 'Send successfully', 'gift4u-gift-cards-all-in-one-for-woo' ),
			'delivery_status' => $delivery_statuses['sent'] ?? '',
		] );
	}

	public function ajax_mark_shipped() {
		if ( empty( $_POST['coupon_id'] ) ) {
			wp_send_json_error( esc_html__( 'Coupon is not exist', 'gift4u-gift-cards-all-in-one-for-woo' ) );
		}

		$coupon_id = absint( $_POST['coupon_id'] );
		$gc_coupon = Utils::get_giftcard_coupon( $coupon_id );

		if ( ! $gc_coupon ) {
			wp_send_json_error( esc_html__( 'Coupon is not exist', 'gift4u-gift-cards-all-in-one-for-woo' ) );
		}

		$expiry_date = $gc_coupon->get_expiry_date();
		if ( ! $expiry_date ) {
			$coupon_options = $gc_coupon->get_coupon_options();

			if ( ! empty( $coupon_options['expiry_date'] ) ) {
				$date = strtotime( "+{$coupon_options['expiry_date']} day" );

				$gc_coupon->set_expiry_date( $date );
			}
		}

		$gc_coupon->set_delivery_status( 'sent' );
		$gc_coupon->save();

		$delivery_statuses = Utils::delivery_status_list();

		$expiry_date = $gc_coupon->get_expiry_date();//get expiry date after save

		wp_send_json_success( [
			'message'         => esc_html__( 'Marked as shipped', 'gift4u-gift-cards-all-in-one-for-woo' ),
			'delivery_status' => $delivery_statuses['sent'] ?? '',
			'expiry_date'     => $expiry_date instanceof \WC_DateTime ? $expiry_date->date_i18n( wc_date_format() ) : '',
		] );
	}

	public function ajax_download_pdf() {

		if ( empty( $_POST['coupon_id'] ) ) {
			wp_send_json_error( esc_html__( 'Coupon is not exist', 'gift4u-gift-cards-all-in-one-for-woo' ), 201 );
		}

		$coupon_id = absint( $_POST['coupon_id'] );
		$gc_coupon = Utils::get_giftcard_coupon( $coupon_id );

		if ( ! $gc_coupon ) {
			wp_send_json_error( esc_html__( 'Coupon is not exist', 'gift4u-gift-cards-all-in-one-for-woo' ), 201 );
		}

		$expiry_date = $gc_coupon->get_expiry_date();
		if ( ! $expiry_date ) {
			$coupon_options = $gc_coupon->get_coupon_options();

			if ( ! empty( $coupon_options['expiry_date'] ) ) {
				$date = strtotime( "+{$coupon_options['expiry_date']} day" );

				$gc_coupon->set_expiry_date( $date );
			}
		}

		$gc_coupon->save();

		$pdf = new PDF( $gc_coupon );
		$pdf->get_download();

		wp_die();
	}

	/**
	 * @param $item_id
	 * @param $item \WC_Order_Item_Product
	 * @param $product
	 */
	public function order_item_meta( $item_id, $item, $product ) {
		$coupon_ids = (array) $item->get_meta( '_gift4u_coupon_ids' );

		if ( ! empty( $coupon_ids ) ) {
			?>
			<div class='gift4u-coupon-list'>
				<?php
				foreach ( $coupon_ids as $id ) {
					$gc_coupon = Utils::get_giftcard_coupon( $id );
					if ( ! $gc_coupon ) {
						continue;
					}

					$coupon_code     = $gc_coupon->get_coupon_code();
					$delivery_status = $gc_coupon->get_delivery_status();
					$status          = $gc_coupon->get_coupon_status();
					$status          = $status == 'trash' ? 'trash' : 'all';
					$url             = add_query_arg( [ 'page' => 'gift4u', 's' => $coupon_code, 'status' => $status ], admin_url( 'admin.php' ) );

					printf( "<a href='%s' target='_blank' class='gift4u-coupon-item gift4u-delivery-%s'>%s</a>",
						esc_url( $url ), esc_attr( $delivery_status ), esc_html( $coupon_code ) );

				}
				?>
			</div>
			<?php
		}

	}
}
