<?php

namespace Gift4U\Admin;

use Gift4U\Inc\Classes\Giftcard_Coupon;
use Gift4U\Inc\Utils;

defined( 'ABSPATH' ) || exit;

class Giftcard_Edit {
	protected $gc_coupon;
	protected $action;

	public function __construct() {
		$this->save_coupon();

		if ( ! empty( $_GET['id'] ) ) {
			$id              = absint( $_GET['id'] );
			$this->gc_coupon = Utils::get_giftcard_coupon( $id );

			if ( ! $this->gc_coupon ) {
				wp_die( esc_html__( 'You attempted to edit an item that does not exist. Perhaps it was deleted?' ) );

				return;
			}
			$this->action = 'edit';
		} else {
			$this->gc_coupon = new Giftcard_Coupon();
			$this->action    = 'add-new';
		}

		$this->display();
	}

	public function display() {
		$delivery_methods   = Utils::delivery_methods_list();
		$delivery_statuses  = Utils::delivery_status_list();
		$shipping_countries = WC()->countries->get_shipping_countries();

		$expiry_date = $this->gc_coupon->get_expiry_date();
		if ( $expiry_date ) {
			$expiry_date = date_i18n( 'Y-m-d', $expiry_date->getTimestamp() );
		}

		$delivery_date = $this->gc_coupon->get_delivery_date();
		if ( $delivery_date ) {
			$delivery_date = date_i18n( 'Y-m-d', $delivery_date->getTimestamp() );
		}

		$order_id = $this->gc_coupon->get_order_id();
		$order    = wc_get_order( $order_id );

		if ( is_object( $order ) ) {
			$shipping_address_1 = $order->get_shipping_address_1();
			$shipping_address_2 = $order->get_shipping_address_2();
			$shipping_city      = $order->get_shipping_city();
			$shipping_country   = $order->get_shipping_country();
			$shipping_state     = $order->get_shipping_state();
			$shipping_postcode  = $order->get_shipping_postcode();
			$shipping_phone     = $order->get_shipping_phone();
		} else {
			$shipping_data      = (array) $this->gc_coupon->get_single_meta_data( 'shipping_address' );
			$shipping_address_1 = $shipping_data['address_1'] ?? '';
			$shipping_address_2 = $shipping_data['address_2'] ?? '';
			$shipping_city      = $shipping_data['city'] ?? '';
			$shipping_country   = $shipping_data['country'] ?? '';
			$shipping_state     = $shipping_data['state'] ?? '';
			$shipping_postcode  = $shipping_data['postcode'] ?? '';
			$shipping_phone     = $shipping_data['phone'] ?? '';
		}

		$categories = get_terms( [ 'taxonomy' => 'product_cat', 'orderby' => 'name', 'hide_empty' => false ] );

		$coupon_options = $this->gc_coupon->get_coupon_options();

		?>
        <div class="wrap">
            <h1 class="wp-heading-inline">
				<?php echo $this->action == 'edit' ? esc_html__( 'Edit giftcard', 'gift4u-gift-cards-all-in-one-for-woo' )
					: esc_html__( 'Add new giftcard', 'gift4u-gift-cards-all-in-one-for-woo' ) ?>
            </h1>
            <hr class="wp-header-end">

            <form class="vi-ui form small" method="post">

                <div class="vi-ui top attached tabular menu">
                    <div class="active item" data-tab="general"><?php esc_html_e( 'General', 'gift4u-gift-cards-all-in-one-for-woo' ) ?></div>
                    <div class="item" data-tab="coupon"><?php esc_html_e( 'Coupon options', 'gift4u-gift-cards-all-in-one-for-woo' ) ?></div>
                </div>

                <div class="vi-ui bottom attached active tab segment small" data-tab="general">
                    <div class="two fields">
                        <div class="field">
                            <label><?php esc_html_e( 'Coupon code', 'gift4u-gift-cards-all-in-one-for-woo' ) ?></label>
                            <input type="text" name="gift4u_coupon_code" value="<?php echo esc_attr( $this->gc_coupon->get_coupon_code() ) ?>"/>
                        </div>

                        <div class="field">
                            <label><?php esc_html_e( 'Amount', 'gift4u-gift-cards-all-in-one-for-woo' ) ?></label>
                            <input type="number" name="gift4u_coupon_amount" value="<?php echo esc_attr( $this->gc_coupon->get_amount() ) ?>" required/>
                        </div>
                    </div>

                    <div class="field">
                        <label><?php esc_html_e( 'Delivery method', 'gift4u-gift-cards-all-in-one-for-woo' ) ?></label>
                        <select name="gift4u_delivery_method" class="vi-ui dropdown fluid selection gift4u-delivery-method">
							<?php
							$selected_method = $this->gc_coupon->get_delivery_method();
							foreach ( $delivery_methods as $method_key => $method_display ) {
								printf( '<option value="%s" %s>%s</option>', esc_attr( $method_key ), selected( $selected_method, $method_key ), esc_html( $method_display ) );
							}
							?>
                        </select>
                    </div>

                    <div class="two fields">
                        <div class="field">
                            <label><?php esc_html_e( 'From email', 'gift4u-gift-cards-all-in-one-for-woo' ) ?></label>
                            <input type="email" name="gift4u_from_email" value="<?php echo esc_attr( $this->gc_coupon->get_from_email() ) ?>"/>
                        </div>

                        <div class="field">
                            <label><?php esc_html_e( 'To email', 'gift4u-gift-cards-all-in-one-for-woo' ) ?></label>
                            <input type="email" name="gift4u_to_email" value="<?php echo esc_attr( $this->gc_coupon->get_to_email() ) ?>"/>
                        </div>
                    </div>

                    <div class="two fields">
                        <div class="field">
                            <label><?php esc_html_e( 'Sender name', 'gift4u-gift-cards-all-in-one-for-woo' ) ?></label>
                            <input type="text" name="gift4u_sender_name" value="<?php echo esc_attr( $this->gc_coupon->get_single_meta_data( 'sender_name' ) ) ?>"/>
                        </div>

                        <div class="field">
                            <label><?php esc_html_e( 'Recipient name', 'gift4u-gift-cards-all-in-one-for-woo' ) ?></label>
                            <input type="text" name="gift4u_recipient_name" value="<?php echo esc_attr( $this->gc_coupon->get_single_meta_data( 'recipient_name' ) ) ?>"/>
                        </div>
                    </div>

                    <div class="field">
                        <label><?php esc_html_e( 'Message', 'gift4u-gift-cards-all-in-one-for-woo' ) ?></label>
                        <textarea rows="4" name="gift4u_message"><?php echo esc_html( $this->gc_coupon->get_single_meta_data( 'message' ) ) ?></textarea>
                    </div>

                    <div class="field">
                        <label><?php esc_html_e( 'Email image', 'gift4u-gift-cards-all-in-one-for-woo' ) ?></label>
                        <div class="gift4u-email-image-group">
							<?php
							$img_id  = $this->gc_coupon->get_single_meta_data( 'email_design' );
							$img_url = wp_get_attachment_url( $img_id );
							$img_url = $img_url ? $img_url : wc_placeholder_img_src();
							?>
                            <div class="vi-ui buttons mini">
                                <button type="button" class="vi-ui button teal gift4u-select-image">
									<?php esc_html_e( 'Select image', 'gift4u-gift-cards-all-in-one-for-woo' ); ?>
                                </button>
                                <button type="button" class="vi-ui button gift4u-remove-image">
									<?php esc_html_e( 'Remove image', 'gift4u-gift-cards-all-in-one-for-woo' ); ?>
                                </button>
                            </div>

                            <img src="<?php echo esc_url( $img_url ) ?>" class="gift4u-email-design"
                                 data-img_placeholder="<?php echo esc_url( wc_placeholder_img_src() ) ?>">

                            <input type="hidden" name="gift4u_email_design" class="gift4u-email-design-id" value="<?php echo esc_attr( $img_id ) ?>"/>
                        </div>
                    </div>

                    <div class="field">
                        <label><?php esc_html_e( 'Card template', 'gift4u-gift-cards-all-in-one-for-woo' ) ?></label>
                        <select name="gift4u_card_template" class="gift4u-card-template"
                                placeholder="<?php echo esc_attr__( 'Search card template', 'gift4u-gift-cards-all-in-one-for-woo' ) ?>">
							<?php
							$card_tmpl_id = $this->gc_coupon->get_single_meta_data( 'card_template' );
							$card_tmpl    = get_post( $card_tmpl_id );
							if ( $card_tmpl ) {
								printf( '<option value="%s" selected>%s</option>', esc_attr( $card_tmpl->ID ), esc_html( $card_tmpl->post_title ) );
							}
							?>
                        </select>
                    </div>

                    <div class="field">
                        <label><?php esc_html_e( 'Expiry date', 'gift4u-gift-cards-all-in-one-for-woo' ) ?></label>
                        <input type="date" name="gift4u_expiry_date" value="<?php echo esc_attr( $expiry_date ) ?>"/>
                    </div>

                    <div class="field">
                        <label><?php esc_html_e( 'Delivery date', 'gift4u-gift-cards-all-in-one-for-woo' ) ?></label>
                        <input type="date" name="gift4u_delivery_date" value="<?php echo esc_attr( $delivery_date ) ?>"/>
                    </div>

                    <div class="field">
                        <label><?php esc_html_e( 'Delivery status', 'gift4u-gift-cards-all-in-one-for-woo' ) ?></label>
                        <select name="gift4u_delivery_status" class="vi-ui dropdown fluid selection">
							<?php
							$delivery_status = $this->gc_coupon->get_delivery_status();
							foreach ( $delivery_statuses as $stt_key => $stt_display ) {
								printf( '<option value="%s" %s>%s</option>', esc_attr( $stt_key ), selected( $delivery_status, $stt_key ), esc_html( $stt_display ) );
							}
							?>
                        </select>
                    </div>

                    <div class="gift4u-edit-shipping-address-section">
                        <h3><?php esc_html_e( 'Shipping address', 'gift4u-gift-cards-all-in-one-for-woo' ) ?></h3>
                        <hr>

                        <div class="two fields">
                            <div class="field">
                                <label><?php esc_html_e( 'Address line 1', 'gift4u-gift-cards-all-in-one-for-woo' ) ?></label>
                                <input type="text" name="gift4u_shipping_address_1" value="<?php echo esc_attr( $shipping_address_1 ) ?>"/>
                            </div>

                            <div class="field">
                                <label><?php esc_html_e( 'Address line 2', 'gift4u-gift-cards-all-in-one-for-woo' ) ?></label>
                                <input type="number" name="gift4u_shipping_address_2" value="<?php echo esc_attr( $shipping_address_2 ) ?>"/>
                            </div>
                        </div>

                        <div class="two fields">
                            <div class="field">
                                <label><?php esc_html_e( 'City', 'gift4u-gift-cards-all-in-one-for-woo' ) ?></label>
                                <input type="text" name="gift4u_shipping_city" value="<?php echo esc_attr( $shipping_city ) ?>"/>
                            </div>

                            <div class="field">
                                <label><?php esc_html_e( 'Postcode / ZIP', 'gift4u-gift-cards-all-in-one-for-woo' ) ?></label>
                                <input type="number" name="gift4u_shipping_postcode" value="<?php echo esc_attr( $shipping_postcode ) ?>"/>
                            </div>
                        </div>

                        <div class="two fields">
                            <div class="field">
                                <label><?php esc_html_e( 'Country / Region', 'gift4u-gift-cards-all-in-one-for-woo' ) ?></label>
                                <select name="gift4u_shipping_country" class="vi-ui dropdown search  gift4u-country-field">
									<?php
									foreach ( $shipping_countries as $code => $country ) {
										printf( '<option value="%s" %s>%s</option>', esc_attr( $code ), selected( $code, $shipping_country ), esc_html( $country ) );
									}
									?>
                                </select>
                            </div>

                            <div class="field">
                                <label><?php esc_html_e( 'State / County', 'gift4u-gift-cards-all-in-one-for-woo' ) ?></label>
                                <input type="text" name="gift4u_shipping_state" value="<?php echo esc_attr( $shipping_state ) ?>" class="gift4u-state-field"/>
                            </div>
                        </div>

                        <div class="field">
                            <label><?php esc_html_e( 'Phone', 'gift4u-gift-cards-all-in-one-for-woo' ) ?></label>
                            <input type="tel" name="gift4u_shipping_phone" value="<?php echo esc_attr( $shipping_phone ) ?>"/>
                        </div>

                    </div>
                </div>

                <div class="vi-ui bottom attached tab segment small" data-tab="coupon">
                    <div class="inline field">
                        <label><?php esc_html_e( 'Allow shipping', 'gift4u-gift-cards-all-in-one-for-woo' ) ?></label>
                        <div class="vi-ui toggle checkbox">
                            <input type="checkbox" name="_gift4u_cp[allow_shipping]" value="yes" <?php checked( $coupon_options['allow_shipping'] ?? '', 'yes' ) ?> />
                            <label> </label>
                        </div>
                    </div>

                    <div class="inline field">
                        <label><?php esc_html_e( 'Individual use only', 'gift4u-gift-cards-all-in-one-for-woo' ) ?></label>
                        <div class="vi-ui toggle checkbox">
                            <input type="checkbox" name="_gift4u_cp[individual]" value="yes" <?php checked( $coupon_options['individual'] ?? '', 'yes' ) ?> />
                            <label> </label>
                        </div>
                    </div>

                    <div class="inline field">
                        <label><?php esc_html_e( 'Exclude sale items', 'gift4u-gift-cards-all-in-one-for-woo' ) ?></label>
                        <div class="vi-ui toggle checkbox">
                            <input type="checkbox" name="_gift4u_cp[exclude_sale_items]" value="yes" <?php checked( $coupon_options['exclude_sale_items'] ?? '', 'yes' ) ?> />
                            <label> </label>
                        </div>
                    </div>

                    <div class="two fields">
                        <div class="field">
                            <label><?php esc_html_e( 'Minimum spend', 'gift4u-gift-cards-all-in-one-for-woo' ) ?></label>
                            <input type="number" min="0" name="_gift4u_cp[minimum_spend]" value="<?php echo esc_attr( $coupon_options['minimum_spend'] ?? '' ) ?>"/>
                        </div>

                        <div class="field">
                            <label><?php esc_html_e( 'Maximum spend', 'gift4u-gift-cards-all-in-one-for-woo' ) ?></label>
                            <input type="number" min="0" name="_gift4u_cp[maximum_spend]" value="<?php echo esc_attr( $coupon_options['maximum_spend'] ?? '' ) ?>"/>
                        </div>
                    </div>

                    <div class="field">
                        <label><?php esc_html_e( 'Products', 'gift4u-gift-cards-all-in-one-for-woo' ) ?></label>
                        <select name="_gift4u_cp[products][]" multiple class="gift4u-products-field"
                                data-placeholder="<?php esc_attr_e( 'Search for a product&hellip;', 'woocommerce' ); ?>"
                                data-action="woocommerce_json_search_products_and_variations">
							<?php
							if ( ! empty( $coupon_options['products'] ) && is_array( $coupon_options['products'] ) ) {
								foreach ( $coupon_options['products'] as $product_id ) {
									$product = wc_get_product( $product_id );
									if ( ! is_object( $product ) ) {
										continue;
									}
									printf( '<option value="%s" %s>%s</option>', esc_attr( $product_id ), selected( true, true, false ), esc_html( wp_strip_all_tags( $product->get_formatted_name() ) ) );
								}
							}
							?>
                        </select>
                    </div>

                    <div class="field">
                        <label><?php esc_html_e( 'Exclude products', 'gift4u-gift-cards-all-in-one-for-woo' ) ?></label>
                        <select name="_gift4u_cp[excl_products][]" multiple class="gift4u-products-field"
                                data-placeholder="<?php esc_attr_e( 'Search for a product&hellip;', 'woocommerce' ); ?>"
                                data-action="woocommerce_json_search_products_and_variations">
							<?php
							if ( ! empty( $coupon_options['excl_products'] ) && is_array( $coupon_options['excl_products'] ) ) {
								foreach ( $coupon_options['excl_products'] as $product_id ) {
									$product = wc_get_product( $product_id );
									if ( ! is_object( $product ) ) {
										continue;
									}
									printf( '<option value="%s" %s>%s</option>', esc_attr( $product_id ), selected( true, true, false ), esc_html( wp_strip_all_tags( $product->get_formatted_name() ) ) );
								}
							}
							?>
                        </select>
                    </div>

                    <div class="field">
                        <label><?php esc_html_e( 'Categories', 'gift4u-gift-cards-all-in-one-for-woo' ) ?></label>
                        <select name="_gift4u_cp[categories][]" multiple class="gift4u-categories-field">
							<?php
							$in_cats = $coupon_options['categories'] ?? [];
							if ( $categories ) {
								foreach ( $categories as $cat ) {
									printf( '<option value="%s" %s>%s</option>', esc_attr( $cat->term_id ), wc_selected( $cat->term_id, $in_cats ), esc_html( $cat->name ) );// phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
								}
							}
							?>
                        </select>
                    </div>

                    <div class="field">
                        <label><?php esc_html_e( 'Exclude categories', 'gift4u-gift-cards-all-in-one-for-woo' ) ?></label>
                        <select name="_gift4u_cp[excl_categories][]" multiple class="gift4u-categories-field">
							<?php
							$ex_cats = $coupon_options['excl_categories'] ?? [];
							if ( $categories ) {
								foreach ( $categories as $cat ) {
									printf( '<option value="%s" %s>%s</option>', esc_attr( $cat->term_id ), wc_selected( $cat->term_id, $ex_cats ), esc_html( $cat->name ) ); // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
								}
							}
							?>
                        </select>
                    </div>

                </div>

                <input type="hidden" name="gift4u_coupon_id" value="<?php echo esc_attr( $this->gc_coupon->get_id() ) ?>"/>

                <button type="submit" class="vi-ui button primary" name="gift4u_save_coupon" value="yes">
					<?php esc_html_e( 'Save', 'gift4u-gift-cards-all-in-one-for-woo' ); ?>
                </button>

				<?php wp_nonce_field( 'gift4u_edit_coupon' ); ?>
            </form>
        </div>
		<?php
	}

	public function save_coupon() {
		if ( empty( $_POST['gift4u_save_coupon'] ) || $_POST['gift4u_save_coupon'] !== 'yes' || ! current_user_can( 'manage_options' ) ) {
			return;
		}

		if ( empty( $_POST['_wpnonce'] ) || ! wp_verify_nonce( sanitize_text_field( wp_unslash( $_POST['_wpnonce'] ) ), 'gift4u_edit_coupon' ) ) {
			return;
		}

		$coupon_id = ! empty( $_POST['gift4u_coupon_id'] ) ? absint( $_POST['gift4u_coupon_id'] ) : '';
		$gc_coupon = $coupon_id ? Utils::get_giftcard_coupon( $coupon_id ) : new Giftcard_Coupon();

		$key_array = [
			'coupon_code',
			'coupon_amount',
			'delivery_method',
			'from_email',
			'to_email',
			'expiry_date',
			'delivery_date',
			'delivery_status',
		];

		foreach ( $key_array as $key ) {
			if ( isset( $_POST[ 'gift4u_' . $key ] ) ) {
				$value = sanitize_text_field( wp_unslash( $_POST[ 'gift4u_' . $key ] ) );
				if ( method_exists( $gc_coupon, 'set_' . $key ) ) {
					$method = 'set_' . $key;
					$gc_coupon->$method( $value );
				}
			}
		}

		$meta_data_array = [ 'sender_name', 'recipient_name', 'card_template', 'email_design' ];
		foreach ( $meta_data_array as $key ) {
			if ( isset( $_POST[ 'gift4u_' . $key ] ) ) {
				$value = sanitize_text_field( wp_unslash( $_POST[ 'gift4u_' . $key ] ) );
				$gc_coupon->set_single_meta_data( $key, $value );
			}
		}

		if ( isset( $_POST['gift4u_message'] ) ) {
			$value = sanitize_textarea_field( wp_unslash( $_POST['gift4u_message'] ) );
			$gc_coupon->set_single_meta_data( 'message', $value );
		}

		$shipping_array_keys = [
			'address_1',
			'address_2',
			'city',
			'state',
			'postcode',
			'country',
			'phone',
		];

		$shipping_data = [];
		foreach ( $shipping_array_keys as $key ) {
			if ( isset( $_POST[ 'gift4u_shipping_' . $key ] ) ) {
				$shipping_data[ $key ] = sanitize_text_field( wp_unslash( $_POST[ 'gift4u_shipping_' . $key ] ) );
			}
		}

		if ( ! empty( $shipping_data ) ) {
			$order_id = $gc_coupon->get_order_id();
			$order    = wc_get_order( $order_id );

			if ( is_object( $order ) ) {
				$order->set_shipping_address( $shipping_data );
				$order->save();
			} else {
				$gc_coupon->set_single_meta_data( 'shipping_address', $shipping_data );
			}
		}

		if ( ! empty( $_POST['_gift4u_cp'] ) ) {
			$new_coupon_options = wc_clean( wp_unslash( $_POST['_gift4u_cp'] ) );
			$gc_coupon->set_coupon_options( $new_coupon_options );
		}

		$return_id = $gc_coupon->save();

		if ( $coupon_id !== $return_id ) {
			wp_safe_redirect( admin_url( "admin.php?page=gift4u&action=edit&id={$return_id}" ) );
			exit;
		}
	}
}

