<?php

namespace Gift4U\Admin;

use Gift4U\GiftCard;
use Gift4U\Inc\Classes\Giftcard_Coupon;
use Gift4U\Inc\Data;
use Gift4U\Inc\Utils;

defined( 'ABSPATH' ) || exit;

if ( ! class_exists( '\WP_List_Table' ) ) {
	require_once ABSPATH . 'wp-admin/includes/class-wp-list-table.php';
}

class Giftcard_Table extends \WP_List_Table {
	protected $total = 0;
	protected $is_trash;
	protected $filter = [
		'delivery_status'    => null,
		'delivery_method'    => null,
		'delivery_date_from' => null,
		'delivery_date_to'   => null,
		'search'             => null,
	];

	public function __construct() {
		parent::__construct(
			[
				'singular' => 'giftcard',
				'plural'   => 'giftcards',
				'screen'   => 'giftcard',
			]
		);

		$this->is_trash = isset( $_REQUEST['status'] ) && 'trash' === $_REQUEST['status'];

		$doaction = $this->current_action();

		if ( $doaction ) {
			check_admin_referer( 'bulk-giftcards' );
			$sendback = remove_query_arg( array( 'trashed', 'untrashed', 'deleted', 'locked', 'ids' ), wp_get_referer() );

			$coupon_ids = ! empty( $_REQUEST['gift4u_coupon_ids'] ) ? array_map( 'absint', (array) $_REQUEST['gift4u_coupon_ids'] ) : [];

			if ( empty( $coupon_ids ) && $doaction !== 'delete_all' ) {
				wp_safe_redirect( $sendback );
				exit;
			}

			global $wpdb;
			$db_table = GiftCard::get( 'db_table' );
			$in_ids   = implode( ',', $coupon_ids );

			switch ( $doaction ) {
				case 'trash':
					$query      = "UPDATE {$db_table} SET coupon_status='trash' where id IN ({$in_ids})";
					$result_qty = $wpdb->query( $query );
					$sendback   = add_query_arg( [ 'trashed' => $result_qty ], $sendback );
					break;

				case 'untrash':
					$query      = "UPDATE {$db_table} SET coupon_status='publish' where id IN ({$in_ids})";
					$result_qty = $wpdb->query( $query );
					$sendback   = add_query_arg( [ 'untrashed' => $result_qty ], $sendback );
					break;

				case 'delete':
					$query      = "DELETE FROM {$db_table} where id IN ({$in_ids})";
					$result_qty = $wpdb->query( $query );
					$sendback   = add_query_arg( 'deleted', $result_qty, $sendback );
					break;

				case 'delete_all':

					$query      = "DELETE FROM {$db_table} where coupon_status='trash'";
					$result_qty = $wpdb->query( $query );
					$sendback   = add_query_arg( 'deleted', $result_qty, $sendback );
					break;

				case 'edit':
					if ( isset( $_REQUEST['bulk_edit'] ) ) {
						$done = bulk_edit_posts( $_REQUEST );

						if ( is_array( $done ) ) {
							$done['updated'] = count( $done['updated'] );
							$done['skipped'] = count( $done['skipped'] );
							$done['locked']  = count( $done['locked'] );
							$sendback        = add_query_arg( $done, $sendback );
						}
					}
					break;

				default:
					$screen   = get_current_screen()->id;
					$sendback = apply_filters( "gift4u_handle_bulk_actions-{$screen}", $sendback, $doaction, $coupon_ids ); // phpcs:ignore WordPress.NamingConventions.ValidHookName.UseUnderscores
					break;
			}

			$sendback = remove_query_arg( array( 'action', 'action2', 'bulk_edit' ), $sendback );

			wp_safe_redirect( $sendback );
			exit;

		} elseif ( ! empty( $_REQUEST['_wp_http_referer'] ) && isset( $_SERVER['REQUEST_URI'] ) ) {
			$referer = esc_url_raw( wp_unslash( $_SERVER['REQUEST_URI'] ) );
			wp_safe_redirect( remove_query_arg( array( '_wp_http_referer', '_wpnonce' ), $referer ) );
			exit;
		}

		$this->filter['delivery_status']    = ! empty( $_GET['delivery_status'] ) ? sanitize_text_field( wp_unslash( $_GET['delivery_status'] ) ) : null;
		$this->filter['delivery_method']    = ! empty( $_GET['delivery_method'] ) ? sanitize_text_field( wp_unslash( $_GET['delivery_method'] ) ) : null;
		$this->filter['delivery_date_from'] = ! empty( $_GET['delivery_date_from'] ) ? sanitize_text_field( wp_unslash( $_GET['delivery_date_from'] ) ) : null;
		$this->filter['delivery_date_to']   = ! empty( $_GET['delivery_date_to'] ) ? sanitize_text_field( wp_unslash( $_GET['delivery_date_to'] ) ) : null;
		$this->filter['search']             = ! empty( $_GET['s'] ) ? sanitize_text_field( wp_unslash( $_GET['s'] ) ) : null;

		$this->prepare_items();
		$this->display();
	}

	public function prepare_items() {
		$items_per_page = $this->get_items_per_page( 'gift4u_per_page' );
		$this->items    = $this->get_entries();
		$this->set_pagination_args( [ 'total_items' => $this->total, 'per_page' => $items_per_page, ] );
	}

	public function get_entries() {
		global $wpdb;
		$db_table = GiftCard::get( 'db_table' );

		$items_per_page = $this->get_items_per_page( 'gift4u_per_page', 30 );
		$paged          = $this->get_pagenum() - 1;
		$offset         = $paged * $items_per_page;
		$where[]        = $this->is_trash ? "coupon_status='trash'" : "coupon_status!='trash'";

		if ( $this->filter['search'] !== null ) {
			$where[] = "coupon_code LIKE '%%{$this->filter['search']}%%' OR from_email LIKE '%%{$this->filter['search']}%%' OR to_email LIKE '%%{$this->filter['search']}%%' ";
		}

		if ( $this->filter['delivery_status'] !== null ) {
			$where[] = $this->filter['delivery_status'] == 'sent' ? "delivery_status='sent'" : "delivery_status='pending' OR delivery_status=''";
		}

		if ( $this->filter['delivery_method'] !== null ) {
			$where[] = "delivery_method='{$this->filter['delivery_method']}'";
		}

		if ( $this->filter['delivery_date_from'] !== null ) {
			$from_date = strtotime( $this->filter['delivery_date_from'] );
			if ( $from_date ) {
				$where[] = "delivery_date >= {$from_date}";
			}
		}

		if ( $this->filter['delivery_date_to'] !== null ) {
			$to_date = strtotime( $this->filter['delivery_date_to'] ) + DAY_IN_SECONDS - 1;
			if ( $to_date ) {
				$where[] = "delivery_date <= {$to_date}";
			}
		}

		$where = array_filter( $where );
		$where = implode( ' AND ', $where );

		$query       = "select SQL_CALC_FOUND_ROWS * from {$db_table} where {$where} order by id desc limit %d, %d";
		$query       = $wpdb->prepare( $query, $offset, $items_per_page );
		$results     = $wpdb->get_results( $query, ARRAY_A );
		$this->total = $wpdb->get_var( "SELECT FOUND_ROWS()" );

		if ( ! empty( $results ) ) {
			foreach ( $results as $key => $props ) {
				$item = new Giftcard_Coupon();
				$item->set_id( $props['id'] );
				$item->set_props( $props );
				$results[ $key ] = $item;
			}
		}

		return $results;
	}

	public function get_columns() {
		$columns = [
			'cb'              => '<input type="checkbox" />',
			'coupon_code'     => esc_html__( 'Coupon code', 'gift4u-gift-cards-all-in-one-for-woo' ),
			'from'            => esc_html__( 'From', 'gift4u-gift-cards-all-in-one-for-woo' ),
			'to'              => esc_html__( 'To', 'gift4u-gift-cards-all-in-one-for-woo' ),
			'amount'          => esc_html__( 'Amount', 'gift4u-gift-cards-all-in-one-for-woo' ),
			'usage'           => esc_html__( 'Usage', 'gift4u-gift-cards-all-in-one-for-woo' ),
			'expiry_date'     => esc_html__( 'Expired date', 'gift4u-gift-cards-all-in-one-for-woo' ),
			'delivery_date'   => esc_html__( 'Delivery date', 'gift4u-gift-cards-all-in-one-for-woo' ),
			'delivery_status' => esc_html__( 'Delivery status', 'gift4u-gift-cards-all-in-one-for-woo' ),
			'delivery_method' => esc_html__( 'Method', 'gift4u-gift-cards-all-in-one-for-woo' ),
			'actions'         => esc_html__( 'Actions', 'gift4u-gift-cards-all-in-one-for-woo' ),
		];

		return $columns;
	}

	public function action_messages() {
		$bulk_counts = array(
			'updated'   => isset( $_REQUEST['updated'] ) ? absint( $_REQUEST['updated'] ) : 0,
			'locked'    => isset( $_REQUEST['locked'] ) ? absint( $_REQUEST['locked'] ) : 0,
			'deleted'   => isset( $_REQUEST['deleted'] ) ? absint( $_REQUEST['deleted'] ) : 0,
			'trashed'   => isset( $_REQUEST['trashed'] ) ? absint( $_REQUEST['trashed'] ) : 0,
			'untrashed' => isset( $_REQUEST['untrashed'] ) ? absint( $_REQUEST['untrashed'] ) : 0,
		);


		$bulk_messages = [
			/* translators: %s: Number of pages. */
			'updated'   => _n( '%s coupon updated.', '%s coupons updated.', $bulk_counts['updated'] ),

			/* translators: %s: Number of pages. */
			'deleted'   => _n( '%s coupon permanently deleted.', '%s coupons permanently deleted.', $bulk_counts['deleted'] ),
			/* translators: %s: Number of pages. */
			'trashed'   => _n( '%s coupon moved to the Trash.', '%s coupons moved to the Trash.', $bulk_counts['trashed'] ),
			/* translators: %s: Number of pages. */
			'untrashed' => _n( '%s coupon restored from the Trash.', '%s coupons restored from the Trash.', $bulk_counts['untrashed'] ),
		];

		$bulk_counts = array_filter( $bulk_counts );
		$messages    = [];

		foreach ( $bulk_counts as $message => $count ) {
			if ( isset( $bulk_messages[ $message ] ) ) {
				$messages[] = sprintf( $bulk_messages[ $message ], number_format_i18n( $count ) );
			}

			if ( 'untrashed' === $message && ! empty( $_REQUEST['ids'] ) ) {
				$ids = wc_clean( wp_unslash( $_REQUEST['ids'] ) );
				$ids = explode( ',', $ids );

				if ( 1 === count( $ids ) && current_user_can( 'edit_post', $ids[0] ) ) {
					$messages[] = sprintf( '<a href="%1$s">%2$s</a>',
						esc_url( get_edit_post_link( $ids[0] ) ),
						esc_html( get_post_type_object( get_post_type( $ids[0] ) )->labels->edit_item )
					);
				}
			}
		}

		if ( $messages ) {
			echo '<div id="message" class="updated notice is-dismissible"><p>' . esc_html( implode( ' ', $messages ) ) . '</p></div>';
		}
		unset( $messages );
	}

	public function display() {
		$singular = $this->_args['singular'];
		?>
        <div class="wrap">

            <h1 class="wp-heading-inline">
				<?php esc_html_e( 'Giftcard manage', 'gift4u-gift-cards-all-in-one-for-woo' ) ?>
                <a href="<?php echo esc_url( admin_url( 'admin.php?page=gift4u&action=add-new' ) ) ?>" class="button">
					<?php esc_html_e( 'Add new', 'gift4u-gift-cards-all-in-one-for-woo' ) ?>
                </a>
            </h1>
            <hr class="wp-header-end">

			<?php $this->action_messages(); ?>
            <form id="posts-filter" method="get">
                <input type="hidden" name="page" value="gift4u">

				<?php
				$this->search_box( esc_html__( 'Search', 'gift4u-gift-cards-all-in-one-for-woo' ), 'post' );
				$this->views();
				$this->display_tablenav( 'top' );
				$this->screen->render_screen_reader_content( 'heading_list' );
				?>
                <input type="hidden" name="status" class="status_page"
                       value="<?php echo ! empty( $_REQUEST['status'] ) ? esc_attr( sanitize_text_field( wp_unslash( $_REQUEST['status'] ) ) ) : 'all'; ?>"/>

                <table class="vi-ui table wp-list-table widefat">
                    <thead>
                    <tr>
						<?php $this->print_column_headers(); ?>
                    </tr>
                    </thead>

                    <tbody id="the-list" <?php echo ! empty( $singular ) ? sprintf( " data-wp-lists='list:%s'", esc_attr( $singular ) ) : ''; ?> >
					<?php

					$this->display_rows_or_placeholder(); ?>
                    </tbody>

                    <tfoot>
                    <tr>
						<?php $this->print_column_headers( false ); ?>
                    </tr>
                    </tfoot>
                </table>

				<?php $this->display_tablenav( 'bottom' ); ?>
            </form>
        </div>
		<?php
	}

	public function extra_tablenav( $which ) {
		if ( $which === 'top' ) {
			$delivery_methods  = [ '' => esc_html__( 'Delivery method', 'gift4u-gift-cards-all-in-one-for-woo' ) ] + Utils::delivery_methods_list();
			$delivery_statuses = [ '' => esc_html__( 'Delivery status', 'gift4u-gift-cards-all-in-one-for-woo' ) ] + Utils::delivery_status_list();
			?>
            <div class="alignleft">
                <select name="delivery_status" id="">
					<?php
					foreach ( $delivery_statuses as $value => $text ) {
						printf( "<option value='%s' %s>%s</option>", esc_attr( $value ), selected( $this->filter['delivery_status'], $value, false ), esc_html( $text ) );
					}
					?>
                </select>

                <select name="delivery_method" id="">
					<?php
					foreach ( $delivery_methods as $value => $text ) {
						printf( "<option value='%s' %s>%s</option>", esc_attr( $value ), selected( $this->filter['delivery_method'], $value, false ), esc_html( $text ) );
					}
					?>
                </select>
                <span>
                    <?php esc_html_e( 'Delivery date:', 'gift4u-gift-cards-all-in-one-for-woo' ); ?>
                    <input type="date" name="delivery_date_from" value="<?php echo esc_attr( $this->filter['delivery_date_from'] ) ?>"/>
                    -
                    <input type="date" name="delivery_date_to" value="<?php echo esc_attr( $this->filter['delivery_date_to'] ) ?>"/>
                </span>
				<?php
				submit_button( esc_html__( 'Filter', 'gift4u-gift-cards-all-in-one-for-woo' ), '', 'filter_action', false, array( 'id' => 'post-query-submit' ) );

				if ( $this->is_trash ) {
					?>
                    <button type="submit" name="action" value="delete_all" class="button"><?php esc_html_e( 'Delete all' ); ?></button>
					<?php
				}
				?>
            </div>
			<?php
		}
	}

	public function get_bulk_actions() {
		return $this->is_trash ?
			[
				'untrash' => esc_html__( 'Restore', 'gift4u-gift-cards-all-in-one-for-woo' ),
				'delete'  => esc_html__( 'Delete', 'gift4u-gift-cards-all-in-one-for-woo' ),
			] :
			[ 'trash' => esc_html__( 'Move to trash', 'gift4u-gift-cards-all-in-one-for-woo' ) ];
	}

	public function get_views() {
		global $wpdb;
		$db_table = GiftCard::get( 'db_table' );

		$query   = "SELECT coupon_status, COUNT( * ) AS num_posts FROM {$db_table}  GROUP BY coupon_status";
		$results = (array) $wpdb->get_results( $query, ARRAY_A );
		$counts  = [];

		foreach ( $results as $row ) {
			$key = ! empty( $row['coupon_status'] ) ? $row['coupon_status'] : 'publish';

			if ( ! isset( $counts[ $key ] ) ) {
				$counts[ $key ] = 0;
			}

			$counts[ $key ] += $row['num_posts'];
		}

		$status_args = [ 'page' => 'gift4u' ];

		$status_links['all'] = array(
			'url'     => esc_url( add_query_arg( $status_args, 'admin.php' ) ),
			'label'   => sprintf( "%s (%d)", esc_html__( 'All', 'gift4u-gift-cards-all-in-one-for-woo' ), $counts['publish'] ?? 0 ),
			'current' => empty( $_REQUEST['status'] ) || ( isset( $_REQUEST['status'] ) && $_REQUEST['status'] == 'all' ),
		);

		$status_args['status'] = 'trash';
		$status_links['trash'] = array(
			'url'     => esc_url( add_query_arg( $status_args, 'admin.php' ) ),
			'label'   => sprintf( "%s (%d)", esc_html__( 'Trash', 'gift4u-gift-cards-all-in-one-for-woo' ), $counts['trash'] ?? 0 ),
			'current' => isset( $_REQUEST['status'] ) && $_REQUEST['status'] == 'trash',
		);

		return $this->get_views_links( $status_links );
	}

	public function print_column_headers( $with_id = true ) {
		list( $columns, $hidden, $sortable, $primary ) = $this->get_column_info();
		$host        = isset( $_SERVER['HTTP_HOST'] ) ? sanitize_text_field( wp_unslash( $_SERVER['HTTP_HOST'] ) ) : '';
		$uri         = isset( $_SERVER['REQUEST_URI'] ) ? sanitize_text_field( wp_unslash( $_SERVER['REQUEST_URI'] ) ) : '';
		$current_url = set_url_scheme( 'http://' . $host . $uri );
		$current_url = remove_query_arg( 'paged', $current_url );

		if ( isset( $_GET['orderby'] ) ) {
			$current_orderby = sanitize_text_field( wp_unslash( $_GET['orderby'] ) );
		} else {
			$current_orderby = '';
		}

		if ( isset( $_GET['order'] ) && 'desc' === $_GET['order'] ) {
			$current_order = 'desc';
		} else {
			$current_order = 'asc';
		}

		if ( ! empty( $columns['cb'] ) ) {
			static $cb_counter = 1;
			$columns['cb'] = '<label class="screen-reader-text" for="cb-select-all-' . $cb_counter . '">' . __( 'Select All' ) . '</label>'
			                 . '<input id="cb-select-all-' . $cb_counter . '" type="checkbox" />';
			$cb_counter ++;
		}

		foreach ( $columns as $column_key => $column_display_name ) {
			$class = array( 'manage-column', "column-$column_key" );

			if ( in_array( $column_key, $hidden, true ) ) {
				$class[] = 'hidden';
			}

			if ( 'cb' === $column_key ) {
				$class[] = 'check-column';
			} elseif ( in_array( $column_key, array( 'posts', 'comments', 'links' ), true ) ) {
				$class[] = 'num';
			}

			if ( $column_key === $primary ) {
				$class[] = 'column-primary';
			}

			if ( isset( $sortable[ $column_key ] ) ) {
				list( $orderby, $desc_first ) = $sortable[ $column_key ];

				if ( $current_orderby === $orderby ) {
					$order = 'asc' === $current_order ? 'desc' : 'asc';

					$class[] = 'sorted';
					$class[] = $current_order;
				} else {
					$order = strtolower( $desc_first );

					if ( ! in_array( $order, array( 'desc', 'asc' ), true ) ) {
						$order = $desc_first ? 'desc' : 'asc';
					}

					$class[] = 'sortable';
					$class[] = 'desc' === $order ? 'asc' : 'desc';
				}

				$column_display_name = sprintf( '<a href="%s"><span>%s</span><span class="sorting-indicator"></span></a>',
					esc_url( add_query_arg( compact( 'orderby', 'order' ), $current_url ) ), $column_display_name );
			}

			$id = $with_id ? "$column_key" : '';

			if ( ! empty( $class ) ) {
				$class = implode( ' ', $class );
			}

			printf( '<th scope="col" id="%s" class="%s">%s</th>', esc_attr( $id ), esc_attr( $class ), $column_display_name ); // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
		}
	}

	protected function single_row_columns( $item ) {
		list( $columns, $hidden, $sortable, $primary ) = $this->get_column_info();

		foreach ( $columns as $column_name => $column_display_name ) {
			$classes = "$column_name column-$column_name";
			if ( $primary === $column_name ) {
				$classes .= ' has-row-actions column-primary';
			}

			if ( in_array( $column_name, $hidden, true ) ) {
				$classes .= ' hidden';
			}

			// Comments column uses HTML in the display name with screen reader text.
			// Strip tags to get closer to a user-friendly string.
			$data = 'data-colname="' . esc_attr( wp_strip_all_tags( $column_display_name ) ) . '"';

			$attributes = "class='$classes' $data";

			if ( 'cb' === $column_name ) {
				echo '<td scope="row" class="check-column">';
				$this->column_cb( $item );
				echo '</th>';
			} elseif ( method_exists( $this, '_column_' . $column_name ) ) {
				echo call_user_func( [ $this, '_column_' . $column_name ], $item, $classes, $data, $primary ); // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
			} elseif ( method_exists( $this, 'column_' . $column_name ) ) {
				echo "<td $attributes>"; // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
				echo call_user_func( array( $this, 'column_' . $column_name ), $item ); // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
				echo $this->handle_row_actions( $item, $column_name, $primary ); // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
				echo '</td>';
			} else {
				echo "<td $attributes>"; // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
				echo wp_kses_post( $this->column_default( $item, $column_name ) );
				echo $this->handle_row_actions( $item, $column_name, $primary ); // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
				echo '</td>';
			}
		}
	}

	/**
	 * @param array|object $item Giftcard_Coupon
	 */
	public function column_cb( $item ) {
		printf( '<label class="screen-reader-text" for="gift4u-coupon-%1$s">%1$s</label>
                        <input type="checkbox" name="gift4u_coupon_ids[]" id="gift4u-coupon-%1$s"  value="%1$s" />',
			esc_attr( $item->get_id() ) );
	}

	/**
	 * @param  $item Giftcard_Coupon
	 * @param string $column
	 *
	 * @return string|void|null
	 */
	public function column_default( $item, $column ) {
		$delivery_method               = $item->get_delivery_method();

		switch ( $column ) {
			case 'coupon_code':
				$id = $item->get_id();

				return sprintf( "<a class='row-title' href='%s'>%s</a>", admin_url( "admin.php?page=gift4u&action=edit&id={$id}" ), $item->get_coupon_code() );

			case 'delivery_status':
				$send_list = Utils::delivery_status_list();

				return ! empty( $send_list[ $item->get_delivery_status() ] ) ? $send_list[ $item->get_delivery_status() ] : '';

			case 'amount':
				return wc_price( $item->get_amount(), [ 'currency' => $item->get_currency() ] );

			case 'usage':
				return $item->get_usage_count() ? esc_html__( 'Used', 'gift4u-gift-cards-all-in-one-for-woo' ) : '';

			case 'expiry_date':
				$exdate              = $item->get_expiry_date();
				$coupon_options      = $item->get_coupon_options();
				$expiry_date_option  = $coupon_options['expiry_date'] ?? '';
				$wait_generate_title = esc_html__( 'The expiration date will be generated when the gift card is sent', 'gift4u-woocommerce-gift-cards-all-in-one' );

				return $exdate instanceof \WC_DateTime ? $exdate->date_i18n( wc_date_format() )
					: ( $expiry_date_option ? "<i class='dashicons dashicons-clock' title='$wait_generate_title'> </i>" : '-' );

			case 'from':
				return $item->get_from_email();

			case 'to':
				$to = '';
				if ( $delivery_method == 'email' ) {
					$to = $item->get_to_email();
				} elseif ( $delivery_method == 'ship_card' ) {
					$order_id = $item->get_order_id();
					$order    = wc_get_order( $order_id );
					if ( $order ) {
						$to = $order->get_formatted_shipping_address();
					}
				}

				return $to;

			case 'delivery_method':
				$methods = Utils::delivery_methods_list();

				return $methods[ $delivery_method ] ?? '';

			case 'delivery_date':
				$date = $item->get_delivery_date();

				return $date instanceof \WC_DateTime ? $date->date_i18n( wc_date_format() ) : '';

			case 'actions':
				$delivery_status = $item->get_delivery_status();
				$add_card_pdf_to_email = Data::get_param( 'add_card_pdf_to_email' );
				$card_tmpl_id          = $item->get_single_meta_data( 'card_template' );

				ob_start();
				?>
                <div class="gift4u-admin-actions">
					<?php

					if ( $card_tmpl_id && ( $delivery_method == 'ship_card' || ( $delivery_method == 'email' && $add_card_pdf_to_email ) ) ) {
						?>
                        <div class="gift4u-download-pdf" data-coupon_id="<?php echo esc_attr( $item->get_id() ) ?>"
                             data-coupon_code="<?php echo esc_attr( $item->get_coupon_code() ) ?>"
                             title="<?php esc_html_e( 'Download PDF file', 'gift4u-gift-cards-all-in-one-for-woo' ); ?>">
                            <i class="file pdf outline icon"> </i>
                        </div>
						<?php
					}

					if ( $delivery_method == 'email' ) {
						?>
                        <div class="gift4u-send-email" data-coupon_id="<?php echo esc_attr( $item->get_id() ) ?>"
                             title="<?php esc_html_e( 'Send email', 'gift4u-gift-cards-all-in-one-for-woo' ); ?>">
                            <i class="dashicons dashicons-email"> </i>
                            <i class="dashicons dashicons-arrow-right-alt"> </i>
                        </div>
						<?php
					} elseif ( $delivery_method == 'ship_card' && $delivery_status != 'sent' ) {
						?>
                        <div class="gift4u-mark-shipped" data-coupon_id="<?php echo esc_attr( $item->get_id() ) ?>"
                             title="<?php esc_html_e( 'Mark as shipped', 'gift4u-gift-cards-all-in-one-for-woo' ); ?>">
                            <i class="truck icon"> </i>
                        </div>
						<?php
					}
					?>
                </div>
				<?php

				return ob_get_clean();

			default:
				do_action( 'gift4u_giftcard_row', $item, $column );

				return null;
		}

	}

}