<?php

namespace Gift4U\Admin;

use Gift4U\GiftCard;
use Gift4U\Inc\Data;
use Gift4U\Inc\Utils;

defined( 'ABSPATH' ) || exit;

class Settings {
	protected static $instance = null;

	protected $dropdown_class = 'vi-ui dropdown fluid gift4u-dropdown';

	public static function instance() {
		return self::$instance == null ? self::$instance = new self() : self::$instance;
	}

	public function __construct() {
		Settings_Helper::config( [ 'slug' => 'gift4u', 'data' => Data::class ] );
		$this->save_settings();
	}

	public function settings_page() {
		$tabs      = $this->define_tabs();
		$first_tab = array_key_first( $tabs );
		?>
        <h1>
			<?php printf( "%s %s", esc_html( GiftCard::get( 'plugin_name' ) ), esc_html__( 'Settings', 'giftcard' ) ); ?>
        </h1>
        <form method="post" id="gift4u-settings-form" class="vi-ui small form">
			<?php wp_nonce_field( 'gift4u-settings' ); ?>

            <div class="vi-ui top attached tabular menu">
				<?php
				foreach ( $tabs as $slug => $text ) {
					$active = $first_tab == $slug ? 'active' : '';
					printf( ' <a class="item %s" data-tab="%s">%s</a>', esc_attr( $active ), esc_attr( $slug ), esc_html( $text ) );
				}
				?>
            </div>
			<?php
			foreach ( $tabs as $slug => $text ) {
				$active = $first_tab == $slug ? 'active' : '';
				$method = $slug . '_options';

				printf( '<div class="vi-ui bottom attached %s tab segment" data-tab="%s">', esc_attr( $active ), esc_attr( $slug ) );

				if ( method_exists( $this, $method ) ) {
					$options = $this->$method();
					Settings_Helper::output_fields( $options );
				} else {
					do_action( 'gift4u_settings_tab', $slug );
				}

				echo '</div>';
			}
			?>
            <p class="gift4u-save-settings-container">
                <button type="submit" class="vi-ui button labeled icon primary gift4u-save-settings" name="gift4u_save_settings" value="save_setting">
                    <i class="save icon"> </i>
					<?php esc_html_e( 'Save Settings', 'giftcard' ); ?>
                </button>
            </p>
        </form>
		<?php
		do_action( 'villatheme_support_gift4u-gift-cards-all-in-one-for-woo' );
	}

	public function define_tabs() {
		return [
			'general' => esc_html__( 'General', 'gift4u-gift-cards-all-in-one-for-woo' ),
			'email'   => esc_html__( 'Email', 'gift4u-gift-cards-all-in-one-for-woo' ),
		];
	}

	public function general_options() {
		$options = [
			[ 'type' => 'section_start' ],
			[
				'id'      => 'discount_umpire',
				'title'   => esc_html__( 'Discount priority', 'gift4u-gift-cards-all-in-one-for-woo' ),
				'desc'    => esc_html__( 'The customer will receive discount with highest/lowest value. Work with user input price', 'gift4u-gift-cards-all-in-one-for-woo' ),
				'type'    => 'select',
				'options' => [
					'max' => esc_html__( 'Highest', 'gift4u-gift-cards-all-in-one-for-woo' ),
					'min' => esc_html__( 'Lowest', 'gift4u-gift-cards-all-in-one-for-woo' ),
				],
				'class'   => $this->dropdown_class,
			],
			[
				'id'      => 'order_statuses',
				'title'   => esc_html__( 'Order status', 'gift4u-gift-cards-all-in-one-for-woo' ),
				'desc'    => esc_html__( 'Coupon will be generated when order has these statuses', 'gift4u-gift-cards-all-in-one-for-woo' ),
				'type'    => 'multiselect',
				'options' => [
					'on-hold'    => esc_html__( 'On hold', 'woocommerce' ),
					'processing' => esc_html__( 'Processing', 'woocommerce' ),
					'completed'  => esc_html__( 'Completed', 'woocommerce' ),
				],
				'class'   => $this->dropdown_class,
			],
			[
				'id'      => 'delivery_methods',
				'title'   => esc_html__( 'Delivery methods', 'gift4u-gift-cards-all-in-one-for-woo' ),
				'desc'    => esc_html__( 'Those methods may be override in product edit page', 'giftcard' ),
				'type'    => 'multiselect',
				'options' => [ '' => esc_html__( 'All methods', 'gift4u-gift-cards-all-in-one-for-woo' ) ] + Utils::delivery_methods_list(),
				'class'   => $this->dropdown_class,
			],
			[
				'id'    => 'myaccount_endpoint',
				'title' => esc_html__( 'My account end point', 'gift4u-gift-cards-all-in-one-for-woo' ),
				'type'  => 'text',
			],
			[
				'id'    => 'add_card_pdf_to_email',
				'title' => esc_html__( 'Add card PDF to email', 'gift4u-gift-cards-all-in-one-for-woo' ),
				'type'  => 'checkbox',
			],
			[
				'id'          => 'pdf_file_name',
				'title'       => esc_html__( 'PDF file name in email', 'gift4u-gift-cards-all-in-one-for-woo' ),
				'type'        => 'text',
				'placeholder' => esc_html__( 'If empty, the file name will be automatically generated.', 'gift4u-gift-cards-all-in-one-for-woo' ),
				'desc'        => esc_html__( 'Availabel shortcode: {coupon_code}', 'giftcard' ),
			],
			[
				'id'    => 'show_coupon_list',
				'title' => esc_html__( 'Show coupon list', 'gift4u-gift-cards-all-in-one-for-woo' ),
				'desc'  => esc_html__( 'Show availabel coupons below apply coupon form with user logged in', 'gift4u-gift-cards-all-in-one-for-woo' ),
				'type'  => 'checkbox',
			],
			[
				'id'    => 'coupon_sticker_bg_color',
				'title' => esc_html__( 'Coupon background color', 'gift4u-gift-cards-all-in-one-for-woo' ),
				'type'  => 'color',
			],
			[
				'id'    => 'coupon_sticker_text_color',
				'title' => esc_html__( 'Coupon text color', 'gift4u-gift-cards-all-in-one-for-woo' ),
				'type'  => 'color',
			],

			[ 'type' => 'section_end' ],
		];

		return $options;
	}

	public function email_options() {
		return [
			[ 'type' => 'section_start' ],

			[
				'id'    => 'email_subject',
				'title' => esc_html__( 'Email subject', 'giftcard' ),
				'type'  => 'text',
				'desc'  => sprintf( "<b>%s:</b> {site_title}, {sender_name}, {recipient_name}, {coupon_code}, {coupon_value}",
					esc_html__( 'Available placeholder strings', 'gift4u-gift-cards-all-in-one-for-woo' ) )
			],

			[
				'id'       => 'email_content',
				'title'    => esc_html__( 'Email content', 'giftcard' ),
				'type'     => 'texteditor',
				'settings' => [ 'editor_height' => 500 ],
				'desc'     => [
					sprintf( "<b>%s:</b>", esc_html__( 'Available placeholder strings', 'gift4u-gift-cards-all-in-one-for-woo' ) ),
					"{site_title}",
					"{site_url}",
					"{coupon_code}",
					"{coupon_value}",
					"{selected_image}",
					"{sender_name}",
					"{recipient_name}",
					"{message}",
					"{apply_coupon_url}",
					'{coupon_expiry_date}',
				]
			],
			[
				'id'    => 'bcc_to_sender',
				'title' => esc_html__( 'Bcc to sender', 'gift4u-gift-cards-all-in-one-for-woo' ),
				'desc'  => esc_html__( 'Bcc to sender when giftcard send to recipient', 'gift4u-gift-cards-all-in-one-for-woo' ),
				'type'  => 'checkbox',
			],

			[ 'type' => 'section_end' ],
		];
	}

	public function save_settings() {
		if ( isset( $_POST['gift4u_save_settings'], $_POST['_wpnonce'] )
		     && $_POST['gift4u_save_settings'] === 'save_setting' && current_user_can( 'manage_options' )
		     && wp_verify_nonce( sanitize_text_field( wp_unslash( $_POST['_wpnonce'] ) ), 'gift4u-settings' ) ) {

			try {
				$tabs    = $this->define_tabs();
				$options = [];
				foreach ( $tabs as $slug => $text ) {
					$method = $slug . '_options';
					if ( method_exists( $this, $method ) ) {
						$options = array_merge( $options, $this->$method() );
					} else {
						$options = array_merge( $options, apply_filters( 'gift4u_save_setting_option', [], $slug ) );
					}
				}

				$options = apply_filters( 'gift4u_settings_before_save', $options );

				add_filter( 'gift4u_admin_settings_sanitize_option_names', [ $this, 'sanitize_textarea_to_array' ], 10, 3 );
				add_filter( 'gift4u_admin_settings_sanitize_option_cmt_frontend', [ $this, 'sanitize_textarea_to_array' ], 10, 3 );
				add_filter( 'gift4u_admin_settings_sanitize_option_reply_content', [ $this, 'sanitize_reply_content' ], 10, 3 );
				add_filter( 'gift4u_admin_settings_sanitize_option_review_rules', [ $this, 'sanitize_review_rules' ], 10, 3 );

				if ( 'save_n_check_key' == $_POST['gift4u_save_settings'] ) {
					delete_site_transient( 'update_plugins' );
					delete_transient( 'villatheme_item_21055' );
					delete_option( 'giftcard_messages' );
					$auto_update_key = ! empty( $_POST['gift4u_auto_update_key'] ) ? sanitize_text_field( wp_unslash( $_POST['gift4u_auto_update_key'] ) ) : '';
					do_action( 'villatheme_save_and_check_key_giftcard', $auto_update_key );
				}

				$old_endpoint = Data::get_param( 'myaccount_endpoint' );
				$new_endpoint = isset( $_POST['gift4u_myaccount_endpoint'] ) ? sanitize_text_field( wp_unslash( $_POST['gift4u_myaccount_endpoint'] ) ) : '';

				Settings_Helper::save_fields( $options );

				Data::init_params();//push new values

				if ( $new_endpoint !== $old_endpoint ) {
					add_rewrite_endpoint( $new_endpoint, EP_ROOT | EP_PAGES );
					flush_rewrite_rules();
				}

			} catch ( \Exception $e ) {
				echo esc_html( $e->getMessage() );
			}
		}
	}

	public function sanitize_textarea_to_array( $value, $option, $raw_value ) {
		if ( empty( $raw_value ) || ! is_array( $raw_value ) ) {
			return [];
		}

		foreach ( $raw_value as $key => $names ) {
			$names         = explode( "\n", wp_kses_post( $names ) );
			$value[ $key ] = array_map( 'trim', $names );
		}

		return $value;
	}

	public function sanitize_review_rules( $value, $option, $raw_value ) {
		if ( empty( $raw_value ) || ! is_array( $raw_value ) ) {
			return [];
		}

		$ids = [];

		foreach ( $raw_value as $rule_key => $rule ) {
			if ( empty( $rule['comments'] ) || ! is_array( $rule['comments'] ) ) {
				continue;
			}

			$cmt_array = [];

			foreach ( $rule['comments'] as $lang => $comments ) {
				if ( empty( $comments ) || ! is_array( $comments ) ) {
					continue;
				}

				foreach ( $comments as $rate => $comment ) {
					if ( empty( $comment ) ) {
						continue;
					}
					$cmt_list = explode( "\n", wp_kses_post( $comment ) );
//					$cmt_list = array_values($cmt_list);
					$cmt_array[ $lang ][ $rate ] = $cmt_list;
				}
			}

			if ( empty( $cmt_array ) ) {
				unset( $value[ $rule_key ] );
			} else {
				$rule             = wc_clean( $rule );
				$rule['comments'] = $cmt_array;
				$ids[]            = $rule['rule_id'];
				update_option( "gift4u_cmt_rule_{$rule['rule_id']}", $rule );
			}
		}

		return $ids;
	}

	public function sanitize_reply_content( $value, $option, $raw_value ) {
		if ( empty( $raw_value ) || ! is_array( $raw_value ) ) {
			return [];
		}

		foreach ( $raw_value as $lang => $content ) {
			if ( empty( $content ) ) {
				continue;
			}

			foreach ( $content as $rate => $replies ) {
				$value[ $lang ][ $rate ] = explode( "\n", wp_kses_post( $replies ) );
			}
		}

		return $value;
	}
}
