<?php

namespace Gift4U\Admin\Views;

use Gift4U\Inc\Utils;

defined( 'ABSPATH' ) || exit;

/** @var \WC_Product $product_object */
global $product_object;

$sale_price_dates_from_timestamp = $product_object->get_date_on_sale_from( 'edit' ) ? $product_object->get_date_on_sale_from( 'edit' )->getOffsetTimestamp() : false;
$sale_price_dates_to_timestamp   = $product_object->get_date_on_sale_to( 'edit' ) ? $product_object->get_date_on_sale_to( 'edit' )->getOffsetTimestamp() : false;

$sale_price_dates_from = $sale_price_dates_from_timestamp ? date_i18n( 'Y-m-d', $sale_price_dates_from_timestamp ) : '';
$sale_price_dates_to   = $sale_price_dates_to_timestamp ? date_i18n( 'Y-m-d', $sale_price_dates_to_timestamp ) : '';

?>
<div class="gift4u-tab-content gift4u-active" data-tab="general">
	<?php
	woocommerce_wp_select( [
		'id'                => '_gift4u_price_type',
		'class'             => 'select short',
		'label'             => esc_html__( 'Pricing type', 'gift4u-gift-cards-all-in-one-for-woo' ),
		'options'           => [
			'fixed'   => esc_html__( 'Fixed', 'gift4u-gift-cards-all-in-one-for-woo' ),
			'input'   => esc_html__( 'User input', 'gift4u-gift-cards-all-in-one-for-woo' ),
			'slider'  => esc_html__( 'Slider (Pro)', 'gift4u-gift-cards-all-in-one-for-woo' ),
			'options' => esc_html__( 'Options (Pro)', 'gift4u-gift-cards-all-in-one-for-woo' ),
			'product' => esc_html__( 'Product (Pro)', 'gift4u-gift-cards-all-in-one-for-woo' ),
		],
		'style'             => '',
		'wrapper_class'     => '',
		'value'             => $product_object->get_meta( '_gift4u_price_type', true ),
		'desc_tip'          => false,
		'custom_attributes' => array(),
	] );

	woocommerce_wp_text_input( [
		'id'          => '_gift4u_giftcard_price',
		'label'       => esc_html__( 'Price (Gift card value)', 'gift4u-gift-cards-all-in-one-for-woo' ),
		'value'       => $product_object->get_regular_price( 'edit' ),
		'default'     => '',
		'class'       => '',
		'placeholder' => '',
		'data_type'   => 'price',
	] );

	woocommerce_wp_text_input( [
		'id'                => '_gift4u_fixed_discount',
		'label'             => esc_html__( 'Sale price', 'gift4u-gift-cards-all-in-one-for-woo' ),
		'value'             => $product_object->get_sale_price( 'edit' ),
		'default'           => '',
		'class'             => '',
		'placeholder'       => '',
		'data_type'         => 'price',
		'type'              => 'number',
		'custom_attributes' => [ 'min' => 0 ]
	] );

	woocommerce_wp_text_input( [
		'id'                => '_gift4u_user_input_min_price',
		'label'             => esc_html__( 'Min price', 'gift4u-gift-cards-all-in-one-for-woo' ),
		'value'             => $product_object->get_meta( '_gift4u_user_input_min_price', true ),
		'default'           => '',
		'class'             => '',
		'placeholder'       => '',
		'data_type'         => 'price',
		'type'              => 'number',
		'custom_attributes' => [ 'min' => 0 ]
	] );

	woocommerce_wp_text_input( [
		'id'                => '_gift4u_user_input_max_price',
		'label'             => esc_html__( 'Max price', 'gift4u-gift-cards-all-in-one-for-woo' ),
		'value'             => $product_object->get_meta( '_gift4u_user_input_max_price', true ),
		'default'           => '',
		'class'             => '',
		'placeholder'       => '',
		'data_type'         => 'price',
		'type'              => 'number',
		'custom_attributes' => [ 'min' => 0 ]
	] );


	?>

    <div class="form-field _gift4u_dynamic_discount_field">
        <label for=""><?php esc_html_e( 'Discount rules', 'gift4u-gift-cards-all-in-one-for-woo' ); ?></label>
        <table class="gift4u-discount-rules">
            <thead>
            <tr>
                <th><?php esc_html_e( 'From price', 'gift4u-gift-cards-all-in-one-for-woo' ); ?></th>
                <th><?php esc_html_e( 'To price', 'gift4u-gift-cards-all-in-one-for-woo' ); ?></th>
                <th><?php esc_html_e( 'Discount amount', 'gift4u-gift-cards-all-in-one-for-woo' ); ?></th>
                <th><?php esc_html_e( 'Remove', 'gift4u-gift-cards-all-in-one-for-woo' ); ?></th>
            </tr>
            </thead>
            <tbody>

            </tbody>

        </table>
        <span class="gift4u-add-rule-row dashicons dashicons-plus-alt2"> </span>
    </div>
    <p class="form-field sale_price_dates_fields">
        <label for="_gift4u_sale_date_from"><?php esc_html_e( 'Discount date from', 'woocommerce' ) ?></label>

        <input type="text" class="short" name="_gift4u_sale_date_from" id="_gift4u_sale_date_from" value="<?php echo esc_attr( $sale_price_dates_from ) ?>"
               placeholder="<?php echo esc_html( _x( 'From&hellip;', 'placeholder', 'woocommerce' ) ) ?> YYYY-MM-DD" maxlength="10"
               pattern="<?php echo esc_attr( apply_filters( 'woocommerce_date_input_html_pattern', '[0-9]{4}-(0[1-9]|1[012])-(0[1-9]|1[0-9]|2[0-9]|3[01])' ) ) ?>"/>
    </p>

    <p class="form-field sale_price_dates_fields">
        <label for="_gift4u_sale_date_to"><?php esc_html_e( 'Discount date to', 'woocommerce' ) ?></label>

        <input type="text" class="short" name="_gift4u_sale_date_to" id="_gift4u_sale_date_to" value="<?php echo esc_attr( $sale_price_dates_to ) ?>"
               placeholder="<?php echo esc_html( _x( 'To&hellip;', 'placeholder', 'woocommerce' ) ) ?>  YYYY-MM-DD"
               maxlength="10" pattern="<?php echo esc_attr( apply_filters( 'woocommerce_date_input_html_pattern', '[0-9]{4}-(0[1-9]|1[012])-(0[1-9]|1[0-9]|2[0-9]|3[01])' ) ) ?>"/>
    </p>

    <hr/><!----------------------HR------------------>

	<?php if ( wc_tax_enabled() ) : ?>
        <div class="">
			<?php
			woocommerce_wp_select(
				array(
					'id'          => '_gift4u_tax_status',
					'value'       => $product_object->get_tax_status( 'edit' ),
					'label'       => __( 'Tax status', 'woocommerce' ),
					'options'     => array(
						'taxable'  => __( 'Taxable', 'woocommerce' ),
						'shipping' => __( 'Shipping only', 'woocommerce' ),
						'none'     => _x( 'None', 'Tax status', 'woocommerce' ),
					),
					'desc_tip'    => 'true',
					'description' => __( 'Define whether or not the entire product is taxable, or just the cost of shipping it.', 'woocommerce' ),
				)
			);

			woocommerce_wp_select(
				array(
					'id'          => '_gift4u_tax_class',
					'value'       => $product_object->get_tax_class( 'edit' ),
					'label'       => __( 'Tax class', 'woocommerce' ),
					'options'     => wc_get_product_tax_class_options(),
					'desc_tip'    => 'true',
					'description' => __( 'Choose a tax class for this product. Tax classes are used to apply different tax rates specific to certain types of product.', 'woocommerce' ),
				)
			);

			do_action( 'woocommerce_product_options_tax' );
			?>
        </div>
        <hr/><!----------------------HR------------------>
	<?php endif; ?>


    <p class="form-field ">
        <label for="gift4u_override_methods"><?php esc_html_e( 'Override delivery methods', 'gift4u-gift-cards-all-in-one-for-woo' ); ?></label>
        <select class="gift4u-select2" multiple="multiple" style="width: 50%;" id="gift4u_override_methods" name="_gift4u_override_methods[]"
                data-placeholder="<?php esc_attr_e( 'Same global option', 'gift4u-gift-cards-all-in-one-for-woo' ); ?>">
			<?php
			$selected_methods = (array) $product_object->get_meta( '_gift4u_override_methods' );
			$methods          = Utils::delivery_methods_list();

			if ( ! empty( $methods ) && is_array( $methods ) ) {
				foreach ( $methods as $method => $method_label ) {
					$selected = in_array( $method, $selected_methods ) ? 'selected' : '';
					printf( '<option value="%s" %s>%s</option>', esc_attr( $method ), esc_attr( $selected ), esc_html( wp_strip_all_tags( $method_label ) ) );
				}
			}
			?>
        </select>
		<?php //echo wc_help_tip( esc_html__( 'Cross-sells are products which you promote in the cart, based on the current product.', 'gift4u-gift-cards-all-in-one-for-woo' ) ); // WPCS: XSS ok.
		?>
    </p>

    <p class="form-field ">
        <label for="gift4u_galleries"><?php esc_html_e( 'Galleries', 'gift4u-gift-cards-all-in-one-for-woo' ); ?></label>
        <select class="gift4u-select-search" multiple="multiple" style="width: 50%;" id="gift4u_galleries" name="_gift4u_galleries[]"
                data-placeholder="<?php esc_attr_e( 'Search for a gallery', 'gift4u-gift-cards-all-in-one-for-woo' ); ?>" data-action="gift4u_search_gallery">
			<?php
			$gallery_ids = $product_object->get_meta( '_gift4u_galleries' );

			if ( ! empty( $gallery_ids ) && is_array( $gallery_ids ) ) {
				foreach ( $gallery_ids as $gallery_id ) {
					$gallery = get_post( $gallery_id );
					if ( is_object( $gallery ) && $gallery->post_type == 'gift4u_gallery' ) {
						printf( '<option value="%s" selected>%s</option>', esc_attr( $gallery_id ), esc_html( wp_strip_all_tags( $gallery->post_title ) ) );
					}
				}
			}

			?>
        </select>
		<?php //echo wc_help_tip( esc_html__( 'Cross-sells are products which you promote in the cart, based on the current product.', 'gift4u-gift-cards-all-in-one-for-woo' ) ); // WPCS: XSS ok.
		?>
    </p>
    <p class="form-field ">
        <label for="gift4u_card_templates"><?php esc_html_e( 'Card templates', 'gift4u-gift-cards-all-in-one-for-woo' ); ?></label>
        <select class="gift4u-select-search" multiple="multiple" style="width: 50%;" id="gift4u_card_templates" name="_gift4u_card_templates[]"
                data-placeholder="<?php esc_attr_e( 'Search for a card template', 'gift4u-gift-cards-all-in-one-for-woo' ); ?>" data-action="gift4u_search_card_template">
			<?php
			$card_template_ids = $product_object->get_meta( '_gift4u_card_templates' );

			if ( ! empty( $card_template_ids ) && is_array( $card_template_ids ) ) {
				foreach ( $card_template_ids as $card_template_id ) {
					$card_template = get_post( $card_template_id );
					if ( is_object( $card_template ) && $card_template->post_type == 'gift4u_tmpl' ) {
						printf( '<option value="%s" selected>%s</option>', esc_attr( $card_template_id ), esc_html( wp_strip_all_tags( $card_template->post_title ) ) );
					}
				}
			}

			?>
        </select>
		<?php //echo wc_help_tip( esc_html__( 'Cross-sells are products which you promote in the cart, based on the current product.', 'gift4u-gift-cards-all-in-one-for-woo' ) ); // WPCS: XSS ok.
		?>
    </p>

	<?php
//	$this->pro_option_field( esc_html__( "Upload customer's design", 'gift4u-gift-cards-all-in-one-for-woo' ) );

	woocommerce_wp_checkbox( [
		'id'    => '_gift4u_delivery_date',
		'value' => $product_object->get_meta( '_gift4u_delivery_date' ) ? 'yes' : 'no',
		'label' => esc_html__( 'Date pickup field', 'gift4u-gift-cards-all-in-one-for-woo' ),
//					'description'   => esc_html__( 'Manage stock level (quantity)', 'gift4u-gift-cards-all-in-one-for-woo' ),
	] );

	woocommerce_wp_checkbox( [
		'id'    => '_gift4u_sender_name',
		'value' => $product_object->get_meta( '_gift4u_sender_name' ) ? 'yes' : 'no',
		'label' => esc_html__( 'Sender name field', 'gift4u-gift-cards-all-in-one-for-woo' ),
//					'description'   => esc_html__( 'Manage stock level (quantity)', 'gift4u-gift-cards-all-in-one-for-woo' ),
	] );

	woocommerce_wp_checkbox( [
		'id'    => '_gift4u_recipient_name',
		'value' => $product_object->get_meta( '_gift4u_recipient_name' ) ? 'yes' : 'no',
		'label' => esc_html__( 'Recipient name field', 'gift4u-gift-cards-all-in-one-for-woo' ),
	] );

	woocommerce_wp_checkbox( [
		'id'    => '_gift4u_message_field',
		'value' => $product_object->get_meta( '_gift4u_message_field' ) ? 'yes' : 'no',
		'label' => esc_html__( 'Message field', 'gift4u-gift-cards-all-in-one-for-woo' ),
	] );

	woocommerce_wp_checkbox( [
		'id'          => '_gift4u_shipping_form',
		'value'       => $product_object->get_meta( '_gift4u_shipping_form' ) ? 'yes' : 'no',
		'label'       => esc_html__( "Shipping address form", 'gift4u-gift-cards-all-in-one-for-woo' ),
		'description' => esc_html__( '(ship card method)', 'gift4u-gift-cards-all-in-one-for-woo' ),

	] );

	woocommerce_wp_text_input( [
		'id'                => '_gift4u_message_character_limit',
		'label'             => esc_html__( 'Message length limit', 'gift4u-gift-cards-all-in-one-for-woo' ),
		'value'             => $product_object->get_meta( '_gift4u_message_character_limit', true ),
		'default'           => '',
		'class'             => '',
		'placeholder'       => '',
		'type'              => 'number',
		'custom_attributes' => [ 'min' => 0 ]
	] );

//	$this->pro_option_field( esc_html__( "Overwrite delivery methods", 'gift4u-gift-cards-all-in-one-for-woo' ) );

	?>

</div>
