jQuery(document).ready(function ($) {
    'use strict';

    let ajaxAction = {action: 'gift4u_ajax_action', nonce: gift4uParams.nonce},
        body = $('body');

    if (gift4uParams.editPage) {
        /* Edit page =================================================*/

        let media = wp.media();
        let imgField = $('.gift4u-email-design');

        $('.vi-ui.dropdown').viDropdown();

        function changeCountry() {
            let country = $(this).viDropdown('get value');
            let stateField = $('.gift4u-state-field'),
                stateValue = stateField[0].localName === 'input' ? stateField.val() : stateField.viDropdown('get value'),
                input_name = stateField.attr('name'),
                input_id = stateField.attr('id'),
                value = stateValue,
                placeholder = stateField.attr('placeholder'),
                $newstate;

            if (!$.isEmptyObject(gift4uParams.countries[country])) {
                let states = gift4uParams.countries[country],
                    $defaultOption = $('<option value=""></option>').text('Select...');

                $newstate = $(`<select class="gift4u-state-field" id="${input_id}" name="${input_name}" placeholder="${placeholder}"></select>`).append($defaultOption);

                $.each(states, function (index) {
                    var $option = $(`<option value="${index}">${states[index]}</option>`);
                    if (index === stateValue) $option.prop('selected');
                    $newstate.append($option);
                });

                $newstate.val(value);
                stateField.replaceWith($newstate);
                $newstate.viDropdown();

            } else {
                $newstate = $(`<input type="text" class="gift4u-state-field" id="${input_id}" name="${input_name}" placeholder="${placeholder}" value="${stateValue}"/>`);
                stateField.replaceWith($newstate);
            }
        }

        function changeDeliveryMethod() {
            let method = $(this).viDropdown('get value');
            if (method === 'email') $('.gift4u-edit-shipping-address-section').hide();
            if (method === 'ship_card') $('.gift4u-edit-shipping-address-section').show(300);
        }

        function selectImage() {
            media.open().off('select close').on('select', function (e) {
                let selection = media.state().get('selection');
                selection.each(function (attachment) {
                    attachment = attachment.toJSON();
                    if (attachment.type === 'image') {
                        imgField.prop('src', attachment.url);
                        $('.gift4u-email-design-id').val(attachment.id);
                    }
                });
            });
        }

        function removeImage() {
            imgField.prop('src', imgField.data('img_placeholder'));
            $('.gift4u-email-design-id').val('');
        }

        body
            .on('change', '.gift4u-country-field', changeCountry)
            .on('change', '.gift4u-delivery-method', changeDeliveryMethod)
            .on('click', '.gift4u-select-image', selectImage)
            .on('click', '.gift4u-remove-image', removeImage);

        $('.gift4u-delivery-method').trigger('change');
        $('.vi-ui.menu .item').viTab();
        $('.gift4u-categories-field').select2({width: '100%'});

        $(':input.gift4u-products-field').filter(':not(.enhanced)').each(function () {
            let select2_args = {
                width: '100%',
                allowClear: true,
                placeholder: $(this).data('placeholder'),
                minimumInputLength: $(this).data('minimum_input_length') ? $(this).data('minimum_input_length') : '2',
                escapeMarkup: m => m,
                ajax: {
                    url: gift4uParams.ajaxUrl,
                    dataType: 'json',
                    delay: 250,
                    data: (params) => {
                        return {
                            term: params.term,
                            action: $(this).data('action'),
                            security: gift4uParams.search_products_nonce,
                            exclude: $(this).data('exclude'),
                            exclude_type: $(this).data('exclude_type'),
                            include: $(this).data('include'),
                            limit: $(this).data('limit'),
                            display_stock: $(this).data('display_stock')
                        };
                    },
                    processResults: (data) => {
                        let terms = [];
                        if (data) {
                            $.each(data, function (id, text) {
                                terms.push({id: id, text: text});
                            });
                        }
                        return {results: terms};
                    },
                    cache: true
                }
            };

            $(this).select2(select2_args);
            $(this).addClass('enhanced');
        });

        let searchCardField = $('.gift4u-card-template');
        let placeholder = searchCardField.attr('placeholder');
        searchCardField.select2({
            width: '100%',
            allowClear: true,
            placeholder: placeholder,
            minimumInputLength: 2,
            escapeMarkup: m => m,
            ajax: {
                url: gift4uParams.ajaxUrl,
                dataType: 'json',
                delay: 250,
                data: (params) => {
                    return {
                        term: params.term,
                        action: 'gift4u_search_card_template',
                        security: gift4uParams.nonce,
                    };
                },
                processResults: (data) => {
                    let terms = [];
                    if (data) {
                        $.each(data, function (id, text) {
                            terms.push({id: id, text: text});
                        });
                    }
                    return {results: terms};
                },
                cache: true
            }
        });

    } else {

        function sendEmail() {
            if (!confirm('Do you want to send this email?')) return;

            let $this = $(this);
            $this.addClass('gift4u-loading');

            $.ajax({
                url: gift4uParams.ajaxUrl,
                type: 'post',
                dataType: 'json',
                data: {
                    ...ajaxAction,
                    sub_action: 'send_email',
                    coupon_id: $this.data('coupon_id'),
                },

                success(res) {
                    res.success ? $.viToast({class: 'success', message: res.data.message,}) : $.viToast({class: 'error', message: res.data,});
                    if (res.data.delivery_status) {
                        $this.closest('tr').find('td.column-delivery_status').text(res.data.delivery_status)
                    }
                },

                complete() {
                    $this.removeClass('gift4u-loading');
                }
            });
        }

        function markShipped() {
            let $this = $(this);
            $this.addClass('gift4u-loading');

            $.ajax({
                url: gift4uParams.ajaxUrl,
                type: 'post',
                dataType: 'json',
                data: {
                    ...ajaxAction,
                    sub_action: 'mark_shipped',
                    coupon_id: $this.data('coupon_id'),
                },

                success(res) {
                    res.success ? $.viToast({class: 'success', message: res.data.message,}) : $.viToast({class: 'error', message: res.data,});
                    if (res.data.delivery_status) {
                        $this.closest('tr').find('td.column-delivery_status').text(res.data.delivery_status)
                    }

                        $this.closest('tr').find('td.column-expiry_date').text(res.data.expiry_date)
                },

                complete() {
                    $this.removeClass('gift4u-loading');
                }
            });
        }

        function downloadPDF() {
            let $this = $(this);
            $this.addClass('gift4u-loading');
            let couponId = $this.data('coupon_id');
            let couponCode = $this.data('coupon_code');

            $.ajax({
                url: gift4uParams.ajaxUrl,
                type: 'post',
                data: {
                    ...ajaxAction,
                    sub_action: 'download_pdf',
                    coupon_id: couponId,
                },
                xhr: function () {
                    let xhr = new XMLHttpRequest();
                    xhr.onreadystatechange = function () {
                        if (xhr.readyState === 2) {
                            if (xhr.status === 200) {
                                xhr.responseType = "blob";
                            } else {
                                xhr.responseType = "text";
                            }
                        }
                    };
                    return xhr;
                },
                success(data) {
                    // let fileName = gift4uParams.fileName ? `${gift4uParams.fileName}.pdf` : `gift4u_${couponCode}.pdf`;
                    let fileName = `gift4u_${couponCode}.pdf`;
                    //Convert the Byte Data to BLOB object.
                    let blob = new Blob([data], {type: "application/octetstream"});

                    //Check the Browser type and download the File.
                    let isIE = !!document.documentMode;
                    if (isIE) {
                        window.navigator.msSaveBlob(blob, fileName);
                    } else {
                        let url = window.URL || window.webkitURL,
                            link = url.createObjectURL(blob),
                            a = $(`<a download="${fileName}" href="${link}"/>`);

                        body.append(a);
                        a[0].click();
                        body.remove(a);
                    }
                },
                error(res) {
                    console.log(res)
                },
                complete() {
                    $this.removeClass('gift4u-loading');
                }
            });
        }

        body
            .on('click', '.gift4u-send-email:not(.gift4u-loading)', sendEmail)
            .on('click', '.gift4u-mark-shipped:not(.gift4u-loading)', markShipped)
            .on('click', '.gift4u-download-pdf:not(.gift4u-loading)', downloadPDF);

    }


});