jQuery(document).ready(function ($) {
    'use strict';

    let $body = $('body');
    let $emailDesignId = $(".gift4u-email-design");
    let $cardTemplateId = $(".gift4u-card-template");
    let $loaded = false;
    let $imgList = $('.gift4u-image-list');
    let $emailMethodDepend = $('.gift4u-email-method');
    let $shipCardMethodDepend = $('.gift4u-ship-card-method');

    let Modal = {
        modal: $("#gift4u-modal-frame"),
        setContent(content) {
            this.modal.find('.gift4u-modal-content-inner').html(content)
        },
        open() {
            this.modal.show(300)
        },
        close() {
            this.modal.hide(300)
        }
    };

    function selectEmailDesign() {
        $('.gift4u-image-list .gift4u-image-item').removeClass('gift4u-selected');
        $(this).addClass('gift4u-selected');
        let $imageId = $(this).data('image_id');
        $emailDesignId.val($imageId);
    }

    function selectCardTemplate() {
        $('.gift4u-card-template-list .gift4u-card-template-item').removeClass('gift4u-selected');
        $(this).addClass('gift4u-selected');
        let $cardId = $(this).data('card_template_id');
        $cardTemplateId.val($cardId);
    }

    function modalSelectEmailDesign() {
        $('#gift4u-modal-frame .gift4u-image-item').removeClass('gift4u-selected');
        $(this).addClass('gift4u-selected');
    }

    function selectDeliveryMethod() {
        $('.gift4u-select-delivery-method').removeClass('gift4u-button-selected');
        $(this).addClass('gift4u-button-selected');
        let $method = $(this).data('value');
        $('[name=gift4u_delivery_method]').val($method);

        $emailMethodDepend.hide();
        $shipCardMethodDepend.hide();

        switch ($method) {
            case 'email':
                $emailMethodDepend.fadeIn(200);
                break;

            case 'ship_card':
                $shipCardMethodDepend.fadeIn(300);
                break;
        }
    }

    function loadMoreGallery() {
        if ($loaded) {
            Modal.setContent($loaded);
            Modal.open();
            return;
        }

        let $thisBtn = $(this);
        let $pid = $thisBtn.data('product_id');

        if ($pid) {
            Modal.close();

            $thisBtn.addClass('gift4u-loading');
            $.ajax({
                url: gift4uParams.ajaxUrl,
                type: 'post',
                dataType: 'json',
                data: {
                    action: 'gift4u_load_more_gallery',
                    pid: $pid,
                    security: gift4uParams.nonce,
                },
                success(res) {
                    if (res.success) {
                        Modal.setContent(res.data);
                        Modal.open();
                        $loaded = res.data;
                    }
                },
                complete() {
                    $thisBtn.removeClass('gift4u-loading');
                }
            })
        }
    }

    function changeTab() {
        let $thisBtn = $(this);
        let $gId = $thisBtn.data('gift4u_gallery_tab');

        $('#gift4u-modal-frame .gift4u-image-item').removeClass('gift4u-selected');
        $('#gift4u-modal-frame .gift4u-gallery-title').removeClass('gift4u-active');
        $('#gift4u-modal-frame .gift4u-images-tab-item').removeClass('gift4u-active');

        let $targetTab = $(`#gift4u-modal-frame .gift4u-images-tab-item[data-gift4u_gallery_tab=${$gId}]`);

        $thisBtn.addClass('gift4u-active');

        $targetTab.addClass('gift4u-active');
        $targetTab.find('.gift4u-image-item').first().addClass('gift4u-selected')
    }

    function modalUseEmailDesign() {
        let $imgSelected = $('#gift4u-modal-frame .gift4u-image-item.gift4u-selected');
        if ($imgSelected.length) {
            let $imageId = $imgSelected.data('image_id');
            $emailDesignId.val($imageId);

            let $thisGallery = $imgSelected.parent().html();
            $imgList.find('.gift4u-image-item').remove();
            $imgList.prepend($thisGallery);
            Modal.close();
        }
    }

    function keyUp(e) {
        if (e.key === 'Escape') Modal.close();
    }

    function emailDesignPreview() {
        if (!gift4uParams.email_content) return;

        let emailTmpl = gift4uParams.email_content;

        let imgUrl = $('.gift4u-section-wrapper .gift4u-image-item.gift4u-selected img').attr('src');
        let senderName = $('.gift4u-sender-name').val();
        let recipientName = $('.gift4u-recipient-name').val();
        let message = $('.gift4u-message').val();

        if (message) message = message.replace(/\n\r?/g, '<br />');

        emailTmpl = emailTmpl.replaceAll('{selected_image}', `<img src="${imgUrl}" style="margin: auto;"/>`);
        emailTmpl = emailTmpl.replaceAll('{sender_name}', senderName);
        emailTmpl = emailTmpl.replaceAll('{recipient_name}', recipientName);
        emailTmpl = emailTmpl.replaceAll('{message}', message);

        if (gift4uParams.priceType === 'fixed') {
            emailTmpl = emailTmpl.replaceAll('{coupon_value}', gift4uParams.regularPrice);
        } else if (gift4uParams.priceType === 'input') {
            let price = $('.gift4u-user-input-amount').val();
            price = accounting.formatMoney(price, gift4uProductParams.currencySymbol, gift4uProductParams.currencyPrecision, gift4uProductParams.thousandsSep, gift4uProductParams.decimalSep, gift4uProductParams.priceFormat);
            emailTmpl = emailTmpl.replaceAll('{coupon_value}', price);
        }

        Modal.setContent(emailTmpl);
        Modal.open();
    }

    function renderView(type, data) {
        let view = data[type],
            {width, height} = data,
            {texts = [], background_color = 'white'} = view,
            textsHtml = texts.map((text) => {
                let {pos_x, pos_y, size, font_weight = 'normal', font_family = 'inherit', content = '', color = 'inherit'} = text;
                let style = `position:absolute; top: ${pos_y}px; left: ${pos_x}px; color:${color};`;
                style += `font-size: ${size}px; font-weight: ${font_weight}; font-family:${font_family}`;

                if (content) {
                    let senderName = $('.gift4u-sender-name').val();
                    let recipientName = $('.gift4u-recipient-name').val();
                    let message = $('.gift4u-message').val();

                    content = content.replaceAll('{sender_name}', senderName);
                    content = content.replaceAll('{recipient_name}', recipientName);
                    content = content.replaceAll('{message}', message);
                }

                return `<div style="${style}">${content}</div>`;
            }).join('');

        let styles = `background-repeat: no-repeat;background-size: contain;background-color:${background_color};`;
        styles += `width: ${width}mm;height: ${height}mm; position:relative;`;

        if (view.imgUrl) styles += `background-image: url(${view.imgUrl});`;

        return `<div class="gift4u-card-preview-item">
                    <div class="gift4u-card-template-${type}">
                        <div style="${styles}">
                        ${textsHtml}
                        </div>
                    </div>
                </div>`;
    }

    function cardTemplatePreview() {
        let cardData = $('.gift4u-section-wrapper .gift4u-card-template-item.gift4u-selected').data('card_data');
        let front = renderView('front', cardData);
        let back = renderView('back', cardData);
        let html = `${front}${back}`;

        Modal.setContent(html);
        Modal.open();

    }

    function getDiscountedAmount(amount, rules) {
        let discountArr = [];

        for (let rule of rules) {
            if (!rule.discount_value) continue;
            if (rule.from_price !== '' && +rule.from_price > +amount) continue;
            if (rule.to_price !== '' && +rule.to_price < +amount) continue;

            let {discount_type = 'fixed', discount_value = 0} = rule;
            switch (discount_type) {
                case 'fixed':
                    discountArr.push(discount_value);
                    break;

                case 'percentage':
                    discountArr.push((amount * discount_value) / 100);
                    break;
            }
        }

        let discountValue = 0;

        if (discountArr.length) discountValue = gift4uProductParams.discount_umpire === 'min' ? Math.min(...discountArr) : Math.max(...discountArr);


        return amount - discountValue;
    }

    function onUserInputChange() {
        let amount = $(this).val(),
            min = $(this).attr('min'),
            max = $(this).attr('max');

        if (typeof gift4uProductParams === 'undefined' || !gift4uProductParams.rules.length) return;
        if (!amount || (min !== '' && +amount < +min) || (max !== '' && +amount > +max)) return;

        amount = getDiscountedAmount(amount, gift4uProductParams.rules);
        amount = amount < 0 ? 0 : amount;
        amount = accounting.formatMoney(amount, gift4uProductParams.currencySymbol, gift4uProductParams.currencyPrecision, gift4uProductParams.thousandsSep, gift4uProductParams.decimalSep, gift4uProductParams.priceFormat);

        $('.gift4u-purchase-amount').html(amount);
    }

    function stopChangeQuantity(e) {
        e.preventDefault();
        e.stopImmediatePropagation();
    }

    function onEmailInputChange() {
        let recipientEmailInput = $(this);
        let mailString = recipientEmailInput.val().trim();
        let emailsMatch = mailString.match(/[a-zA-Z0-9-_.]+@[a-zA-Z0-9-_.]+/ig);
        let quantityInput = $('form .quantity input');
        let minusBtn = $('form .quantity .minus');
        let plusBtn = $('form .quantity .plus');

        if (mailString.length) {
            let splitLength = mailString.split(',').length;
            emailsMatch === null || emailsMatch.length !== splitLength ? recipientEmailInput.addClass('gift4u-error') : recipientEmailInput.removeClass('gift4u-error')
        } else {
            recipientEmailInput.removeClass('gift4u-error');
        }

        if (emailsMatch && emailsMatch.length > 1) {
            quantityInput.attr('disabled', true);
            quantityInput.val(1);
            minusBtn.addClass('disabled');
            plusBtn.addClass('disabled');

            $('form .quantity .minus, form .quantity .plus').on('click', stopChangeQuantity);
        } else {
            quantityInput.attr('disabled', false);
            minusBtn.removeClass('disabled');
            plusBtn.removeClass('disabled');
            $('form .quantity .minus, form .quantity .plus').off('click', stopChangeQuantity);
        }
    }

    function checkCharacterLimit() {
        let limit = $(this).data('character_limit'),
            content = $(this).val();

        if (limit === '') return;

        content.length > limit ? $(this).addClass('gift4u-error') : $(this).removeClass('gift4u-error')
    }

    function preventEventsInPreview(e) {
        e.preventDefault();
        e.stopImmediatePropagation();
    }

    $body.on('keyup', keyUp)
        .on('click', '.gift4u-image-list .gift4u-image-item', selectEmailDesign)
        .on('click', '.gift4u-card-template-list .gift4u-card-template-item', selectCardTemplate)
        .on('click', '#gift4u-modal-frame .gift4u-image-item', modalSelectEmailDesign)
        .on('click', '.gift4u-select-delivery-method', selectDeliveryMethod)
        .on('click', '.gift4u-see-more-btn:not(.gift4u-loading)', loadMoreGallery)
        .on('click', '.gift4u-modal-overlay, .gift4u-modal-close', () => Modal.close())
        .on('click', '.gift4u-gallery-title', changeTab)
        .on('click', '.gift4u-use-design', modalUseEmailDesign)
        .on('click', '.gift4u-email-design-preview:not(.gift4u-loading)', emailDesignPreview)
        .on('click', '.gift4u-card-template-preview:not(.gift4u-loading)', cardTemplatePreview)
        .on('keyup change', '.gift4u-user-input-amount', onUserInputChange)
        .on('change', '.gift4u-recipient-email', onEmailInputChange)
        .on('keyup', '.gift4u-message', checkCharacterLimit)
        .on('click', '#gift4u-email-design-preview', preventEventsInPreview);

    $body.find('.gift4u-user-input-amount').trigger('keyup');

    $('.gift4u-select-delivery-method.gift4u-button-selected').trigger('click');
    $(document.body).trigger('country_to_state_changed'); // Trigger select2 to load.

});