<?php

namespace Gift4U\Front;

use Gift4U\GiftCard;
use Gift4U\Inc\Classes\Email;
use Gift4U\Inc\Classes\Giftcard_Coupon;
use Gift4U\Inc\Data;
use Gift4U\Inc\Utils;

defined( 'ABSPATH' ) || exit;

class My_Account {
	protected static $instance = null;
	protected $endpoint = null;

	public function __construct() {
		$this->endpoint = Data::get_param( 'myaccount_endpoint' );

		if ( $this->endpoint ) {
			add_filter( 'woocommerce_account_menu_items', [ $this, 'add_my_account_tab' ] );
			add_action( "woocommerce_account_{$this->endpoint}_endpoint", [ $this, 'giftcard_content' ] );
//			add_filter( 'query_vars', [ $this, 'add_query_vars_giftcard' ], 0 );
			add_filter( 'woocommerce_get_query_vars', [ $this, 'add_query_vars' ] );
//			add_action( 'init', [ $this, 'register_new_item_endpoint' ] );
			add_action( 'init', [ $this, 'request_send_giftcard' ] );
		}
	}

	public static function instance() {
		return self::$instance == null ? self::$instance = new self() : self::$instance;
	}

	public function register_new_item_endpoint() {
		add_rewrite_endpoint( 'giftcard', EP_ROOT | EP_PAGES );
	}

	public function add_query_vars( $vars ) {
		$vars['giftcard'] = $this->endpoint;

		return $vars;

	}

	public function add_query_vars_giftcard( $vars ) {
		$vars[] = $this->endpoint;

		$vars[] = 'paged';

		return $vars;
	}

	public function add_my_account_tab( $items ) {
		$last_item      = array_key_last( $items );
		$last_item_text = $items[ $last_item ];

		unset( $items[ $last_item ] );

		$items[ $this->endpoint ] = esc_html__( 'Gift cards', 'gift4u-gift-cards-all-in-one-for-woo' );
		$items[ $last_item ]      = $last_item_text;

		return $items;
	}

	public function giftcard_content() {
		$current = get_query_var( 'giftcard' ) ? (int) get_query_var( 'giftcard' ) : 1;

		$user = wp_get_current_user();

		$send_status  = ! empty( $_GET['gift4u-send-status'] ) ? sanitize_text_field( wp_unslash( $_GET['gift4u-send-status'] ) ) : '';
		$send_message = '';
		if ( $send_status == 'success' ) {
			$send_message = esc_html__( 'Email is sent successfully', 'gift4u-gift-cards-all-in-one-for-woo' );
		} elseif ( $send_status == 'error' ) {
			$send_message = esc_html__( 'Email is sent failed', 'gift4u-gift-cards-all-in-one-for-woo' );
		}

		$results = Utils::get_giftcard_coupons( [
			'per_page'   => 10,
			'page'       => $current,
			'sender_id'  => $user->ID,
			'from_email' => $user->user_email,
			'to_email'   => $user->user_email,
			'paginate'   => true,
		] );

		Utils::get_template( 'myaccount-giftcard-table', [
			'giftcards'           => $results['giftcard_coupons'],
			'total'               => $results['max_page'],
			'methods_list'        => Utils::delivery_methods_list(),
			'nonce'               => wp_create_nonce( 'send_giftcard' ),
			'send_message_status' => $send_status,
			'send_message'        => $send_message,
		] );
	}

	public function request_send_giftcard() {
		if ( empty( $_GET['gift4u_send'] ) || empty( $_GET['gift4u_nonce'] ) || ! is_user_logged_in() ) {
			return;
		}

		if ( ! wp_verify_nonce( sanitize_text_field( wp_unslash( $_GET['gift4u_nonce'] ) ), 'send_giftcard' ) ) {
			return;
		}

		$gift_card_id = absint( $_GET['gift4u_send'] );
		$gc_coupon    = new Giftcard_Coupon( $gift_card_id );
		$gc_mail      = new Email( $gc_coupon );

		$result   = $gc_mail->send_email( $gc_coupon->get_to_email() );
		$status   = $result ? 'success' : 'error';
		$url      = remove_query_arg( [ 'gift4u_send', 'gift4u_nonce' ] );
		$redirect = add_query_arg( [ 'gift4u-send-status' => $status ], $url );

		wp_safe_redirect( $redirect );
		exit;
	}

}
