<?php

namespace Gift4U\Front;

use Gift4U\Inc\Classes\Email;
use Gift4U\Inc\Classes\Giftcard_Coupon;
use Gift4U\Inc\Classes\PDF;
use Gift4U\Inc\Data;
use Gift4U\Inc\Utils;

defined( 'ABSPATH' ) || exit;

class Order_Process {
	protected static $instance = null;

	public function __construct() {
		add_action( 'woocommerce_add_cart_item_data', [ $this, 'woocommerce_add_cart_item_data' ], 10, 4 );
		add_action( 'woocommerce_add_to_cart', [ $this, 'add_to_cart_processing' ], 10, 6 );
		add_filter( 'woocommerce_add_to_cart_validation', [ $this, 'add_to_cart_validation' ], 10, 3 );
		add_filter( 'woocommerce_get_cart_item_from_session', [ $this, 'calculate_quantity' ], 10, 2 );
		add_action( 'woocommerce_before_calculate_totals', [ $this, 'calculator_cart_item_pricing' ] );
		add_filter( 'woocommerce_cart_item_price', [ $this, 'cart_item_price' ], 100, 3 );
		add_action( 'woocommerce_order_status_changed', [ $this, 'order_status_changed' ], 10, 4 );
		add_action( 'woocommerce_checkout_create_order_line_item', [ $this, 'checkout_create_order_line_item' ], 10, 4 );

		add_action( 'woocommerce_after_cart_item_name', [ $this, 'edit_item_button' ], 10, 2 );
		add_filter( 'woocommerce_get_item_data', [ $this, 'display_coupon_amount' ], 10, 2 );
		add_filter( 'woocommerce_get_shop_coupon_data', [ $this, 'add_coupon' ], 10, 3 );

		add_action( 'woocommerce_after_cart_table', [ $this, 'coupon_list_dropdown' ] );
		add_action( 'woocommerce_before_checkout_form', [ $this, 'coupon_list_dropdown' ] );

		add_action( 'wp_head', [ $this, 'apply_coupon_via_comeback_url' ] );

		add_action( 'woocommerce_order_status_pending', [ $this, 'update_coupon_usage_counts' ], 1 );
		add_action( 'woocommerce_order_status_completed', [ $this, 'update_coupon_usage_counts' ], 1 );
		add_action( 'woocommerce_order_status_processing', [ $this, 'update_coupon_usage_counts' ], 1 );
		add_action( 'woocommerce_order_status_on-hold', [ $this, 'update_coupon_usage_counts' ], 1 );
		add_action( 'woocommerce_order_status_cancelled', [ $this, 'update_coupon_usage_counts' ], 1 );

		add_filter( 'woocommerce_ship_to_different_address_checked', [ $this, 'ship_to_different_address_checkbox' ] );
	}

	public static function instance() {
		return self::$instance == null ? self::$instance = new self : self::$instance;
	}

	public function woocommerce_add_cart_item_data( $cart_item_data, $product_id, $variation_id, $quantity ) {
		$product = wc_get_product( $product_id );

		if ( ! $product || 'giftcard' != $product->get_type() ) {
			return $cart_item_data;
		}

		$arr_inputs = [ 'delivery_date', 'sender_name', 'recipient_name', 'email_design', 'card_template' ];

		foreach ( $arr_inputs as $input ) {
			if ( ! empty( $_REQUEST[ 'gift4u_' . $input ] ) ) {
				$value = sanitize_text_field( wp_unslash( $_REQUEST[ 'gift4u_' . $input ] ) );

				$cart_item_data['gift4u_data'][ $input ] = $value;
			}
		}


		if ( ! empty( $_REQUEST['gift4u_message'] ) ) {
			$value = sanitize_textarea_field( wp_unslash( $_REQUEST['gift4u_message'] ) );

			$cart_item_data['gift4u_data']['message'] = $value;
		}

		if ( ! empty( $_REQUEST['gift4u_recipient_email'] ) ) {
			$emails = sanitize_text_field( wp_unslash( $_REQUEST['gift4u_recipient_email'] ) );
			preg_match_all( '/[a-zA-Z0-9-_.]+@[a-zA-Z0-9-_.]+/', $emails, $matches );

			$emails = $matches[0];
			$emails = array_map( 'sanitize_email', $emails );

			if ( ! empty( $emails ) ) {
				$cart_item_data['gift4u_data']['recipient_emails'] = $emails;
			}
		}

		if ( ! empty( $_REQUEST['gift4u_coupon_amount'] ) ) {
			$recipient_date = floatval( wp_unslash( $_REQUEST['gift4u_coupon_amount'] ) );

			$cart_item_data['gift4u_data']['coupon_amount'] = $recipient_date;
		}

		$delivery_methods = [ 'email', 'ship_card' ];

		$delivery_method = ! empty( $_REQUEST['gift4u_delivery_method'] ) && in_array( $_REQUEST['gift4u_delivery_method'], $delivery_methods )
			? sanitize_text_field( wp_unslash( $_REQUEST['gift4u_delivery_method'] ) ) : 'email';

		$cart_item_data['gift4u_data']['delivery_method'] = $delivery_method;

		return $cart_item_data;
	}

	public function add_to_cart_validation( $passed, $product_id, $quantity ) {
		$product = wc_get_product( $product_id );
		if ( ! $product->is_type( 'giftcard' ) ) {
			return $passed;
		}

		WC()->customer->set_props( [
			'shipping_country'   => isset( $_REQUEST['shipping_country'] ) ? wc_clean( wp_unslash( $_REQUEST['shipping_country'] ) ) : null,
			'shipping_state'     => isset( $_REQUEST['shipping_state'] ) ? wc_clean( wp_unslash( $_REQUEST['shipping_state'] ) ) : null,
			'shipping_postcode'  => isset( $_REQUEST['shipping_postcode'] ) ? wc_clean( wp_unslash( $_REQUEST['shipping_postcode'] ) ) : null,
			'shipping_city'      => isset( $_REQUEST['shipping_city'] ) ? wc_clean( wp_unslash( $_REQUEST['shipping_city'] ) ) : null,
			'shipping_address_1' => isset( $_REQUEST['shipping_address_1'] ) ? wc_clean( wp_unslash( $_REQUEST['shipping_address_1'] ) ) : null,
			'shipping_address_2' => isset( $_REQUEST['shipping_address_2'] ) ? wc_clean( wp_unslash( $_REQUEST['shipping_address_2'] ) ) : null,
		] );

		WC()->customer->save();

		if ( ! empty( $_REQUEST['gift4u_recipient_email'] ) ) {
			$emails = sanitize_text_field( wp_unslash( $_REQUEST['gift4u_recipient_email'] ) );
			preg_match_all( '/[a-zA-Z0-9-_.]+@[a-zA-Z0-9-_.]+/', $emails, $matches );

			$emails = $matches[0];
			$emails = array_map( 'sanitize_email', $emails );

			if ( empty( $emails ) ) {
				wc_add_notice( esc_html__( 'Email is invalid', 'gift4u-gift-cards-all-in-one-for-woo' ), 'error' );

				return false;
			}
		}

//		if ( ! empty( $_REQUEST['gift4u_delivery_date'] ) ) {
//			$delivery_date = sanitize_text_field( wp_unslash( $_REQUEST['gift4u_delivery_date'] ) );
//			if ( strtotime( $delivery_date ) < strtotime( 'tomorrow' ) ) {
//				wc_add_notice( esc_html__( 'Please select delivery date in future', 'gift4u-gift-cards-all-in-one-for-woo' ), 'error' );
//
//				return false;
//			}
//		}

		$price_type = $product->get_price_type();

		if ( 'input' == $price_type ) {
			if ( empty( $_REQUEST['gift4u_coupon_amount'] ) ) {
				$passed = false;
				wc_add_notice( esc_html__( 'Please input your amount', 'gift4u-gift-cards-all-in-one-for-woo' ), 'error' );
			} else {
				$coupon_amount = floatval( wp_unslash( $_REQUEST['gift4u_coupon_amount'] ) );

				if ( $coupon_amount != floatval( $coupon_amount ) ) {
					$passed = false;
					wc_add_notice( esc_html__( 'Amount is invalid', 'gift4u-gift-cards-all-in-one-for-woo' ), 'error' );
				}

				$min = $product->get_user_input_min_price();
				if ( '' != $min && $coupon_amount < $min ) {
					$passed = false;
					wc_add_notice( sprintf( "%s %s", esc_html__( 'Please input value greater than', 'gift4u-gift-cards-all-in-one-for-woo' ), wc_price( $min ) ), 'error' );
				}

				$max = $product->get_user_input_max_price();
				if ( '' != $max && $coupon_amount > $max ) {
					$passed = false;
					wc_add_notice( sprintf( "%s %s", esc_html__( 'Please input value smaller than', 'gift4u-gift-cards-all-in-one-for-woo' ), wc_price( $max ) ), 'error' );
				}
			}
		}

		$c_limit = $product->get_meta( '_gift4u_message_character_limit' );
		if ( ! empty( $_REQUEST['gift4u_message'] ) && $c_limit !== '' ) {
			$message        = sanitize_textarea_field( wp_unslash( $_REQUEST['gift4u_message'] ) );
			$message_length = strlen( $message );
			if ( $message_length > $c_limit ) {
				wc_add_notice( esc_html__( 'Message is too long', 'gift4u-gift-cards-all-in-one-for-woo' ), 'error' );

				return false;
			}

		}

		return $passed;
	}

	public function add_to_cart_processing( $cart_item_key, $product_id, $quantity, $variation_id, $variation, $cart_item_data ) {
		if ( ! empty( $_REQUEST['gift4u_update_cart_item'] ) ) {
			$remove_key = sanitize_text_field( wp_unslash( $_REQUEST['gift4u_update_cart_item'] ) );
			wc()->cart->remove_cart_item( $remove_key );
		}
	}

	public function calculate_quantity( $cart_item, $values ) {
		if ( ! empty( $values['gift4u_data']['gift4u_recipient_emails'] ) ) {
			$email_qty = count( $values['gift4u_data']['gift4u_recipient_emails'] );
			if ( $email_qty > 1 ) {
				$cart_item['quantity'] = $email_qty;
			}
		}

		return $cart_item;
	}

	/**
	 * @param  $cart \WC_Cart
	 */
	public function calculator_cart_item_pricing( $cart ) {
		$cart_contents = $cart->get_cart_contents();

		if ( empty( $cart_contents ) ) {
			return;
		}

		foreach ( $cart_contents as $item_key => $cart_item ) {
			$product = $cart_item['data'];
			if ( ! ( $product instanceof \WC_Product_Giftcard ) || empty( $cart_item['gift4u_data'] ) ) {
				continue;
			}

			$data = $cart_item['gift4u_data'];

			$price_type = $product->get_price_type();

			if ( 'input' == $price_type ) {
				$amount = $data['coupon_amount'] ?? '';
				if ( ! $amount ) {
					continue;
				}

				$discount_rules = (array) $product->get_user_input_discount_rule();
				$discount_arr   = [];

				foreach ( $discount_rules as $rule ) {
					if ( empty( $rule['discount_value'] ) ) {
						continue;
					}

					if ( isset( $rule['from_price'] ) && $rule['from_price'] !== '' && $rule['from_price'] > $amount ) {
						continue;
					}

					if ( isset( $rule['to_price'] ) && $rule['to_price'] !== '' && $rule['to_price'] < $amount ) {
						continue;
					}

					$discount_type  = ! empty( $rule['discount_type'] ) ? $rule['discount_type'] : 'percentage';
					$discount_value = ! empty( $rule['discount_value'] ) ? $rule['discount_value'] : 0;

					switch ( $discount_type ) {
						case 'fixed':
							$discount_arr[] = $discount_value;
							break;

						case 'percentage':
							$discount_arr[] = ( $amount * $discount_value ) / 100;
							break;
					}
				}

				$discount_umpire = Data::get_param( 'discount_umpire' );
				$discount        = 0;

				if ( ! empty( $discount_arr ) ) {
					$discount = $discount_umpire == 'min' ? min( $discount_arr ) : max( $discount_arr );
				}

				$product->set_price( $amount - $discount );
			}

			$delivery_method = ! empty( $data['delivery_method'] ) ? $data['delivery_method'] : 'email';
			if ( 'email' == $delivery_method ) {
				$product->set_virtual( true );
			}

		}

	}

	public function cart_item_price( $price, $cart_item, $cart_item_key ) {

		return $price;
	}

	/**
	 * @param $order_id
	 * @param $from
	 * @param $to
	 * @param $order \WC_Order
	 */
	public function order_status_changed( $order_id, $from, $to, $order ) {
		$accept_statuses = (array) Data::get_param( 'order_statuses' );
		$order_status    = $order->get_status();

		if ( ! in_array( $order_status, $accept_statuses ) ) {
			return;
		}

		$items = $order->get_items();

		if ( ! empty( $items ) ) {
			foreach ( $items as $item ) {
				$gc_coupon_id = $item->get_meta( '_gift4u_coupon_ids' );

				/** @var \WC_Product_Giftcard $product */
				$product = $item->get_product();

				if ( $product->get_type() != 'giftcard' || $gc_coupon_id ) {
					continue;
				}

				$meta_data = $item->get_meta( '_gift4u_data' );

				$price_type    = $product->get_price_type();
				$coupon_amount = $price_type == 'input' ? ( $meta_data['coupon_amount'] ?? 0 ) : $product->get_regular_price();

				if ( ! $coupon_amount ) {
					continue;
				}

				$delivery_method         = $meta_data['delivery_method'] ?? 'email';
				$delivery_date           = $meta_data['delivery_date'] ?? '';
				$gc_coupon_generated_ids = [];
				$time_offest             = 7 - (int) get_option( 'gmt_offset' );

				switch ( $delivery_method ) {
					case 'email':
						$recipient_emails = $meta_data['recipient_emails'] ?? [];

						if ( empty( $recipient_emails ) ) {
							$recipient_emails = [ $order->get_billing_email() ];
						}

						$email_count = count( $recipient_emails );

						if ( $email_count > 1 ) {
							foreach ( $recipient_emails as $email ) {
								$gc_coupon = $this->generate_coupon( [
									'order'           => $order,
									'email'           => $email,
									'coupon_amount'   => $coupon_amount,
									'delivery_method' => $delivery_method,
									'delivery_date'   => $delivery_date ? $delivery_date : time(),
									'meta_data'       => $meta_data
								] );

								if ( ! $gc_coupon ) {
									continue;
								}

								$gc_coupon_generated_ids[] = $gc_coupon->get_id();

								if ( ! $delivery_date || strtotime( $delivery_date ) <= time() ) {
									$this->send_email( $gc_coupon, $email );
								} elseif ( strtotime( $delivery_date ) > time() ) {
									$time = strtotime( $delivery_date . "+{$time_offest}hour" );
									wp_schedule_single_event( $time, 'gift4u_schedule_send_giftcard', [ $gc_coupon ] );
								}
							}
						} else {
							$item_qty = $item->get_quantity();
							for ( $i = 0; $i < $item_qty; $i ++ ) {
								$email     = current( $recipient_emails );
								$gc_coupon = $this->generate_coupon( [
									'order'           => $order,
									'email'           => $email,
									'coupon_amount'   => $coupon_amount,
									'delivery_method' => $delivery_method,
									'delivery_date'   => $delivery_date ? $delivery_date : time(),
									'meta_data'       => $meta_data
								] );

								if ( ! $gc_coupon ) {
									continue;
								}

								$gc_coupon_generated_ids[] = $gc_coupon->get_id();

								if ( ! $delivery_date || strtotime( $delivery_date ) <= time() ) {
									$this->send_email( $gc_coupon, $email );
								} elseif ( strtotime( $delivery_date ) > time() ) {
									$time = strtotime( $delivery_date . "+{$time_offest}hour" );
									wp_schedule_single_event( $time, 'gift4u_schedule_send_giftcard', [ $gc_coupon ] );
								}
							}
						}

						break;

					case 'ship_card':
						$item_qty = $item->get_quantity();
						$email    = $order->get_billing_email();

						for ( $i = 0; $i < $item_qty; $i ++ ) {
							$gc_coupon = $this->generate_coupon( [
								'order'           => $order,
								'email'           => $email,
								'coupon_amount'   => $coupon_amount,
								'delivery_method' => $delivery_method,
								'delivery_date'   => $delivery_date,
								'meta_data'       => $meta_data
							] );

							if ( ! $gc_coupon ) {
								continue;
							}

							$gc_coupon_generated_ids[] = $gc_coupon->get_id();
						}

						break;
				}

				if ( ! empty( $gc_coupon_generated_ids ) ) {
					$item->add_meta_data( '_gift4u_coupon_ids', $gc_coupon_generated_ids );
					$item->save_meta_data();
				}
			}
		}
	}


	/**
	 * @param array $args
	 *
	 * @return bool|Giftcard_Coupon|\WP_Error|null
	 */
	public function generate_coupon( $args = [] ) {
		$args = wp_parse_args( $args, [
			'order'           => '',
			'email'           => '',
			'coupon_amount'   => '',
			'delivery_method' => '',
			'delivery_date'   => '',
			'expiry_date'     => '',
			'meta_data'       => ''
		] );

		if ( empty( $args['order'] ) || ! $args['order'] instanceof \WC_Order ) {
			return false;
		}

		/** @var \WC_Order $order */
		$order = $args['order'];

		$gc_coupon = Utils::create_giftcard_coupon( [
			'order_id'        => $order->get_id(),
			'from_email'      => $order->get_billing_email(),
			'to_email'        => $args['email'],
			'amount'          => $args['coupon_amount'],
			'delivery_date'   => $args['delivery_date'],
			'delivery_method' => $args['delivery_method'],
			'expiry_date'     => $args['expiry_date'],
			'currency'        => $order->get_currency(),
			'meta_data'       => $args['meta_data']
		] );

		return $gc_coupon;
	}

	/**
	 * @param Giftcard_Coupon $gc_coupon
	 * @param $email
	 */
	public function send_email( $gc_coupon, $email ) {
		$id = $gc_coupon->get_id();
//		$delivery_date = $gc_coupon->get_delivery_date();

		if ( ! $id ) {
			return;
		}

		//send email
		$gc_mail = new Email( $gc_coupon );
		$gc_mail->send_email( $email );
	}

	/**
	 * @param $item \WC_Order_Item_Product
	 * @param $cart_item_key
	 * @param $values
	 * @param $order \WC_Order
	 */
	public function checkout_create_order_line_item( $item, $cart_item_key, $values, $order ) {
		$product = $values['data'];

		if ( ! ( $product instanceof \WC_Product_Giftcard ) || empty( $values['gift4u_data'] ) ) {
			return;
		}

		$data = $values['gift4u_data'];

		$data['coupon_options'] = [
			'allow_shipping'     => $product->get_meta( '_gift4u_cp_allow_shipping' ),
			'email_restrictions' => $product->get_meta( '_gift4u_cp_email_restrictions' ),
			'expiry_date'        => $product->get_meta( '_gift4u_cp_expiry_date' ),
			'minimum_spend'      => $product->get_meta( '_gift4u_cp_minimum_spend' ),
			'maximum_spend'      => $product->get_meta( '_gift4u_cp_maximum_spend' ),
			'individual'         => $product->get_meta( '_gift4u_cp_individual' ),
			'exclude_sale_items' => $product->get_meta( '_gift4u_cp_exclude_sale_items' ),
			'products'           => $product->get_meta( '_gift4u_cp_products' ),
			'excl_products'      => $product->get_meta( '_gift4u_cp_excl_products' ),
			'categories'         => $product->get_meta( '_gift4u_cp_categories' ),
			'excl_categories'    => $product->get_meta( '_gift4u_cp_excl_categories' ),
		];


		$item->add_meta_data( '_gift4u_data', $data );
	}

	public function display_coupon_amount( $item_data, $cart_item ) {
		$product = $cart_item['data'];
		if ( ! ( $product instanceof \WC_Product_Giftcard ) || empty( $cart_item['gift4u_data'] ) ) {
			return $item_data;
		}

		$amount     = '';
		$price_type = $product->get_price_type();

		switch ( $price_type ) {
			case 'input':
				$amount = $cart_item['gift4u_data']['coupon_amount'] ?? $amount;
				break;

			case 'fixed':
				$amount = $product->get_regular_price();
				break;
		}

		if ( $amount ) {
			$quantity = $cart_item['quantity'] ?? 0;
			$unit     = $quantity > 1 ? '/1 ' . esc_html__( 'gift card', 'gift4u-gift-cards-all-in-one-for-woo' ) : '';

			$item_data[] = [
				'key'   => esc_html__( 'Gift card value', 'gift4u-gift-cards-all-in-one-for-woo' ),
				'value' => wc_price( $amount ),
			];

			$gift4u_data = $cart_item['gift4u_data'];

			if ( ! empty( $gift4u_data['recipient_emails'] ) ) {
				$email_qty   = count( $gift4u_data['recipient_emails'] );
				$item_data[] = [
					'key'   => _n( 'Email', 'Emails', $email_qty, 'gift4u-gift-cards-all-in-one-for-woo' ),
					'value' => implode( ', ', $gift4u_data['recipient_emails'] ),
				];
			}

			if ( ! empty( $gift4u_data['delivery_date'] ) ) {
				$item_data[] = [
					'key'   => esc_html__( 'Delivery date', 'gift4u-gift-cards-all-in-one-for-woo' ),
					'value' => $gift4u_data['delivery_date'],
				];
			}

			if ( ! empty( $gift4u_data['sender_name'] ) ) {
				$item_data[] = [
					'key'   => esc_html__( 'Sender', 'gift4u-gift-cards-all-in-one-for-woo' ),
					'value' => $gift4u_data['sender_name'],
				];
			}

			if ( ! empty( $gift4u_data['recipient_name'] ) ) {
				$item_data[] = [
					'key'   => esc_html__( 'recipient', 'gift4u-gift-cards-all-in-one-for-woo' ),
					'value' => $gift4u_data['recipient_name'],
				];
			}

			if ( ! empty( $gift4u_data['message'] ) ) {
				$item_data[] = [
					'key'   => esc_html__( 'Message', 'gift4u-gift-cards-all-in-one-for-woo' ),
					'value' => $gift4u_data['message'],
				];
			}

		}

		return $item_data;
	}

	public function coupon_list_dropdown() {
		if ( ! is_user_logged_in() || ! Data::get_param( 'show_coupon_list' ) ) {
			return;
		}

		$class   = is_cart() ? 'gift4u-cart-page' : ( is_checkout() ? 'gift4u-checkout-page' : '' );
		$user    = wp_get_current_user();
		$results = Utils::get_giftcard_coupons( [
			'per_page'   => 5,
			'page'       => 1,
			'sender_id'  => $user->ID,
			'from_email' => $user->user_email,
			'to_email'   => $user->user_email,
			'used'       => false,
		] );

		if ( ! empty( $results ) ) {
			$applied_coupons = wc()->cart->get_applied_coupons();
			?>
            <div class="form-row form-row-wide <?php echo esc_attr( $class ) ?>">
                <label><?php esc_html_e( 'Coupons available', 'gift4u-gift-cards-all-in-one-for-woo' ); ?></label>
                <div class="gift4u-coupons-list">
					<?php
					/** @var Giftcard_Coupon $gc_coupon */
					foreach ( $results as $gc_coupon ) {
						$gc_coupon_code = $gc_coupon->get_coupon_code();
						$disable        = in_array( $gc_coupon_code, $applied_coupons ) ? 'gift4u-disable' : '';
						?>
                        <div class="gift4u-coupon-item <?php echo esc_attr( $disable ) ?>"
                             data-coupon_code="<?php echo esc_attr( $gc_coupon->get_coupon_code() ) ?>">
                            <div class="gift4u-coupon-handler">%</div>
                            <div class="gift4u-coupon-code">
                                <div><?php echo esc_html( $gc_coupon->get_coupon_code() ) ?></div>
                                <small><?php echo wp_kses_post( wc_price( $gc_coupon->get_amount() ) ) ?></small>
                            </div>
                        </div>
						<?php
					}
					?>
                </div>
            </div>
			<?php
		}
	}

	public function add_coupon( $status, $coupon_code, $coupon ) {
		if ( wc()->cart == null ) {
			return $status;
		}

		$excl_gc_products = [];
		$cart_contents    = wc()->cart->get_cart_contents();
		if ( ! empty( $cart_contents ) ) {
			foreach ( $cart_contents as $cart_item ) {
				$product = $cart_item['data'];
				if ( is_object( $product ) && $product instanceof \WC_Product_Giftcard ) {
					$excl_gc_products[] = $product->get_id();
				}
			}
		}

		$gc_coupons = Utils::get_giftcard_coupons( [ 'coupon_code' => $coupon_code, 'used' => false, 'per_page' => 1 ] );

		if ( ! empty( $gc_coupons ) ) {
			/** @var Giftcard_Coupon $gc_coupon */
			$gc_coupon            = current( $gc_coupons );
			$coupon_options       = $gc_coupon->get_coupon_options();
			$email_restrictions   = ! empty( $coupon_options['email_restrictions'] ) ? [ $gc_coupon->get_to_email() ] : [];
			$expiry               = $gc_coupon->get_expiry_date();
			$expiry               = $expiry ? $expiry->getOffsetTimestamp() : null;
			$excluded_product_ids = (array) ( $coupon_options['excl_products'] ?? [] );
			$excluded_product_ids = array_merge( $excl_gc_products, $excluded_product_ids );

			return [
				'usage_limit'                 => 1,
				'usage_limit_per_user'        => 1,
				'discount_type'               => 'fixed_cart',
				'amount'                      => $gc_coupon->get_amount(),
				'free_shipping'               => ! empty( $coupon_options['allow_shipping'] ?? '' ),
				'product_categories'          => (array) ( $coupon_options['categories'] ?? [] ),
				'excluded_product_categories' => (array) ( $coupon_options['excl_categories'] ?? [] ),
				'product_ids'                 => (array) ( $coupon_options['products'] ?? [] ),
				'excluded_product_ids'        => $excluded_product_ids,
				'minimum_amount'              => $coupon_options['minimum_spend'] ?? '',
				'maximum_amount'              => $coupon_options['maximum_spend'] ?? '',
				'email_restrictions'          => $email_restrictions,
				'used_by'                     => array(),
				'date_expires'                => $expiry,
				'date_created'                => null,
				'date_modified'               => null,
			];
		}

		return $status;
	}

	public function update_coupon_usage_counts( $order_id ) {
		$order = wc_get_order( $order_id );

		if ( ! $order ) {
			return;
		}

		$has_recorded = $order->get_data_store()->get_recorded_coupon_usage_counts( $order );

		if ( $order->has_status( 'cancelled' ) && $has_recorded ) {
			$action = 'reduce';
		} elseif ( ! $order->has_status( 'cancelled' ) && ! $has_recorded ) {
			$action = 'increase';
		} elseif ( $order->has_status( 'cancelled' ) ) {
			return;
		} else {
			return;
		}


		if ( count( $order->get_coupon_codes() ) > 0 ) {
			foreach ( $order->get_coupon_codes() as $code ) {
				if ( ! $code ) {
					continue;
				}

				$coupon = Utils::get_giftcard_coupon_by_code( $code );

				if ( ! $coupon ) {
					continue;
				}

				$used_by = $order->get_user_id();

				if ( ! $used_by ) {
					$used_by = $order->get_billing_email();
				}

				switch ( $action ) {
					case 'reduce':
						$coupon->decrease_usage_count( $used_by );
						break;
					case 'increase':
						$coupon->increase_usage_count( $used_by, $order );
						break;
				}
			}
		}
	}

	public function apply_coupon_via_comeback_url() {
		if ( ! Utils::is_request_frontend() ) {
			return;
		}

		if ( ! empty( $_GET['gift4u_coupon'] ) ) {
			$coupon = wc_sanitize_coupon_code( wp_unslash( $_GET['gift4u_coupon'] ) );
			wc()->cart->apply_coupon( $coupon );
		}
	}

	public function edit_item_button( $cart_item, $cart_item_key ) {
		$product = $cart_item['data'];

		if ( $product->get_type() !== 'giftcard' ) {
			return;
		}

		$product_permalink = apply_filters( 'woocommerce_cart_item_permalink', $product->is_visible() ? $product->get_permalink( $cart_item ) : '', $cart_item, $cart_item_key );
		if ( $product_permalink ) {
			$url = add_query_arg( [ 'gift4u-edit-item' => $cart_item_key ], $product_permalink );
			printf( "<div><a href='%s'>%s</a></div>", esc_url( $url ), esc_html__( 'Edit', 'gift4u-gift-cards-all-in-one-for-woo' ) );
		}
	}

	public function ship_to_different_address_checkbox( $checked ) {
		$billing  = wc()->customer->get_billing();
		$shipping = wc()->customer->get_shipping();

		$c_billing  = [];
		$c_shipping = [];

		$shipping_fields = WC()->checkout()->get_checkout_fields( 'shipping' );
		if ( ! empty( $shipping_fields ) ) {
			foreach ( $shipping_fields as $key => $value ) {
				$key = str_replace( 'shipping_', '', $key );
				if ( isset( $shipping[ $key ] ) ) {
					$c_shipping[ $key ] = $shipping[ $key ];
					$c_billing[ $key ]  = $billing[ $key ] ?? '';
				}
			}
		}

		if ( wp_json_encode( $c_billing ) !== wp_json_encode( $c_shipping ) ) {
			return true;
		}

		return $checked;
	}
}
