<?php

namespace Gift4U\Front;

use Gift4U\Inc\Data;
use Gift4U\Inc\Utils;

defined( 'ABSPATH' ) || exit;

class Product_Summary {
	protected static $instance = null;
	protected $edit_data;

	public function __construct() {
		add_action( 'woocommerce_giftcard_add_to_cart', 'woocommerce_simple_add_to_cart' );
		add_filter( 'woocommerce_product_single_add_to_cart_text', [ $this, 'change_atc_text_to_update_cart' ], 10, 2 );
		add_action( 'woocommerce_before_add_to_cart_button', [ $this, 'before_add_to_cart_button' ] );
		add_action( 'gift4u_giftcard_form_start', [ $this, 'input_giftcard_amount' ] );
		add_action( 'wp_footer', [ $this, 'email_design_modal' ] );

		add_action( 'wp_ajax_gift4u_load_more_gallery', [ $this, 'load_more_gallery' ] );
		add_action( 'wp_ajax_nopriv_gift4u_load_more_gallery', [ $this, 'load_more_gallery' ] );


		add_action( 'woocommerce_cart_loaded_from_session', [ $this, 'set_edit_cart_item_data' ] );
	}

	public static function instance() {
		return self::$instance == null ? self::$instance = new self() : self::$instance;
	}

	public function set_edit_cart_item_data() {
		if ( ! empty( $_GET['gift4u-edit-item'] ) ) {
			$cart_item_key                    = sanitize_text_field( wp_unslash( $_GET['gift4u-edit-item'] ) );
			$item                             = wc()->cart->get_cart_item( $cart_item_key );
			$this->edit_data                  = $item['gift4u_data'] ?? [];
			$this->edit_data['cart_item_key'] = $cart_item_key;
		}
	}

	public function before_add_to_cart_button() {
		global $product;
		if ( $product->get_type() !== 'giftcard' ) {
			return;
		}

		$image_ids     = [];
		$galleries     = $product->get_meta( '_gift4u_galleries' );
		$show_see_more = false;

		if ( ! empty( $galleries ) && is_array( $galleries ) ) {
			$gallery_id    = current( $galleries );
			$image_ids     = get_post_meta( $gallery_id, '_gift4u_image_ids', true );
			$show_see_more = count( $galleries ) > 1;

			$img_selected = absint( $this->edit_data['email_design'] ?? '' );
			if ( $img_selected && ! in_array( $img_selected, $image_ids ) ) {
				array_unshift( $image_ids, $img_selected );
			}
		}

		$add_card_pdf_to_email = Data::get_param( 'add_card_pdf_to_email' );
		$delivery_method_keys  = $product->get_delivery_method();
		$edit_data             = $this->edit_data;

		if ( empty( $this->edit_data ) ) {
			$edit_data['recipient_emails'] = ! empty( $_REQUEST['gift4u_recipient_email'] ) ? sanitize_text_field( wp_unslash( $_REQUEST['gift4u_recipient_email'] ) ) : '';
			$edit_data['delivery_date']    = ! empty( $_REQUEST['gift4u_delivery_date'] ) ? sanitize_text_field( wp_unslash( $_REQUEST['gift4u_delivery_date'] ) ) : '';
			$edit_data['sender_name']      = ! empty( $_REQUEST['gift4u_sender_name'] ) ? sanitize_text_field( wp_unslash( $_REQUEST['gift4u_sender_name'] ) ) : '';
			$edit_data['recipient_name']   = ! empty( $_REQUEST['gift4u_recipient_name'] ) ? sanitize_text_field( wp_unslash( $_REQUEST['gift4u_recipient_name'] ) ) : '';
			$edit_data['message']          = ! empty( $_REQUEST['gift4u_message'] ) ? sanitize_textarea_field( wp_unslash( $_REQUEST['gift4u_message'] ) ) : '';
			$edit_data['email_design']     = ! empty( $_REQUEST['gift4u_email_design'] ) ? absint( wp_unslash( $_REQUEST['gift4u_email_design'] ) ) : '';
			$edit_data['card_template']     = ! empty( $_REQUEST['gift4u_card_template'] ) ? absint( wp_unslash( $_REQUEST['gift4u_card_template'] ) ) : '';
		}

		Utils::get_template( 'product-details', [
			'image_ids'             => $image_ids,
			'show_see_more'         => $show_see_more,
			'add_card_pdf_to_email' => $add_card_pdf_to_email,
			'delivery_methods'      => array_intersect_key( Utils::delivery_methods_list(), array_flip( $delivery_method_keys ) ),
			'delivery_method_keys'  => $delivery_method_keys,
			'edit_data'             => $edit_data,
		] );
	}

	public function email_design_modal() {

		if ( ! is_product() ) {
			return;
		}

		global $product;

		if ( $product->get_type() !== 'giftcard' ) {
			return;
		}

		?>
        <div id='gift4u-modal-frame'>
            <div class='gift4u-modal-overlay'></div>
            <div class='gift4u-modal-content-wrapper'>
                <div class='gift4u-modal-close'>×</div>
                <div class='gift4u-modal-content-inner'></div>
            </div>
        </div>
		<?php
	}

	public function load_more_gallery() {
		check_ajax_referer( 'gift4u_ajax_nonce', 'security' );

		if ( empty( $_POST['pid'] ) ) {
			wp_send_json_error();
		}

		$pid     = absint( $_POST['pid'] );
		$product = wc_get_product( $pid );

		if ( ! $product->is_type( 'giftcard' ) ) {
			wp_send_json_error();
		}

		$gallery_ids = $product->get_meta( '_gift4u_galleries' );

		$galleries = get_posts( [
			'numberposts' => - 1,
			'post_type'   => 'gift4u_gallery',
			'include'     => $gallery_ids
		] );

		if ( empty( $galleries ) ) {
			wp_send_json_error();
		}

		$titles = [];
		$images = [];

		foreach ( $galleries as $gallery ) {
			$g_id      = $gallery->ID;
			$image_ids = get_post_meta( $g_id, '_gift4u_image_ids', true );

			if ( empty( $image_ids ) || ! is_array( $image_ids ) ) {
				continue;
			}

			$img_tmp = [];
			foreach ( $image_ids as $image_id ) {
				$img_tmp[ $image_id ] = wp_get_attachment_image( $image_id, 'full' );
			}

			$titles[ $g_id ] = $gallery->post_title;
			$images[ $g_id ] = $img_tmp;
		}

		ob_start();
		Utils::get_template( 'modal-image-list', [ 'galleries' => $images, 'titles' => $titles ] );
		$modal = ob_get_clean();

		wp_send_json_success( $modal );
	}

	/**
	 * @param $product \WC_Product_Giftcard
	 */
	public function input_giftcard_amount( $product ) {
		if ( ! $product->is_type( 'giftcard' ) ) {
			return;
		}

		$price_type  = $product->get_price_type();

		switch ( $price_type ) {
			case 'input':
				$min = $product->get_user_input_min_price();
				$max = $product->get_user_input_max_price();

				if ( ! $min || $min < 0 ) {
					$min = 0;
				}

				$discount_rules = $product->get_user_input_discount_rule();

				$localize = [
					'isRTL'             => is_rtl(),
					'currencySymbol'    => get_woocommerce_currency_symbol(),
					'currencyPrecision' => wc_get_price_decimals(),
					'thousandsSep'      => wc_get_price_thousand_separator(),
					'decimalSep'        => wc_get_price_decimal_separator(),
					'priceFormat'       => get_woocommerce_price_format(),
					'rules'             => $discount_rules,
					'discount_umpire'   => Data::get_param( 'discount_umpire' ),
				];

				wp_enqueue_script( 'gift4u-giftcard-footer-script' );
				wp_localize_script( 'gift4u-giftcard-footer-script', 'gift4uProductParams', $localize );

				?>
                <div class="gift4u-row">
                    <label for="">
						<?php esc_html_e( 'Input amount', 'gift4u-gift-cards-all-in-one-for-woo' ); ?>
                    </label>
                    <input type="number" required value="<?php echo esc_attr( $this->edit_data['coupon_amount'] ?? '' ) ?>"
                           name="gift4u_coupon_amount" class="gift4u-input gift4u-user-input-amount"
                           min="<?php echo esc_attr( $min ) ?>" max="<?php echo esc_attr( $max ) ?>">

					<?php
					if ( $product->is_on_sale() && ! empty( $discount_rules ) ) {
						?>
                        <div>
							<?php esc_html_e( 'You will purchase:', 'gift4u-gift-cards-all-in-one-for-woo' ); ?>
                            <span class="gift4u-purchase-amount"> </span>
                        </div>
                        <div>
                            <table class="gift4u-discount-rules">
                                <thead>
                                <tr>
                                    <th><?php esc_html_e( 'From', 'gift4u-gift-cards-all-in-one-for-woo' ); ?></th>
                                    <th><?php esc_html_e( 'To', 'gift4u-gift-cards-all-in-one-for-woo' ); ?></th>
                                    <th><?php esc_html_e( 'Discount', 'gift4u-gift-cards-all-in-one-for-woo' ); ?></th>
                                </tr>
                                </thead>
                                <tbody>
								<?php
								foreach ( $discount_rules as $rule ) {
									if ( empty( $rule['discount_value'] ) || empty( $rule['discount_type'] ) ) {
										continue;
									}

									$from_price      = $rule['from_price'] !== '' ? wc_price( floatval( $rule['from_price'] ) ) : '';
									$to_price        = $rule['to_price'] !== '' ? wc_price( floatval( $rule['to_price'] ) ) : '';
									$discount_amount = floatval( $rule['discount_value'] );
									$discount_amount = $rule['discount_type'] == 'fixed' ? wc_price( $discount_amount ) : $discount_amount . '%';
									printf( "<tr><td>%s</td><td>%s</td><td>%s</td></tr>", $from_price, $to_price, $discount_amount );// phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
								}
								?>
                                </tbody>
                            </table>
                        </div>
						<?php
					}
					?>
                </div>
				<?php
				break;
		}
	}

	public function change_atc_text_to_update_cart( $text, $product ) {
		if ( ! empty( $this->edit_data ) ) {
			$text = esc_html__( 'Update', 'gift4u-gift-cards-all-in-one-for-woo' );
		}

		return $text;
	}
}
