<?php
/**
 * Plugin Name: GIFT4U - Gift Cards All in One for Woo
 * Plugin URI: https://villatheme.com/extensions/gift4u-gift-cards-all-in-one-for-woo/
 * Description: Unlock the potential of gift card products and see how they attract customers. Easily create, sell, and manage a WooCommerce gift card with a few simple steps.
 * Author: VillaTheme
 * Version: 1.0.5
 * Author URI: http://villatheme.com
 * Text Domain: gift4u-gift-cards-all-in-one-for-woo
 * Domain Path: /languages
 * Copyright 2023 VillaTheme.com. All rights reserved.
 * Requires at least: 5.0
 * Tested up to: 6.4
 * WC requires at least: 5.0
 * WC tested up to: 8.4
 * Requires PHP: 7.0
 */

namespace Gift4U;

use Gift4U\Inc\Activation;
use Gift4U\Inc\Load_Classes;

defined( 'ABSPATH' ) || exit;

require_once plugin_dir_path( __FILE__ ) . 'autoload.php';

class GiftCard {
	private static $args = [];

	public function __construct() {
		add_action( 'plugins_loaded', [ $this, 'init' ] );
		register_activation_hook( __FILE__, [ $this, 'active' ] );
		add_action( 'before_woocommerce_init', [ $this, 'custom_order_tables_declare_compatibility' ] );
	}

	public static function get( $key ) {
		if ( ! self::$args ) {
			global $wpdb;
			$assets_url = plugins_url( 'assets/', __FILE__ );

			self::$args = [
				'version'       => '1.0.5',
				'plugin_name'   => 'GIFT4U - Gift Cards All in One for Woo',
				'slug'          => 'gift4u-gift-cards-all-in-one-for-woo',
				'file'          => __FILE__,
				'basename'      => plugin_basename( __FILE__ ),
				'plugin_dir'    => plugin_dir_path( __FILE__ ),
				'templates_dir' => plugin_dir_path( __FILE__ ) . 'templates/',
				'js_url'        => $assets_url . 'js/',
				'css_url'       => $assets_url . 'css/',
				'img_url'       => $assets_url . 'img/',
				'libs_url'      => $assets_url . 'libs/',
				'db_table'      => $wpdb->prefix . 'gift4u_giftcard_coupons',
			];
		}

		return self::$args[ $key ] ?? '';
	}

	public function init() {
		if ( ! class_exists( 'VillaTheme_Require_Environment' ) ) {
			include_once GiftCard::get( 'plugin_dir' ) . 'support/support.php';
		}

		$environment = new \VillaTheme_Require_Environment( [
				'plugin_name'     => GiftCard::get( 'plugin_name' ),
				'php_version'     => '7.0',
				'wp_version'      => '5.0',
				'wc_version'      => '5.0',
				'require_plugins' => [
					[
						'slug' => 'woocommerce',
						'name' => 'WooCommerce',
					],
				]
			]
		);

		if ( $environment->has_error() ) {
			return;
		}

		add_filter( 'plugin_action_links_' . GiftCard::get( 'basename' ), [ $this, 'setting_link' ] );

		$this->load_text_domain();
		Load_Classes::instance();
	}

	public function active( $network_wide ) {
		$activation = new Activation();
		$activation->install( $network_wide );
	}

	public function setting_link( $links ) {
		$settings_link = [
			sprintf( "<a href='%1s' >%2s</a>", esc_url( admin_url( 'admin.php?page=gift4u-settings' ) ), esc_html__( 'Settings', 'gift4u-gift-cards-all-in-one-for-woo' ) )
		];

		return array_merge( $settings_link, $links );
	}

	public function load_text_domain() {
		$locale   = determine_locale();
		$locale   = apply_filters( 'plugin_locale', $locale, 'gift4u-gift-cards-all-in-one-for-woo' );
		$basename = plugin_basename( dirname( __FILE__ ) );

		unload_textdomain( 'gift4u-gift-cards-all-in-one-for-woo' );

		load_textdomain( 'gift4u-gift-cards-all-in-one-for-woo', WP_LANG_DIR . "/{$basename}/{$basename}-{$locale}.mo" );
		load_plugin_textdomain( 'gift4u-gift-cards-all-in-one-for-woo', false, $basename . '/languages' );
	}

	public function custom_order_tables_declare_compatibility() {
		if ( class_exists( \Automattic\WooCommerce\Utilities\FeaturesUtil::class ) ) {
			\Automattic\WooCommerce\Utilities\FeaturesUtil::declare_compatibility( 'custom_order_tables', __FILE__, true );
		}
	}
}

new GiftCard();








