<?php

namespace Gift4U\Inc;

use Gift4U\GiftCard;

defined( 'ABSPATH' ) || exit;

class Activation {

	public function install( $network_wide ) {
		$this->add_term();

		global $wpdb;
		if ( function_exists( 'is_multisite' ) && is_multisite() && $network_wide ) {
			$current_blog = $wpdb->blogid;
			$blogs        = $wpdb->get_col( "SELECT blog_id FROM {$wpdb->blogs}" );
			//Multi site activate action
			foreach ( $blogs as $blog ) {
				switch_to_blog( $blog );
				$this->create_table();
			}
			switch_to_blog( $current_blog );
		} else {
			//Single site activate action
			$this->create_table();
		}

		$endpoint = Data::get_param( 'myaccount_endpoint' );
		add_rewrite_endpoint( $endpoint, EP_ROOT | EP_PAGES );
		flush_rewrite_rules();

		if ( ! get_option( 'gift4u_data' ) ) {
			if ( ! class_exists( 'WooCommerce' ) ) {
				return;
			}

			add_action( 'activated_plugin', [ $this, 'after_activated' ] );
		}
	}

	public function add_term() {
		if ( ! get_term_by( 'slug', 'giftcard', 'product_type' ) ) {
			wp_insert_term( 'giftcard', 'product_type' );
		}
	}

	public function create_table() {
		if ( ! function_exists( 'dbDelta' ) ) {
			require_once ABSPATH . 'wp-admin/includes/upgrade.php';
		}

		global $wpdb;

		$collate = $wpdb->has_cap( 'collation' ) ? $wpdb->get_charset_collate() : '';
		$table   = $wpdb->prefix . 'gift4u_giftcard_coupons';

		$query = "CREATE TABLE IF NOT EXISTS {$table}
					(
			          	id              bigint unsigned auto_increment	primary key,
					    date_created    datetime                      	null,
					    coupon_code     varchar(10)                   	null,
					    amount          tinytext                      	not null,
					    order_id        bigint unsigned               	null,
					    delivery_status varchar(10) default 'pending' 	not null,
					    sender_id       bigint      default 0          	null,
					    from_email      varchar(100)                  	null,
					    to_email        varchar(100)                  	null,
					    delivery_method varchar(20)                   	not null,
					    currency        varchar(5)                    	null,
					    quantity        smallint    default 0           null,
					    usage_count     smallint    default 0         	null,
					    delivery_date   int                           	null,
					    expiry_date     int                           	null,
					    coupon_status   varchar(20) default 'publish' 	not null,
					    meta_data       longtext                      	null
					)
					{$collate}";

		dbDelta( $query );
	}

	public function after_activated( $plugin ) {
		if ( $plugin === GiftCard::get( 'basename' ) ) {
			$url = add_query_arg( [ 'page' => 'gift4u-setup' ], admin_url() );
			wp_safe_redirect( $url );
			exit();
		}
	}

}

