<?php

namespace Gift4U\Inc\Classes;

use Gift4U\Inc\Data;
use Gift4U\Inc\Utils;

defined( 'ABSPATH' ) || exit;

class Email {
	protected $content;

	protected $gc_coupon;

	/**
	 * Email constructor.
	 *
	 * @param $gc_coupon Giftcard_Coupon
	 */
	public function __construct( $gc_coupon ) {
		$this->gc_coupon = $gc_coupon;

		add_action( 'gift4u_email_content', [ $this, 'email_content' ] );
	}

	public function replace_placeholder( $text, $strip_tags = false ) {
		$meta         = $this->gc_coupon->get_meta_data();
		$email_design = $meta['email_design'] ?? '';
		$image        = $email_design ? wp_get_attachment_image( $email_design, false ) : '';

		return Utils::replace_placeholder( $text, $this->gc_coupon, [ '{selected_image}' => $image ], $strip_tags );
	}

	public function email_content() {
		echo do_shortcode( wp_kses_post( wpautop( wptexturize( $this->replace_placeholder( Data::get_param( 'email_content' ) ) ) ) ) );
	}

	public function get_email_content() {
		ob_start();
		Utils::get_template( 'email-template' );

		return ob_get_clean();
	}

	public function generate_card_pdf() {
		$pdf = new PDF( $this->gc_coupon );

		return $pdf->render();
	}


	public function send_email( $email = '' ) {
		if ( ! $this->gc_coupon ) {
			return false;
		}

		$add_card_pdf_to_email = Data::get_param( 'add_card_pdf_to_email' );

		$subject = $this->replace_placeholder( Data::get_param( 'email_subject' ), true );

		$headers = [ "Content-Type: text/html" ];

		if ( Data::get_param( 'bcc_to_sender' ) ) {
			$from_email = $this->gc_coupon->get_from_email();
			$headers[]  = "Bcc: {$from_email}";
		}

		$expiry_date = $this->gc_coupon->get_expiry_date();

		if ( ! $expiry_date ) {
			$coupon_options = $this->gc_coupon->get_coupon_options();

			if ( ! empty( $coupon_options['expiry_date'] ) ) {
				$date = strtotime( "+{$coupon_options['expiry_date']} day" );

				$this->gc_coupon->set_expiry_date( $date );
			}
		}

		$attachments = [];
		$file        = $add_card_pdf_to_email ? $this->generate_card_pdf() : '';

		if ( $file ) {
			$attachments[] = $file;
		}

		$email = $email ? $email : $this->gc_coupon->get_to_email();

		$result = wp_mail( $email, $subject, $this->get_email_content(), $headers, $attachments );

		if ( $result ) {
			$this->gc_coupon->set_delivery_status( 'sent' );
			$this->gc_coupon->set_delivery_date( time() );
		}

		$this->gc_coupon->save();

		if ( $file ) {
			unlink( $file );
		}

		return $result;
	}
}
