<?php

namespace Gift4U\Inc\Classes;

use Gift4U\GiftCard;

defined( 'ABSPATH' ) || exit;

class Giftcard_Coupon_Data_Store {
	protected $table;

	public function __construct() {
		$this->table = GiftCard::get( 'db_table' );
	}

	/**
	 * @param $gc_coupon Giftcard_Coupon
	 */
	public function read( &$gc_coupon ) {
		global $wpdb;
		$id     = $gc_coupon->get_id();
		$query  = "select * from {$this->table} where id = %s";
		$result = $wpdb->get_row( $wpdb->prepare( $query, $id ), ARRAY_A );

		if ( ! empty( $result ) ) {
			$gc_coupon->set_props( $result );
		}
	}

	/**
	 * @param $gc_coupon Giftcard_Coupon
	 *
	 * @return int
	 */
	public function create( &$gc_coupon ) {
		global $wpdb;
		$coupon_code = $gc_coupon->get_coupon_code();

		if ( ! $coupon_code ) {
			$coupon_code = $this->create_coupon_code();
			$gc_coupon->set_coupon_code( $coupon_code );
		}

		if ( ! $gc_coupon->get_date_created( 'edit' ) ) {
			$gc_coupon->set_date_created( time() );
		}

		$data = [
			'date_created'    => gmdate( 'Y-m-d H:i:s', $gc_coupon->get_date_created( 'edit' )->getOffsetTimestamp() ),
			'coupon_code'     => $coupon_code,
			'delivery_status' => $gc_coupon->get_delivery_status(),
			'sender_id'       => $gc_coupon->get_sender_id(),
			'from_email'      => $gc_coupon->get_from_email(),
			'to_email'        => $gc_coupon->get_to_email(),
			'order_id'        => $gc_coupon->get_order_id(),
			'amount'          => $gc_coupon->get_amount(),
			'delivery_date'   => $gc_coupon->get_delivery_date() instanceof \WC_DateTime ? $gc_coupon->get_delivery_date()->getTimestamp() : '',
			'delivery_method' => $gc_coupon->get_delivery_method(),
			'currency'        => $gc_coupon->get_currency(),
			'meta_data'       => wp_json_encode( $gc_coupon->get_meta_data() ),
			'expiry_date'     => $gc_coupon->get_expiry_date() instanceof \WC_DateTime ? $gc_coupon->get_expiry_date()->getTimestamp() : '',
			'quantity'        => 1,
		];

		$wpdb->insert( $this->table, $data, [ '%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s', '%d' ] );

		return $wpdb->insert_id;
	}

	/**
	 * @param $gc_coupon Giftcard_Coupon
	 */
	public function update( &$gc_coupon ) {
		global $wpdb;

		$wpdb->update( $this->table, [
			'coupon_code'     => $gc_coupon->get_coupon_code(),
			'delivery_status' => $gc_coupon->get_delivery_status(),
			'sender_id'       => $gc_coupon->get_sender_id(),
			'from_email'      => $gc_coupon->get_from_email(),
			'to_email'        => $gc_coupon->get_to_email(),
			'order_id'        => $gc_coupon->get_order_id(),
			'amount'          => $gc_coupon->get_amount(),
			'delivery_date'   => $gc_coupon->get_delivery_date() instanceof \WC_DateTime ? $gc_coupon->get_delivery_date()->getTimestamp() : null,
			'delivery_method' => $gc_coupon->get_delivery_method(),
			'currency'        => $gc_coupon->get_currency(),
			'meta_data'       => wp_json_encode( $gc_coupon->get_meta_data() ),
			'expiry_date'     => $gc_coupon->get_expiry_date() instanceof \WC_DateTime ? $gc_coupon->get_expiry_date()->getTimestamp() : null,
		], [ 'id' => $gc_coupon->get_id() ], [ '%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s' ], [ '%d' ] );
	}

	protected function create_coupon_code() {
		$arr_char = array_merge( range( 0, 9 ), range( 'a', 'z' ) );
		$code     = '';

		for ( $i = 0; $i < 6; $i ++ ) {
			$code .= $arr_char[ wp_rand( 0, count( $arr_char ) - 1 ) ];
		}

		if ( $this->find_row_by_coupon_code( $code ) ) {
			return $this->create_coupon_code();
		}

		return $code;
	}

	protected function find_row_by_coupon_code( $code ) {
		global $wpdb;
		$query = "select coupon_code from {$this->table} where coupon_code='%s'; ";

		return $wpdb->get_var( $wpdb->prepare( $query, $code ) );
	}

	/**
	 * @param $coupon Giftcard_Coupon
	 * @param $used_by
	 * @param $order
	 */
	public function increase_usage_count( &$coupon, $used_by, $order ) {
		$new_count = $this->update_usage_count_meta( $coupon, 'increase' );
		$coupon->set_usage_count( $new_count );
	}

	/**
	 * @param $coupon Giftcard_Coupon
	 * @param $used_by
	 */
	public function decrease_usage_count( &$coupon, $used_by ) {
		$new_count = $this->update_usage_count_meta( $coupon, 'decrease' );
		$coupon->set_usage_count( $new_count );
	}

	/**
	 * @param $coupon Giftcard_Coupon
	 * @param string $operation
	 *
	 * @return int
	 */
	private function update_usage_count_meta( &$coupon, $operation = 'increase' ) {
		global $wpdb;

		$id       = $coupon->get_id();
		$operator = ( 'increase' === $operation ) ? '+' : '-';

		$query = "UPDATE {$this->table} SET usage_count = usage_count {$operator} 1 WHERE id = %d;";
		// phpcs:ignore WordPress.DB.PreparedSQL.NotPrepared, WordPress.DB.PreparedSQL.InterpolatedNotPrepared
		$wpdb->query( $wpdb->prepare( $query, $id ) );

		return (int) $wpdb->get_var( $wpdb->prepare( "SELECT usage_count FROM {$this->table} WHERE id = %d;", $id ) );
	}
}
