<?php

namespace Gift4U\Inc\Classes;

defined( 'ABSPATH' ) || exit;

class Giftcard_Coupon {
	protected $id;
	protected $data_store;
	protected $data = [
		'coupon_code'     => '',
		'date_created'    => '',
		'amount'          => '',
		'delivery_status' => 'pending',
		'sender_id'       => 0,
		'from_email'      => '',
		'to_email'        => '',
		'order_id'        => '',
		'delivery_date'   => '',
		'delivery_method' => '',
		'currency'        => '',
		'coupon_options'  => '',
		'expiry_date'     => '',
		'meta_data'       => [],
		'usage_count'     => '',
		'quantity'        => '',
		'coupon_status'   => '',
	];

	public function __construct( $giftcard_id = 0 ) {
		$this->id         = $giftcard_id;
		$this->data_store = new Giftcard_Coupon_Data_Store();

		if ( $this->get_id() > 0 ) {
			$this->data_store->read( $this );
		}
	}

	public function set_date_prop( $value ) {
		if ( empty( $value ) ) {
			return null;
		}

		if ( is_a( $value, '\WC_DateTime' ) ) {
			$datetime = $value;
		} elseif ( is_numeric( $value ) ) {
			// Timestamps are handled as UTC timestamps in all cases.
			$datetime = new \WC_DateTime( "@{$value}", new \DateTimeZone( 'UTC' ) );
		} else {
			// Strings are defined in local WP timezone. Convert to UTC.
			if ( 1 === preg_match( '/^(\d{4})-(\d{2})-(\d{2})T(\d{2}):(\d{2}):(\d{2})(Z|((-|\+)\d{2}:\d{2}))$/', $value, $date_bits ) ) {
				$offset    = ! empty( $date_bits[7] ) ? iso8601_timezone_to_offset( $date_bits[7] ) : wc_timezone_offset();
				$timestamp = gmmktime( $date_bits[4], $date_bits[5], $date_bits[6], $date_bits[2], $date_bits[3], $date_bits[1] ) - $offset;
			} else {
				$timestamp = wc_string_to_timestamp( get_gmt_from_date( gmdate( 'Y-m-d H:i:s', wc_string_to_timestamp( $value ) ) ) );
			}
			$datetime = new \WC_DateTime( "@{$timestamp}", new \DateTimeZone( 'UTC' ) );
		}

		// Set local timezone or offset.
		if ( get_option( 'timezone_string' ) ) {
			$datetime->setTimezone( new \DateTimeZone( wc_timezone_string() ) );
		} else {
			$datetime->set_utc_offset( wc_timezone_offset() );
		}

		return $datetime;
	}

	public function set_id( $value ) {
		$this->id = $value;
	}

	public function set_coupon_code( $value ) {
		$this->data['coupon_code'] = $value;
	}

	public function set_coupon_amount( $value ) {
		$this->data['amount'] = $value;
	}

	public function set_delivery_status( $value ) {
		$this->data['delivery_status'] = $value;
	}

	public function set_sender_id( $value = 0 ) {
		$this->data['sender_id'] = $value;
	}

	public function set_from_email( $value ) {
		$this->data['from_email'] = $value;
	}

	public function set_to_email( $value ) {
		$this->data['to_email'] = $value;
	}

	public function set_order_id( $value ) {
		$this->data['order_id'] = $value;
	}

	public function set_delivery_date( $value ) {
		try {
			$this->data['delivery_date'] = $this->set_date_prop( $value );
		} catch ( \Exception $e ) {
		} // @codingStandardsIgnoreLine.
	}

	public function set_delivery_method( $value ) {
		$this->data['delivery_method'] = $value;
	}

	public function set_coupon_options( $value ) {
		$this->data['coupon_options']              = $value;
		$this->data['meta_data']['coupon_options'] = $value;
	}

	public function set_expiry_date( $value ) {
		$this->data['expiry_date'] = $this->set_date_prop( $value );
	}

	public function set_currency( $value ) {
		if ( ! $value ) {
			$value = get_woocommerce_currency();
		}
		$this->data['currency'] = $value;
	}

	public function set_date_created( $value ) {
		try {
			$this->data['date_created'] = $this->set_date_prop( $value );
		} catch ( \Exception $e ) {
		} // @codingStandardsIgnoreLine.
	}

	public function set_props( $props ) {
		$props = (array) $props;

		$meta_data = json_decode( $props['meta_data'], true );

		$props['meta_data']      = $meta_data;
		$props['coupon_options'] = $meta_data['coupon_options'] ?? [];

		$this->data = wp_parse_args( $props, $this->data );

		$this->set_delivery_date( $this->data['delivery_date'] );
		$this->set_date_created( $this->data['date_created'] );
		$this->set_expiry_date( $this->data['expiry_date'] );
		$this->set_currency( $this->data['currency'] );
	}

	public function set_meta_data( $value ) {
		$this->data['meta_data'] = $value;
	}

	public function set_usage_count( $value ) {
		$this->data['usage_count'] = $value;
	}

	public function set_quantity( $value ) {
		$this->data['quantity'] = $value;
	}

	public function set_single_meta_data( $key, $value ) {
		$this->data['meta_data'][ $key ] = $value;
	}

	public function get_id() {
		return $this->id;
	}

	public function get_coupon_code() {
		return $this->data['coupon_code'] ?? '';
	}

	public function get_amount() {
		return floatval( $this->data['amount'] ?? '' );
	}

	public function get_delivery_status() {
		return $this->data['delivery_status'] ?? '';
	}

	public function get_sender_id() {
		return $this->data['sender_id'] ?? 0;
	}

	public function get_from_email() {
		return $this->data['from_email'] ?? '';
	}

	public function get_to_email() {
		return $this->data['to_email'] ?? '';
	}

	public function get_order_id() {
		return $this->data['order_id'] ?? '';
	}

	public function get_delivery_date() {
		return $this->data['delivery_date'] ?? '';
	}

	public function get_delivery_method() {
		return $this->data['delivery_method'] ?? '';
	}

	public function get_coupon_options() {
		return (array) $this->get_single_meta_data( 'coupon_options' );
	}

	public function get_expiry_date() {
		return $this->data['expiry_date'] ?? null;
	}

	public function get_currency() {
		return $this->data['currency'] ?? '';
	}

	public function get_date_created( $context = 'view' ) {
		return $this->data['date_created'] ?? '';
	}

	public function get_meta_data() {
		return $this->data['meta_data'] ?? '';
	}

	public function get_usage_count() {
		return $this->data['usage_count'] ?? '';
	}

	public function get_quantity() {
		return $this->data['quantity'] ?? '';
	}

	public function get_single_meta_data( $key ) {
		return $this->get_meta_data()[ $key ] ?? '';
	}

	public function get_coupon_status() {
		return $this->data['coupon_status'] ?? '';
	}

	public function save() {
		if ( $this->id ) {
			$this->data_store->update( $this );
		} else {
			$this->id = $this->data_store->create( $this );
		}

		return $this->id;
	}

	public function increase_usage_count( $used_by = '', $order = null ) {
		if ( $this->get_id() && $this->data_store ) {
			$this->data_store->increase_usage_count( $this, $used_by, $order );
		}
	}

	public function decrease_usage_count( $used_by = '' ) {
		if ( $this->get_id() && $this->get_usage_count() > 0 && $this->data_store ) {
			$this->data_store->decrease_usage_count( $this, $used_by );
		}
	}
}
