<?php

namespace Gift4U\Inc\Classes;

use Gift4U\GiftCard;
use Gift4U\Inc\Data;
use Gift4U\Inc\Utils;

defined( 'ABSPATH' ) || exit;

class PDF {
	protected $gc_coupon;
	protected $pdf_tmpl;
	protected $mpdf;

	/**
	 * PDF constructor.
	 *
	 * @param $gc_coupon Giftcard_Coupon
	 */
	public function __construct( $gc_coupon ) {
		$this->gc_coupon = $gc_coupon;

		$this->init();
	}

	private function init() {
		if ( ! $this->gc_coupon ) {
			return;
		}

		$meta_data = $this->gc_coupon->get_meta_data();

		if ( empty( $meta_data['card_template'] ) ) {
			return;
		}

		$temp_id        = $meta_data['card_template'];
		$this->pdf_tmpl = get_post_meta( $temp_id, '_gift4u_card_template', true );

		if ( empty( $this->pdf_tmpl ) ) {
			return;
		}

		include_once GiftCard::get( 'plugin_dir' ) . '/vendor/autoload.php';

		$this->mpdf = new \Mpdf\Mpdf( [
			'tempDir'       => WP_CONTENT_DIR . '/cache',
			'mode'          => 'utf-8',
			'format'        => [ $this->pdf_tmpl->width, $this->pdf_tmpl->height ],
			'margin_left'   => $this->pdf_tmpl->margin->left,
			'margin_top'    => $this->pdf_tmpl->margin->top,
			'margin_bottom' => $this->pdf_tmpl->margin->bottom,
			'margin_right'  => $this->pdf_tmpl->margin->right,
			'fontdata'      => [
				'opensan' =>
					[
						'R'          => 'OpenSans-Regular.ttf',
						'B'          => 'OpenSans-Bold.ttf',
						'I'          => 'OpenSans-Italic.ttf',
						'BI'         => 'OpenSans-BoldItalic.ttf',
						'useOTL'     => 255,
						'useKashida' => 75,
					]
			],
			'default_font'  => 'opensan'
		] );
	}

	public function render() {
		if ( ! $this->mpdf ) {
			return false;
		}

		$coupon_code = $this->gc_coupon->get_coupon_code();

		$pdf_filename = Data::get_param( 'pdf_file_name' );
		$pdf_filename = str_replace( [ ' ', '{coupon_code}' ], [ '', $coupon_code ], $pdf_filename );

		$file_path = $pdf_filename ? $pdf_filename : 'gift4u' . current_time( 'U' );
		$file_path .= '.pdf';

		try {
			$html = $this->get_pdf_html();

			ob_start();
			?>
            body {
            font-family: serif;
            }
			<?php
			$print_style = ob_get_clean();

			$this->mpdf->WriteHTML( $print_style, 1 );
			$this->mpdf->WriteHTML( $html, 2 );
			$this->mpdf->Output( $file_path, 'F' );

			return $file_path;

		} catch ( \Exception $e ) {
		}

		return false;
	}

	public function get_download() {

		if ( ! $this->mpdf ) {
			return;
		}

		try {
			ob_start();
			?>
            body {
            font-family: serif;
            }
			<?php
			$print_style = ob_get_clean();

			$html        = $this->get_pdf_html();
			$coupon_code = $this->gc_coupon->get_coupon_code();

			$pdf_filename = Data::get_param( 'pdf_file_name' );
			$pdf_filename = str_replace( [ ' ', '{coupon_code}' ], [ '', $coupon_code ], $pdf_filename );
			$file_path    = $pdf_filename ? $pdf_filename : "gift4u_{$coupon_code}";
			$file_path    .= '.pdf';

			$this->mpdf->WriteHTML( $print_style, 1 );
			$this->mpdf->WriteHTML( $html, 2 );
			$this->mpdf->Output( $file_path, 'D' );
		} catch ( \Exception $e ) {
		}
	}

	public function get_pdf_html() {
		if ( ! $this->pdf_tmpl ) {
			return '';
		}

		ob_start();

		if ( ! empty( $this->pdf_tmpl->images ) ) {
			foreach ( $this->pdf_tmpl->images as $image ) {
				$top   = ! empty( $image->pos_y ) ? $image->pos_y : 0;
				$left  = ! empty( $image->pos_x ) ? $image->pos_x : 0;
				$width = ! empty( $image->width ) ? $image->width . 'px' : '100%';

				$image_styles = implode( '; ', [
					'position: fixed',
					"top: {$top}",
					"left: {$left}",
					"width: {$width}",
				] );

				printf( "<div style='%s'><img src='%s' style='width: 100%%; vertical-align: middle;' /></div>", esc_attr( $image_styles ), esc_url( $image->url ) );
			}
		}

		if ( ! empty( $this->pdf_tmpl->texts ) ) {
			foreach ( $this->pdf_tmpl->texts as $text ) {
				$top         = ! empty( $text->pos_y ) ? $text->pos_y : 0;
				$left        = ! empty( $text->pos_x ) ? $text->pos_x : 0;
				$color       = ! empty( $text->color ) ? $text->color : '#000000';
				$size        = ! empty( $text->size ) ? $text->size : '16';
				$weight      = ! empty( $text->font_weight ) ? $text->font_weight : 'normal';
				$font_family = ! empty( $text->font_family ) ? $text->font_family : 'arial';
				$align       = ! empty( $text->align ) ? $text->align : 'left';
				$width       = ! empty( $text->width ) ? $text->width . 'mm' : 'auto';

				$styles = implode( '; ', [
					'position: fixed',
					"top: {$top}",
					"left: {$left}",
					"color: {$color}",
					"font-size: {$size}",
					"font-weight: {$weight}",
					"font-family: {$font_family}",
					"text-align: {$align}",
					"width: {$width}",
				] );

				printf( "<div style='%s'>%s</div>", esc_attr( $styles ), wp_kses_post( $text->content ) );
			}
		}

		return Utils::replace_placeholder( ob_get_clean(), $this->gc_coupon, [] );
	}

}
