<?php

use Gift4U\Inc\Utils;
use Gift4U\Inc\Data;

defined( 'ABSPATH' ) || exit;

class WC_Product_Giftcard extends \WC_Product {
	protected $price_type;

	public function get_type() {
		return 'giftcard';
	}

	public function is_purchasable() {
		$price_type = $this->get_price_type();
		$price_cond = false;
		switch ( $price_type ) {
			case 'fixed':
				$price_cond = $this->get_meta( '_gift4u_giftcard_price' );
				break;
			case 'input':
				$price_cond = true;
				break;
		}

		return apply_filters( 'woocommerce_is_purchasable', $this->exists() && ( 'publish' === $this->get_status() || current_user_can( 'edit_post', $this->get_id() ) ) && $price_cond, $this );
	}

	public function set_date_on_sale_from( $date = null ) {
		if ( did_action( 'save_post_product' ) ) {
			if ( isset( $_POST['_gift4u_sale_date_from'] ) ) {
				$date = sanitize_text_field( wp_unslash( $_POST['_gift4u_sale_date_from'] ) );
			}
		}

		$this->set_date_prop( 'date_on_sale_from', $date );
	}

	public function set_date_on_sale_to( $date = null ) {
		if ( did_action( 'save_post_product' ) ) {
			if ( isset( $_POST['_gift4u_sale_date_to'] ) ) {
				$date = sanitize_text_field( wp_unslash( $_POST['_gift4u_sale_date_to'] ) );
			}
		}

		$this->set_date_prop( 'date_on_sale_to', $date );
	}

	public function get_price_type() {
		if ( ! $this->price_type ) {
			$this->price_type = $this->get_meta( '_gift4u_price_type' );
		}

		return $this->price_type;
	}

	public function get_price( $context = 'view' ) {
		$method = 'get_price_' . $this->get_price_type();

		return method_exists( __CLASS__, $method ) ? $this->$method() : $this->get_prop( 'price', $context );
	}

	public function time_in_sale( $context = 'view' ) {
		$on_sale = true;

		if ( $this->get_date_on_sale_from( $context ) && $this->get_date_on_sale_from( $context )->getTimestamp() > time() ) {
			$on_sale = false;
		}

		if ( $this->get_date_on_sale_to( $context ) && $this->get_date_on_sale_to( $context )->getTimestamp() < time() ) {
			$on_sale = false;
		}

		return $on_sale;
	}

	public function is_on_sale( $context = 'view' ) {
		if ( '' !== (string) $this->get_sale_price( $context ) && $this->get_regular_price( $context ) > $this->get_sale_price( $context ) ) {
			$on_sale = $this->time_in_sale( $context );
		} else {
			$on_sale = false;
		}

		return 'view' === $context ? apply_filters( 'woocommerce_product_is_on_sale', $on_sale, $this ) : $on_sale;
	}

	public function get_price_html( $deprecated = '' ) {
		$price      = '';
		$price_type = $this->get_price_type();

		switch ( $price_type ) {
			case 'input':
				$min = $this->get_user_input_min_price();
				$max = $this->get_user_input_max_price();
				$tmp = [];

				if ( $min !== '' ) {
					$tmp['min'] = wc_price( floatval( $min ) );
				}

				if ( $max !== '' ) {
					$tmp['max'] = wc_price( floatval( $max ) );
				}

				if ( ! empty( $tmp ) ) {
					$price = implode( ' - ', $tmp );
				}

				break;

			case 'fixed':
				if ( '' === $this->get_price() ) {
					$price = apply_filters( 'woocommerce_empty_price_html', '', $this );
				} elseif ( $this->is_on_sale() ) {

					$regular_price  = wc_get_price_to_display( $this, array( 'price' => $this->get_regular_price() ) );
					$purchase_price = wc_get_price_to_display( $this );

					$price = sprintf( "%s (%s) %s", is_numeric( $regular_price ) ? wc_price( $regular_price ) : $regular_price,
						is_numeric( $purchase_price ) ? wc_price( $purchase_price ) : $purchase_price, $this->get_price_suffix() );
				} else {
					$price = wc_price( wc_get_price_to_display( $this ) ) . $this->get_price_suffix();
				}
				break;
		}


		return apply_filters( 'woocommerce_get_price_html', $price, $this );
	}

	public function get_user_input_min_price() {
		if ( $this->get_price_type() !== 'input' ) {
			return false;
		}

		return $this->get_meta( '_gift4u_user_input_min_price' );
	}

	public function get_user_input_max_price() {
		if ( $this->get_price_type() !== 'input' ) {
			return false;
		}

		return $this->get_meta( '_gift4u_user_input_max_price' );
	}

	public function get_user_input_discount_rule() {
		if ( $this->get_price_type() !== 'input' ) {
			return false;
		}

		return $this->get_meta( '_gift4u_discount_rules' );
	}

	public function add_to_cart_text() {
//		$text = $this->is_purchasable() && $this->is_in_stock() ? __( 'Add to cart1', 'woocommerce' ) : __( 'Read more', 'woocommerce' );
		$text = esc_html__( 'Select options', 'gift4u-gift-cards-all-in-one-for-woo' );

		return apply_filters( 'woocommerce_product_add_to_cart_text', $text, $this );
	}

	public function add_to_cart_url() {
		return apply_filters( 'woocommerce_product_add_to_cart_url', $this->get_permalink(), $this );
	}

	public function get_delivery_method() {
		$methods = $this->get_meta( '_gift4u_override_methods' );

		if ( empty( $methods ) ) {
			$methods = Data::get_param( 'delivery_methods' );
		}

		if ( empty( $methods ) ) {
			$methods = array_keys( Utils::delivery_methods_list() );
		}

		return $methods;
	}
}
