<?php

namespace Gift4U\Inc;

use Gift4U\GiftCard;

defined( 'ABSPATH' ) || exit;

class Enqueue {
	protected static $instance = null;
	protected $slug = 'gift4u-';
	protected $suffix = WP_DEBUG ? '' : '.min';

	public function __construct() {
		add_action( 'wp_enqueue_scripts', [ $this, 'front_enqueue_scripts' ] );
		add_action( 'admin_enqueue_scripts', [ $this, 'admin_enqueue_scripts' ] );
	}

	public static function instance() {
		return self::$instance == null ? self::$instance = new self() : self::$instance;
	}

	public function register_scripts() {
		$lib_styles = [
			'button',
			'tab',
			'input',
			'icon',
			'segment',
			'image',
			'modal',
			'dimmer',
			'transition',
			'menu',
			'grid',
			'search',
			'message',
			'loader',
			'label',
			'select2',
			'header',
			'accordion',
			'dropdown',
			'checkbox',
			'form',
			'table',
			'toast',
			'popup',
			'step',
			'jquery-ui-helper',
		];

		foreach ( $lib_styles as $style ) {
			wp_register_style( $this->slug . $style, GiftCard::get( 'libs_url' ) . $style . '.min.css', '', GiftCard::get( 'version' ) );
		}

		//*************************************//
		$styles = [ 'setup-wizard', 'settings', 'product-edit', 'gallery', 'giftcard-manage', 'card-template', 'shop-order' ];

		foreach ( $styles as $style ) {
			wp_register_style( $this->slug . $style, GiftCard::get( 'css_url' ) . $style . $this->suffix . '.css', '', GiftCard::get( 'version' ) );
		}

		//*************************************//

		$lib_scripts = [ 'fabric', 'jqColorPicker', 'select2', 'transition', 'dimmer', 'accordion', 'dropdown', 'jquery.address', 'tab', 'modal', 'toast' ];

		foreach ( $lib_scripts as $script ) {
			wp_register_script( $this->slug . $script, GiftCard::get( 'libs_url' ) . $script . '.min.js', [ 'jquery' ], GiftCard::get( 'version' ) );
		}

		//*************************************//

		$scripts = [
			'settings'        => [ 'jquery' ],
			'gallery'         => [ 'jquery' ],
			'card-template'   => [ 'jquery', 'jquery-ui-draggable', 'jquery-ui-resizable', 'jquery-ui-dialog' ],
			'product-edit'    => [ 'jquery' ],
			'giftcard-manage' => [ 'jquery' ],
			'setup-wizard'    => [ 'jquery' ],
		];

		foreach ( $scripts as $script => $depend ) {
			wp_register_script( $this->slug . $script, GiftCard::get( 'js_url' ) . $script . $this->suffix . '.js', $depend, GiftCard::get( 'version' ) );
		}
	}

	public function admin_enqueue_scripts() {

		Utils::set_page( 'product' );
		Utils::set_page( 'shop_order' );
		Utils::set_page( 'gallery', 'gift4u_gallery' );
		Utils::set_page( 'card_template', 'gift4u_tmpl' );

		$screen_id = get_current_screen()->id;
		$pages     = Utils::get_pages();

		if ( ! in_array( $screen_id, $pages ) ) {
			return;
		}

		$this->register_scripts();

		$enqueue_scripts = $enqueue_styles = [];
		$localize_script = $inline_handle = $css = '';
		$params          = [ 'ajaxUrl' => admin_url( 'admin-ajax.php' ), 'nonce' => wp_create_nonce( 'gift4u_ajax_nonce' ) ];

		switch ( $screen_id ) {
			case $pages['settings']:
				$enqueue_scripts = [ 'jqColorPicker', 'select2', 'jquery.address', 'tab', 'dropdown', 'transition', 'settings' ];
				$enqueue_styles  = [ 'select2', 'segment', 'menu', 'tab', 'form', 'button', 'icon', 'checkbox', 'transition', 'dropdown', 'settings' ];
				$localize_script = 'settings';

				$params['emailTmplSamples'] = Utils::email_templates_storage();

				add_filter( 'mce_buttons_3', function ( $buttons ) {
					array_push( $buttons, 'fontselect', 'fontsizeselect', 'gift4u_sample_templates', 'gift4u_shortcode' );

					return $buttons;
				} );

				add_filter( 'mce_external_plugins', function ( $plugin_array ) {

					$plugin_array['gift4u_sample_templates'] = GiftCard::get( 'js_url' ) . 'tinymce-extra' . $this->suffix . '.js';

					return $plugin_array;
				} );

				add_editor_style( array( GiftCard::get( 'css_url' ) . 'tinymce-extra' . $this->suffix . '.css' ) );

				break;

			case $pages['product']:
				$enqueue_scripts = [ 'product-edit' ];
				$enqueue_styles  = [ 'product-edit' ];
				$localize_script = 'product-edit';

				global $post, $hook_suffix;
				$discount_rules                  = get_post_meta( $post->ID, '_gift4u_discount_rules', true );
				$params['rules']                 = $discount_rules;
				$params['discount_type_options'] = [ 'percentage' => '%', 'fixed' => esc_html__( 'Fixed', 'gift4u-gift-cards-all-in-one-for-woo' ) ];

				if ( ! empty( $_GET['gift4u_add_new'] ) && $hook_suffix == 'post-new.php' ) {
					$params['autoChangeProductType'] = true;
				}

				break;

			case $pages['gallery']:
				global $post;
				$post_id   = $post->ID;
				$image_ids = get_post_meta( $post_id, '_gift4u_image_ids', true );
				$images    = [];

				if ( ! empty( $image_ids ) ) {
					foreach ( $image_ids as $image_id ) {
						$src                 = wp_get_attachment_image_url( $image_id, 'medium' );
						$images[ $image_id ] = $src;
					}
				}

				wp_enqueue_media();
				$enqueue_scripts  = [ 'gallery' ];
				$enqueue_styles   = [ 'button', 'icon', 'segment', 'gallery' ];
				$params['images'] = $images;
				$localize_script  = 'gallery';
				break;

			case $pages['giftcard']:
				wp_enqueue_media();
				$enqueue_styles                  = [ 'select2', 'checkbox', 'form', 'form', 'segment', 'tab', 'menu', 'label', 'dropdown', 'input', 'button', 'icon', 'table', 'transition', 'toast', 'giftcard-manage' ];
				$enqueue_scripts                 = [ 'select2', 'tab', 'dropdown', 'transition', 'toast', 'giftcard-manage' ];
				$params['countries']             = array_merge( WC()->countries->get_allowed_country_states(), WC()->countries->get_shipping_country_states() );
				$params['search_products_nonce'] = wp_create_nonce( 'search-products' );
				$params['fileName']              = Data::get_param( 'pdf_file_name' );

				if ( ! empty( $_GET['action'] ) && ( $_GET['action'] == 'edit' || $_GET['action'] == 'add-new' ) ) {
					$params['editPage'] = true;
				}

				$localize_script = 'giftcard-manage';

				break;

			case $pages['card_template']:
				wp_enqueue_media();
				$enqueue_styles  = [ 'jquery-ui-helper', 'popup', 'form', 'header', 'button', 'input', 'label', 'icon', 'card-template' ];
				$enqueue_scripts = [ 'jqColorPicker', 'fabric', 'card-template' ];

				$params['fontFamilies'] = [
					[ 'value' => 'arial', 'text' => 'Arial' ],
					[ 'value' => 'tahoma', 'text' => 'Tahoma' ],
				];
				$params['fontWeight']   = [
					[ 'value' => 'normal', 'text' => 'Normal' ],
					[ 'value' => 'bold', 'text' => 'Bold' ],
				];

				$params['replaceShortcodes'] = [
					'{site_title}'         => get_bloginfo( 'name' ),
					'{site_url}'           => site_url(),
					'{coupon_code}'        => 'CODE123',
					'{coupon_value}'       => wc_price( 100 ),
					'{sender_name}'        => 'John',
					'{recipient_name}'     => 'Jane',
					'{message}'            => esc_html__( 'Just a little something to show my appreciation', 'gift4u-gift-cards-all-in-one-for-woo' ),
					'{coupon_expiry_date}' => date_i18n( wc_date_format(), current_time( 'U' ) ),
					'{apply_coupon_url}'   => add_query_arg( [ 'gift4u_coupon' => 'CODE123' ], site_url() ),
				];

				global $post;
				$post_id                = $post->ID;
				$card_template          = get_post_meta( $post_id, '_gift4u_card_template', true );
				$params['cardTemplate'] = $card_template;

				$localize_script = 'card-template';

				break;

			case $pages['shop_order']:
				$enqueue_styles = [ 'shop-order' ];

				break;

			case $pages['setup_wizard']:
				$enqueue_styles  = [ 'button', 'header', 'step', 'segment', 'setup-wizard' ];
				$enqueue_scripts = [ 'setup-wizard' ];
				$localize_script = 'setup-wizard';

				$params['createProductUrl'] = add_query_arg( [ 'post_type' => 'product', 'gift4u_add_new' => 1 ], admin_url( 'post-new.php' ) );
				break;
		}

		foreach ( $enqueue_scripts as $script ) {
			wp_enqueue_script( $this->slug . $script );
		}

		foreach ( $enqueue_styles as $style ) {
			wp_enqueue_style( $this->slug . $style );
		}

		if ( $localize_script ) {
			wp_localize_script( $this->slug . $localize_script, 'gift4uParams', $params );
		}
		if ( $inline_handle ) {
			wp_add_inline_style( $this->slug . $inline_handle, $css );
		}
	}

	public function front_enqueue_scripts() {
		wp_register_style( $this->slug . 'single-product', GiftCard::get( 'css_url' ) . 'single-product' . $this->suffix . '.css', '', GiftCard::get( 'version' ) );
		wp_register_script( $this->slug . 'single-product', GiftCard::get( 'js_url' ) . 'single-product' . $this->suffix . '.js', [ 'jquery', 'selectWoo', 'wc-country-select', 'accounting' ], GiftCard::get( 'version' ) );

		wp_register_script( $this->slug . 'giftcard-footer-script', false, '', '', true );

		switch ( true ) {
			case is_product():
				global $post;
				$pid     = $post->ID;
				$product = wc_get_product( $pid );

				/** @var \WC_Product_Giftcard $product */
				if ( $product->get_type() !== 'giftcard' ) {
					return;
				}

				wp_enqueue_style( 'select2' );

				wp_enqueue_style( $this->slug . 'single-product' );
				wp_enqueue_script( $this->slug . 'single-product' );

				$replace = [
					'{site_title}'         => get_bloginfo( 'name' ),
					'{site_url}'           => site_url(),
					'{coupon_code}'        => 'CODE123',
					'{coupon_expiry_date}' => '',
				];

				$email_content = str_replace( array_keys( $replace ), array_values( $replace ), Data::get_param( 'email_content' ) );
				$email_content = '<div id="gift4u-email-design-preview"><div class="gift4u-email-design-wrapper">'
				                 . wp_kses_post( wpautop( $email_content ) )
				                 . '</div></div>';

				wp_localize_script( $this->slug . 'single-product', 'gift4uParams', [
					'ajaxUrl'         => admin_url( 'admin-ajax.php' ),
					'nonce'           => wp_create_nonce( 'gift4u_ajax_nonce' ),
					'email_content'   => $email_content,
					'discount_umpire' => Data::get_param( 'discount_umpire' ),
					'priceType'       => $product->get_price_type(),
					'regularPrice'    => wc_price( $product->get_regular_price() ),
				] );

				break;

			case is_account_page():
				wp_register_style( $this->slug . 'my-account', GiftCard::get( 'css_url' ) . 'my-account' . $this->suffix . '.css', '', GiftCard::get( 'version' ) );
				wp_register_script( $this->slug . 'my-account', GiftCard::get( 'js_url' ) . 'my-account' . $this->suffix . '.js', [ 'jquery' ], GiftCard::get( 'version' ) );

				wp_enqueue_style( $this->slug . 'my-account' );
				wp_enqueue_script( $this->slug . 'my-account' );

				wp_localize_script( $this->slug . 'my-account', 'gift4uParams', [ 'allow_send_message' => 'Are you sure you want to send?' ] );
				break;

			case is_cart():
			case is_checkout():
				wp_register_style( $this->slug . 'cart-checkout', GiftCard::get( 'css_url' ) . 'cart-checkout' . $this->suffix . '.css', '', GiftCard::get( 'version' ) );
				wp_register_script( $this->slug . 'cart-checkout', GiftCard::get( 'js_url' ) . 'cart-checkout' . $this->suffix . '.js', [ 'jquery', 'wc-checkout' ], GiftCard::get( 'version' ) );

				wp_enqueue_style( $this->slug . 'cart-checkout' );
				wp_enqueue_script( $this->slug . 'cart-checkout' );

				$cp_bg_color   = Data::get_param( 'coupon_sticker_bg_color' );
				$cp_text_color = Data::get_param( 'coupon_sticker_text_color' );
				$css           = ".gift4u-coupons-list .gift4u-coupon-item {background-color:{$cp_bg_color}; color:{$cp_text_color};}";
				$css           .= ".gift4u-coupons-list .gift4u-coupon-item span.amount {color:{$cp_text_color};}";

				wp_add_inline_style( $this->slug . 'cart-checkout', $css );
				break;

		}

	}
}
