<?php

namespace Gift4U\Inc;

use Gift4U\GiftCard;

defined( 'ABSPATH' ) || exit;

class Setup_Wizard {
	protected static $instance = null;

	public function __construct() {
		add_action( 'admin_head', [ $this, 'setup_wizard' ] );
		add_action( 'wp_ajax_gift4u_setup_wizard', [ $this, 'setup_processing' ] );
	}

	public static function instance() {
		return self::$instance == null ? self::$instance = new self() : self::$instance;
	}

	public function setup_wizard() {
		$screen_id   = get_current_screen()->id;
		$admin_pages = Utils::get_pages();

		if ( ! isset( $admin_pages['setup_wizard'] ) || $admin_pages['setup_wizard'] !== $screen_id ) {
			return;
		}
		?>
        <div class="gift4u-setup-wizard-container">

            <div class="gift4u-setup-wizard-column">
                <h2 class="vi-ui header  center aligned ">
					<?php echo esc_html( GiftCard::get( 'plugin_name' ) ) ?>
					<?php echo esc_html__( 'Setup', 'gift4u-gift-cards-all-in-one-for-woo' ) ?>
                </h2>
                <div class="vi-ui steps">
                    <div class="step" data-step="1">
                        <i class="icon">1</i>
                        <div class="content">
                            <div class="title">Card template</div>
                            <div class="description">Generate sample card template</div>
                        </div>
                    </div>
                    <div class="step" data-step="2">
                        <i class="icon">2</i>
                        <div class="content">
                            <div class="title">Gallery</div>
                            <div class="description">Add list of images to a gallery</div>
                        </div>
                    </div>
                    <div class="step" data-step="3">
                        <i class="icon">3</i>
                        <div class="content">
                            <div class="title">Create gift card product</div>
                        </div>
                    </div>
                </div>
                <div class="gift4u-row">
                    <button class="vi-ui button primary gift4u-start-setup">
						<?php esc_html_e( 'Start', 'gift4u-gift-cards-all-in-one-for-woo' ) ?>
                    </button>
                </div>
                <div class="gift4u-row">
                    <a href="<?php echo esc_url( admin_url( 'admin.php?page=gift4u-settings' ) ) ?>">
						<?php esc_html_e( 'Not now. Go to Settings page.', 'gift4u-gift-cards-all-in-one-for-woo' ) ?>
                    </a>
                </div>
            </div>
        </div>
		<?php
		exit;
	}

	public function setup_processing() {
		check_ajax_referer( 'gift4u_ajax_nonce', 'nonce' );

		if ( ! current_user_can( 'manage_options' ) ) {
			wp_send_json_error();
		}

		if ( empty( $_POST['step'] ) ) {
			wp_send_json_error();
		}

		ini_set( 'max_execution_time', '3000' );
		ini_set( 'max_input_time', '3000' );
		ini_set( 'default_socket_timeout', '3000' );
		@set_time_limit( 0 );

		$step    = absint( $_POST['step'] );
		$results = [ 'step' => $step ];

		switch ( $step ) {
			case 1:
				$this->generate_card_template();
				break;

			case 2:
				$this->generate_gallery();
				break;

			case 3:
				$pid = $this->generate_giftcard_product();
				if ( $pid ) {
					$results['editProductUrl'] = add_query_arg( [ 'post' => $pid, 'action' => 'edit' ], admin_url( 'post.php' ) );
				}

				break;
		}

		wp_send_json_success( $results );
	}

	public function upload_image( $filename ) {
		if ( ! function_exists( 'wp_generate_attachment_metadata' ) ) {
			require_once( ABSPATH . 'wp-admin/includes/image.php' );
		}

		$image_path = GiftCard::get( 'plugin_dir' ) . '/assets/img/' . $filename;
		$image_data = file_get_contents( $image_path );
		$upload_dir = wp_upload_dir();

		if ( wp_mkdir_p( $upload_dir['path'] ) ) {
			$file = $upload_dir['path'] . '/' . $filename;
		} else {
			$file = $upload_dir['basedir'] . '/' . $filename;
		}

		file_put_contents( $file, $image_data );

		$wp_filetype = wp_check_filetype( $filename, null );

		$attachment = [
			'post_mime_type' => $wp_filetype['type'],
			'post_title'     => sanitize_file_name( $filename ),
			'post_content'   => '',
			'post_status'    => 'inherit'
		];

		$attach_id   = wp_insert_attachment( $attachment, $file );
		$attach_data = wp_generate_attachment_metadata( $attach_id, $file );
		wp_update_attachment_metadata( $attach_id, $attach_data );

		return $attach_id;
	}

	private function generate_card_template() {
		$attach_id = $this->upload_image( 'card-template.jpg' );

		$pid = wp_insert_post( [
			'post_status'   => 'publish',
			'post_type'     => 'gift4u_tmpl',
			'post_title'    => 'Gift voucher card',
			'_thumbnail_id' => $attach_id
		] );

		if ( $pid ) {
			$sample_data = (object) [
				'width'  => 109.5,
				'height' => 64,
				'margin' => (object) [ 'top' => 1, 'right' => 1, 'bottom' => 1, 'left' => 1 ],
				'images' => [
					(object) [
						'id'    => $attach_id,
						'url'   => wp_get_attachment_url( $attach_id ),
						'title' => '1403_03',
						'top'   => 0,
						'left'  => 0,
						'width' => 405.469,
						'pos_x' => 0,
						'pos_y' => 0,
					],
				],
				'texts'  => [
					(object) [
						'id'          => 1670481503814,
						'content'     => '{coupon_code}',
						'pos_x'       => 106,
						'pos_y'       => 196,
						'color'       => '#FFFBFB',
						'size'        => '18',
						'font_family' => 'arial',
						'font_weight' => 'bold',
					],
					(object) [
						'id'          => 1670481537765,
						'content'     => '{message}',
						'pos_x'       => 241,
						'pos_y'       => 162,
						'color'       => '#FFFFFF',
						'size'        => '12',
						'font_family' => 'arial',
						'font_weight' => 'normal',
						'width'       => '38',
						'align'       => 'left',
					],
					(object) [
						'id'          => 1670481594318,
						'content'     => '{site_title}',
						'pos_x'       => 12,
						'pos_y'       => 24,
						'color'       => '#FFFFFF',
						'size'        => 16,
						'font_family' => 'arial',
						'font_weight' => 'normal',
					],
					(object) [
						'id'          => 1670482294129,
						'content'     => '{site_url}',
						'pos_x'       => 12,
						'pos_y'       => 49,
						'color'       => '#F8F8F8',
						'size'        => '12',
						'font_family' => 'arial',
						'font_weight' => 'normal',
					],
					(object) [
						'id'          => 1670483162232,
						'content'     => 'Coupon code:',
						'pos_x'       => 12,
						'pos_y'       => 198,
						'color'       => '#FFFFFF',
						'size'        => '14',
						'font_family' => 'arial',
						'font_weight' => 'normal',
					],
					(object) [
						'id'          => 1670481348304,
						'content'     => '{coupon_value}',
						'pos_x'       => 16,
						'pos_y'       => 106,
						'color'       => '#FFFFFF',
						'size'        => '36',
						'font_family' => 'arial',
						'font_weight' => 'bold',
					],
				],
			];

			update_post_meta( $pid, '_gift4u_card_template', $sample_data );
		}
	}

	private function generate_gallery() {
		$images_in_gallery = [];

		$images = [
			'appreciate.png',
			'happy-birthday.png',
			'just-for-you.png',
			'love.png',
			'thank-you.png',
		];

		foreach ( $images as $image ) {
			$images_in_gallery[] = $this->upload_image( $image );
		}

		if ( ! empty( $images_in_gallery ) ) {
			$pid = wp_insert_post( [
				'post_status' => 'publish',
				'post_type'   => 'gift4u_gallery',
				'post_title'  => 'Standard',
			] );

			if ( $pid ) {
				update_post_meta( $pid, '_gift4u_image_ids', $images_in_gallery );
			}
		}
	}

	public function generate_giftcard_product() {
		$card_templates = get_posts( [ 'post_type' => 'gift4u_tmpl', 'post_status' => 'publish' ] );
		$galleries      = get_posts( [ 'post_type' => 'gift4u_gallery', 'post_status' => 'publish' ] );

		$card_tmpl_ids = wp_list_pluck( $card_templates, 'ID' );
		$gallery_ids   = wp_list_pluck( $galleries, 'ID' );

		$gc_product = new \WC_Product_Giftcard();
		$gc_product->set_name( 'Gift card' );
		$gc_product->add_meta_data( '_gift4u_card_templates', $card_tmpl_ids );
		$gc_product->add_meta_data( '_gift4u_galleries', $gallery_ids );
		$gc_product->add_meta_data( '_gift4u_sender_name', 'yes' );
		$gc_product->add_meta_data( '_gift4u_recipient_name', 'yes' );
		$gc_product->add_meta_data( '_gift4u_message_field', 'yes' );
		$gc_product->add_meta_data( '_gift4u_shipping_form', 'yes' );
		$gc_product->add_meta_data( '_gift4u_message_character_limit', 100 );

		$gc_product->save();

		return $gc_product->get_id();
	}
}
