<?php

namespace Gift4U\Inc;

use Gift4U\GiftCard;
use Gift4U\Inc\Classes\Giftcard_Coupon;

defined( 'ABSPATH' ) || exit;

class Utils {
	private static $admin_pages = [];

	public static function set_page( $key, $value = '' ) {
		if ( ! $value ) {
			$value = $key;
		}
		self::$admin_pages[ $key ] = $value;
	}

	public static function get_pages() {
		return self::$admin_pages;
	}

	public static function get_template( $name, $args = [] ) {
		wc_get_template( $name . '.php', $args, 'giftcard', GiftCard::get( 'templates_dir' ) );
	}

	public static function create_giftcard_coupon( $args = [] ) {
		$default_args = array(
			'giftcard_id'     => 0,
			'from_email'      => '',
			'to_email'        => '',
			'delivery_status' => '',
			'order_id'        => '',
			'amount'          => '',
			'delivery_date'   => '',
			'delivery_method' => '',
			'currency'        => '',
			'expiry_date'     => '',
			'meta_data'       => '',
		);

		$args = wp_parse_args( $args, $default_args );

		if ( $args['amount'] == 0 ) {
			return null;
		}

		try {
			$giftcard_coupon = new Giftcard_Coupon( $args['giftcard_id'] );
			$giftcard_coupon->set_coupon_amount( $args['amount'] );
			$giftcard_coupon->set_from_email( $args['from_email'] );
			$giftcard_coupon->set_to_email( $args['to_email'] );
			$giftcard_coupon->set_delivery_method( $args['delivery_method'] );
			$giftcard_coupon->set_expiry_date( $args['expiry_date'] );
			$giftcard_coupon->set_delivery_date( $args['delivery_date'] );
			$giftcard_coupon->set_order_id( $args['order_id'] );
			$giftcard_coupon->set_meta_data( $args['meta_data'] );

			$giftcard_coupon->save();

		} catch ( \Exception $e ) {
			return new \WP_Error( 'error', $e->getMessage() );
		}

		return $giftcard_coupon;
	}

	/**
	 * @param $id
	 *
	 * @return Giftcard_Coupon|null
	 */
	public static function get_giftcard_coupon( $id ) {
		if ( ! $id ) {
			return null;
		}

		global $wpdb;
		$db_table = GiftCard::get( 'db_table' );
		$query    = "SELECT * FROM {$db_table} where id=%d";
		$result   = $wpdb->get_row( $wpdb->prepare( $query, $id ), ARRAY_A );

		if ( ! $result ) {
			return null;
		}

		$gc_coupon = new Giftcard_Coupon();
		$gc_coupon->set_id( $result['id'] );
		$gc_coupon->set_props( $result );

		return $gc_coupon;
	}

	/**
	 * @param $coupon_code
	 *
	 * @return Giftcard_Coupon|null
	 */
	public static function get_giftcard_coupon_by_code( $coupon_code ) {
		if ( ! $coupon_code ) {
			return null;
		}

		global $wpdb;
		$db_table = GiftCard::get( 'db_table' );
		$query    = "SELECT * FROM {$db_table} where coupon_code='%s'";
		$result   = $wpdb->get_row( $wpdb->prepare( $query, $coupon_code ), ARRAY_A );

		if ( ! $result ) {
			return null;
		}

		$gc_coupon = new Giftcard_Coupon();
		$gc_coupon->set_id( $result['id'] );
		$gc_coupon->set_props( $result );

		return $gc_coupon;
	}

	public static function get_giftcard_coupons( $args ) {
		global $wpdb;
		$db_table = GiftCard::get( 'db_table' );

		$default = [
			'per_page'           => '',
			'page'               => 1,
			'sender_id'          => 0,
			'from_email'         => '',
			'to_email'           => '',
			'used'               => null,
			'delivery_status'    => '',
			'coupon_code'        => '',
			'order_id'           => '',
			'delivery_date_from' => '',
			'delivery_date_to'   => '',
			'delivery_method'    => '',
			'paginate'           => false,
			'currency'           => get_woocommerce_currency(),
		];

		$args = wp_parse_args( $args, $default );

		$where = [];

		$where_user = [];
		if ( ! empty( $args['from_email'] ) ) {
			$where_user[] = "from_email='{$args['from_email']}'";
		}

		if ( ! empty( $args['to_email'] ) ) {
			$where_user[] = "to_email='{$args['to_email']}'";
		}

		if ( ! empty( $args['sender_id'] ) ) {
			$where_user[] = "sender_id='{$args['sender_id']}'";
		}

		if ( ! empty( $where_user ) ) {
			$where[] = '(' . implode( ' OR ', $where_user ) . ')';
		}

		if ( $args['used'] === false ) {
			$where[] = "usage_count < 1";
		} elseif ( $args['used'] === true ) {
			$where[] = "usage_count > 0";
		}

		if ( ! empty( $args['delivery_status'] ) ) {
			$where[] = "delivery_status='{$args['delivery_status']}'";
		}

		if ( ! empty( $args['coupon_code'] ) ) {
			$where[] = "coupon_code='{$args['coupon_code']}'";
		}

		if ( ! empty( $args['order_id'] ) ) {
			$where[] = "order_id='{$args['order_id']}'";
		}

		if ( ! empty( $args['delivery_date_from'] ) ) {
			$where[] = "delivery_date>{$args['delivery_date_from']}";
		}

		if ( ! empty( $args['delivery_date_to'] ) ) {
			$where[] = "delivery_date<{$args['delivery_date_to']}";
		}

		if ( ! empty( $args['delivery_method'] ) ) {
			$where[] = "delivery_method='{$args['delivery_method']}'";
		}

		if ( class_exists( 'WOOMULTI_CURRENCY' ) || class_exists( 'WOOMULTI_CURRENCY_F' ) ) {
			if ( ! empty( $args['currency'] ) ) {
//				$where[] = "currency='{$args['currency']}'";
			}
		}

		$where = ! empty( $where ) ? implode( ' AND ', $where ) : "1=1";

		$limit = '';
		if ( $args['per_page'] && absint( $args['page'] ) ) {
			$offset = ( $args['page'] - 1 ) * $args['per_page'];
			$limit  = "limit {$offset}, {$args['per_page']}";
		}

		$query = "select SQL_CALC_FOUND_ROWS * from {$db_table} where {$where} {$limit}";

		$results = $wpdb->get_results( $query, ARRAY_A );

		$total = $wpdb->get_var( "SELECT FOUND_ROWS()" );

		if ( ! empty( $results ) ) {
			foreach ( $results as $key => $props ) {
				$item = new Giftcard_Coupon();
				$item->set_id( $props['id'] );
				$item->set_props( $props );
				$results[ $key ] = $item;
			}
		}

		return $args['paginate'] ?
			[
				'giftcard_coupons' => $results,
				'total'            => $total,
				'max_page'         => ceil( $total / $args['per_page'] )
			]
			: $results;
	}

	public static function delivery_methods_list() {
		return [
			'email'     => esc_html__( 'Email', 'gift4u-gift-cards-all-in-one-for-woo' ),
			'ship_card' => esc_html__( 'Ship card', 'gift4u-gift-cards-all-in-one-for-woo' ),
		];
	}

	public static function delivery_status_list() {
		return [
			'pending' => esc_html__( 'Pending', 'gift4u-gift-cards-all-in-one-for-woo' ),
			'sent'    => esc_html__( 'Sent', 'gift4u-gift-cards-all-in-one-for-woo' ),
		];
	}

	/**
	 * @param $text
	 * @param $gc_coupon Giftcard_Coupon
	 * @param array $args
	 * @param bool $strip_tags
	 *
	 * @return mixed
	 */
	public static function replace_placeholder( $text, $gc_coupon, $args = [], $strip_tags = false ) {
		if ( ! $text || ! $gc_coupon ) {
			return $text;
		}

		$meta    = $gc_coupon->get_meta_data();
		$message = '';

		if ( ! empty( $meta['message'] ) ) {
			$message = ( wptexturize( $meta['message'] ) );
		}

		$date_expiry  = $gc_coupon->get_expiry_date();
		$coupon_value = wc_price( $gc_coupon->get_amount(), [ 'currency' => $gc_coupon->get_currency() ] );

		$replace = wp_parse_args( $args, [
			'{site_title}'         => get_bloginfo( 'name' ),
			'{site_url}'           => site_url(),
			'{coupon_code}'        => strtoupper( $gc_coupon->get_coupon_code() ),
			'{coupon_value}'       => $strip_tags ? wp_strip_all_tags( $coupon_value ) : $coupon_value,
			'{sender_name}'        => ! empty( $meta['sender_name'] ) ? $meta['sender_name'] : $gc_coupon->get_from_email(),
			'{recipient_name}'     => ! empty( $meta['recipient_name'] ) ? $meta['recipient_name'] : $gc_coupon->get_to_email(),
			'{message}'            => $message,
			'{coupon_expiry_date}' => $date_expiry instanceof \WC_DateTime ? $date_expiry->date_i18n( wc_date_format() ) : '',
			'{apply_coupon_url}'   => add_query_arg( [ 'gift4u_coupon' => $gc_coupon->get_coupon_code() ], apply_filters( 'gift4u_apply_coupon_url', site_url() ) ),
		] );

		return str_replace( array_keys( $replace ), array_values( $replace ), $text );
	}

	public static function is_request_frontend() {
		return ( ! is_admin() || defined( 'DOING_AJAX' ) ) && ! defined( 'DOING_CRON' ) && ! wc()->is_rest_api_request();
	}

	public static function email_templates_storage() {
		return [
			'default' => '<table style="border-collapse:collapse;border-spacing:0;margin:0 auto;width:100%" border="0"><tbody><tr><td style="font-size:12px;font-family:Arial,Helvetica,sans-serif;color:#666;line-height:1.2em;padding:0;width:100%;background-color:#f3f3f3" bgcolor="#f3f3f3"><table class="full-width-table" style="width:500px;border-collapse:separate;border-spacing:20px;margin:0 auto" border="0"><tbody><tr><td style="padding:0;text-align:center;height:40px" valign="middle"></td></tr></tbody></table><table class="full-width-table" style="width:500px;border-radius:4px;border-collapse:collapse;border-spacing:0;margin:0 auto" border="0" bgcolor="ffffff"><tbody><tr><td style="padding:0"><table style="border-collapse:collapse;border-spacing:0;margin:0 auto" border="0"><tbody><tr><td style="line-height:0!important;padding:0;text-align:center"></td></tr><tr><td id="mainImageContainer" style="margin:0 auto;padding:0;text-align:center;width:520px" colspan="5">{selected_image}</td></tr><tr><td style="border-bottom:1px solid #eee;padding:30px"><p class="gc-message gc-break-long-words" style="font-size:20px;color:#333;line-height:32px;margin:0">{message}</p></td></tr><tr><td style="padding:0"><table style="width:100%"><tbody><tr><td style="font-family:Arial,Helvetica,sans-serif;color:#444;padding:30px;width:70%" valign="middle"><span style="font-size:24pt"><strong>{coupon_value}</strong></span>&nbsp;<br/><br/><span style="font-size:18px;font-family:Arial,Helvetica,sans-serif;color:#868686">{site_title} Gift Card</span></td><td style="padding:30px;text-align:right;width:30%" rowspan="2" valign="middle"></td></tr></tbody></table></td></tr></tbody></table></td></tr></tbody></table><table class="full-width-table" style="width:500px;border-collapse:separate;border-spacing:20px;margin:0 auto" border="0"><tbody><tr><td style="padding:0;text-align:center;height:40px" valign="middle"></td></tr></tbody></table><table style="border-collapse:collapse;border-spacing:0;margin:0 auto;width:100%;background-color:#444" border="0" bgcolor="444444"><tbody><tr><td style="padding:10px"><table class="full-width-table" style="border-collapse:collapse;border-spacing:0;margin:0 auto;width:500px" border="0"><tbody><tr><td style="padding:0;height:36px" colspan="3"></td></tr><tr><td style="padding:0;width:100px"></td><td style="font-size:14px;border:2px solid #c9c9c9;padding:0;text-align:center;width:400px"><table align="center"><tbody><tr><td style="padding:10px;text-align:center"><span style="color:#c9c9c9">Coupon Code:<strong>{coupon_code} - </strong></span><a href="{apply_coupon_url}"><span style="color:#0ff">Click to use</span></a></td></tr></tbody></table></td><td style="padding:0;width:100px"></td></tr><tr><td style="padding:0;height:36px"></td></tr></tbody></table></td></tr></tbody></table></td></tr></tbody></table>'
		];
	}
}
