<?php
defined( 'ABSPATH' ) || exit;

/** @var  $giftcards */
/** @var  $methods_list */
/** @var  $send_message */
/** @var  $send_message_status */

$user = wp_get_current_user();

if ( $send_message ) {
	wc_print_notice( $send_message, $send_message_status );
}
?>
<div id="gift4u-myaccount-coupon-table">
    <table id="gift4u-coupon-manage-list">
        <thead>
        <tr>
            <th><?php esc_html_e( 'Code', 'gift4u-gift-cards-all-in-one-for-woo' ); ?></th>
            <th><?php esc_html_e( 'Amount', 'gift4u-gift-cards-all-in-one-for-woo' ); ?></th>
            <th><?php esc_html_e( 'User', 'gift4u-gift-cards-all-in-one-for-woo' ); ?></th>
            <th><?php esc_html_e( 'Delivery', 'gift4u-gift-cards-all-in-one-for-woo' ); ?></th>
            <th><?php esc_html_e( 'Method', 'gift4u-gift-cards-all-in-one-for-woo' ); ?></th>
            <th><?php esc_html_e( 'Expiry', 'gift4u-gift-cards-all-in-one-for-woo' ); ?></th>
            <th><?php esc_html_e( 'Usage', 'gift4u-gift-cards-all-in-one-for-woo' ); ?></th>
            <th><?php esc_html_e( 'Actions', 'gift4u-gift-cards-all-in-one-for-woo' ); ?></th>
        </tr>
        </thead>
        <tbody>
		<?php
		if ( ! empty( $giftcards ) ) {
			/** @var \Gift4U\Inc\Classes\Giftcard_Coupon $giftcard */
			foreach ( $giftcards as $giftcard ) {
				$expiry_date         = $giftcard->get_expiry_date();
				$format_expiry_date  = $expiry_date instanceof \WC_DateTime ? $expiry_date->date_i18n( wc_date_format() ) : '-';
				$expiry_date_in_time = $expiry_date ? $expiry_date->getTimestamp() > time() : true;
				?>
                <tr>
                    <td data-title="<?php esc_attr_e( 'Code', 'gift4u-gift-cards-all-in-one-for-woo' ) ?>">
						<?php echo esc_html( $giftcard->get_coupon_code() ) ?>
                    </td>

                    <td data-title="<?php esc_attr_e( 'Amount', 'gift4u-gift-cards-all-in-one-for-woo' ) ?>">
						<?php echo wp_kses_post( wc_price( $giftcard->get_amount() ) ); ?>
                    </td>

                    <td data-title="<?php esc_attr_e( 'User', 'gift4u-gift-cards-all-in-one-for-woo' ) ?>">
                        <div class="gift4u-users-row">
							<?php
							if ( $user->user_email !== $giftcard->get_from_email() ) {
								printf( "%s: %s", esc_html__( 'From', 'gift4u-gift-cards-all-in-one-for-woo' ), esc_html( $giftcard->get_from_email() ) );
							} else {
								if ( $user->user_email !== $giftcard->get_to_email() ) {
									printf( "%s: %s", esc_html__( 'To', 'gift4u-gift-cards-all-in-one-for-woo' ), esc_html( $giftcard->get_to_email() ) );
								}
							}
							?>
                        </div>
                    </td>

                    <td data-title="<?php esc_attr_e( 'Delivery', 'gift4u-gift-cards-all-in-one-for-woo' ) ?>">
						<?php
						echo $giftcard->get_delivery_status() ? esc_html__( 'Sent', 'gift4u-gift-cards-all-in-one-for-woo' ) : '';
						?>
                    </td>

                    <td data-title="<?php esc_attr_e( 'Method', 'gift4u-gift-cards-all-in-one-for-woo' ) ?>">
						<?php
						$method = $methods_list[ $giftcard->get_delivery_method() ] ?? '';
						echo esc_html( $method );
						?>
                    </td>

                    <td data-title="<?php esc_attr_e( 'Expiry', 'gift4u-gift-cards-all-in-one-for-woo' ) ?>">
						<?php echo esc_html( $format_expiry_date ); ?>
                    </td>

                    <td data-title="<?php esc_attr_e( 'Usage', 'gift4u-gift-cards-all-in-one-for-woo' ) ?>">
						<?php echo $giftcard->get_usage_count() ? esc_html__( 'Used', 'gift4u-gift-cards-all-in-one-for-woo' ) : ''; ?>
                    </td>

                    <td data-title="<?php esc_attr_e( 'Actions', 'gift4u-gift-cards-all-in-one-for-woo' ) ?>">
						<?php
						if ( $giftcard->get_delivery_method() == 'email' && $user->user_email == $giftcard->get_from_email() && ! $giftcard->get_usage_count() && $expiry_date_in_time ) {
							$send_btn = $giftcard->get_delivery_status()
								? esc_html__( 'Re-send', 'gift4u-gift-cards-all-in-one-for-woo' )
								: esc_html__( 'Send now', 'gift4u-gift-cards-all-in-one-for-woo' );

							$url = add_query_arg( [ 'gift4u_send' => $giftcard->get_id(), 'gift4u_nonce' => $nonce ], remove_query_arg( [ 'gift4u_send', 'gift4u_nonce' ] ) );

							printf( '<a href="%s" class="button gift4u-send-giftcard-btn">%s</a>', esc_url( $url ), esc_html( $send_btn ) );
						}
						?>

                    </td>
                </tr>
				<?php
			}
		}
		?>

        </tbody>
    </table>
</div>
<?php
$total = isset( $total ) ? $total : wc_get_loop_prop( 'total_pages' );

if ( $total <= 1 ) {
	return;
}

$base    = wc_get_endpoint_url( 'giftcard' ) . '%#%';
$current = get_query_var( 'giftcard' ) ? (int) get_query_var( 'giftcard' ) : 1;
?>
<nav class="gift4u-pagination">
	<?php
	echo paginate_links(// phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
		apply_filters(
			'gift4u_pagination_args',
			array(
				'base'      => $base,
				'format'    => '%#%/',
				'add_args'  => false,
				'current'   => max( 1, $current ),
				'total'     => $total,
				'prev_text' => is_rtl() ? '&rarr;' : '&larr;',
				'next_text' => is_rtl() ? '&larr;' : '&rarr;',
				'type'      => 'list',
				'end_size'  => 2,
				'mid_size'  => 2,
			)
		)
	);

	?>
</nav>
